function adv = cleanup_adv(adv,flags)

% CLEANUP_ADV
%
% adv_clean = cleanup_adv(adv,flags);
%
% Cleans up ADV data, where "adv' is a data structure from load_adv_data
% and 'flags' is a structure with the following fields:
%
% If flags.use_amp is enabled (1), then marks out of water periods where
% signal amplitude is less than 50 counts. Identifies short dropouts/spikes
% in signal amplitude and interpolates or eliminates.
%
% If flags.use_snr is enabled (1), then marks out of water periods where
% the signal-to-noise ratio (SNR) is less than 20. Identifies short
% dropouts/spikes in SNR and interpolates or eliminates.
%
% If flags.use_corr is enabled (1), then identifies spikes from
% correlations below 70%.
%
% If use_phase is enabled (1), then identifies spikes using phase space methods
% implemented by Mori after Goring and Nikora.
%
% Any spikes identifed are then replaced using replace_bad_data.
%
% Finally returns 3 components (u,v,w) of cleaned-up ADV velocities in the
% adv_clean structure, plus separate in or out of water records (inuv,
% inw).


% setup
disp('      cleanup_adv starting')
if adv.downlooker
    adv.u = adv.vel(:,1);
    adv.v = adv.vel(:,2);
    adv.w = (adv.vel(:,3)+adv.vel(:,4))/2;
else
    adv.u = adv.vel(:,1);
    adv.v = adv.vel(:,2);
    adv.w = adv.vel(:,3);
end

% find in/out of water periods
if flags.use_amp || flags.use_snr
    disp('         cleanup_adv: find in/out of water')
    if flags.use_amp && flags.use_snr % in water only if both signal amplitude and signal to noise ratio (SNR) are good
        adv_amp = find_inout_using_amplitude(adv); % signal amplitude
        adv_snr = find_inout_using_snr(adv); % SNR
        adv.inuv = adv_amp.inuv & adv_snr.inuv; % u and v
        adv.inw = adv_amp.inw & adv_snr.inw; % w
    elseif flags.use_amp % in water only if signal amplitude is good
        adv = find_inout_using_amplitude(adv);
    elseif flags.use_snr % in water only if SNR is good
        adv = find_inout_using_snr(adv);
    end
else
    disp('         cleanup_adv: assuming ADV always in water')
    adv.inuv = ones(length(adv.u),1);
    adv.inw = adv.inuv;
end

% force u,v,w to 0 when out of water and track those indices
adv.u = adv.u .* adv.inuv;
adv.v = adv.v .* adv.inuv;
adv.w = adv.w .* adv.inw;
ind_out_uv = find(~adv.inuv);
ind_out_w = find(~adv.inw);

% identify bad points with correlation
if flags.use_corr
    disp('         cleanup_adv: find bad points with correlation')
    if adv.downlooker
        ind_bad_corr_uv = find(mean(adv.corr,2) < 70);
        ind_bad_corr_w = ind_bad_corr_uv;
    else
        ind_bad_corr_uv = find(mean(adv.corr(:,1:2),2) < 70);
        ind_bad_corr_w = find(mean(adv.corr(:,1:4),2) < 70);
    end
else
    ind_bad_corr_uv = [];
    ind_bad_corr_w = [];
end

% identify bad points with phase space method
if flags.use_phase
    disp('         cleanup_adv: find bad points with phase space')
    adv.u(ind_out_uv) = NaN;
    adv.v(ind_out_uv) = NaN;
    adv.w(ind_out_w) = NaN;
    [~,ind_bad_ps_u]=func_despike_phasespace3d(adv.u,0);
    [~,ind_bad_ps_v]=func_despike_phasespace3d(adv.v,0);
    [~,ind_bad_ps_w]=func_despike_phasespace3d(adv.w,0);
    adv.u(ind_out_uv) = 0;
    adv.v(ind_out_uv) = 0;
    adv.w(ind_out_w) = 0;
else
    ind_bad_ps_u = [];
    ind_bad_ps_v = [];
    ind_bad_ps_w = [];
end

% unify the bad points and exclude out-of-water periods
ind_bad_u = union(ind_bad_ps_u, ind_bad_corr_uv);
ind_bad_u = setdiff(ind_bad_u, ind_out_uv);
ind_bad_v = union(ind_bad_ps_v, ind_bad_corr_uv);
ind_bad_v = setdiff(ind_bad_v, ind_out_uv);
ind_bad_w = union(ind_bad_ps_w, ind_bad_corr_w);
ind_bad_w = setdiff(ind_bad_w, ind_out_w);

% fix bad points
disp('         cleanup_adv replacing bad data')
if ~isempty(ind_bad_u)
    adv.u = replace_bad_data(adv.u,adv.fs,ind_bad_u,adv.fs/5);
end
if ~isempty(ind_bad_v)
    adv.v = replace_bad_data(adv.v,adv.fs,ind_bad_v,adv.fs/5);
end
if ~isempty(ind_bad_w)
    adv.w = replace_bad_data(adv.w,adv.fs,ind_bad_w,adv.fs/5);
end

% smoothing
if flags.smoothing
    disp('         cleanup_adv smoothing')
    switch flags.smoothing
        case 1 % MILD
            A=5;B=3;C=11;
        case 2 % MEDIUM
            A=7;B=5;C=21;
        case 3 % HEAVY
            A=9;B=5;C=25;
        case 4 % EXTREME
            A=11;B=7;C=31;
    end
    % u
    adv.u = smoothdata(adv.u,'movmedian',A);
    adv.u = smoothdata(adv.u,'movmedian',B);
    adv.u = smoothdata(adv.u,'gaussian',C);
    % v
    adv.v = smoothdata(adv.v,'movmedian',A);
    adv.v = smoothdata(adv.v,'movmedian',B);
    adv.v = smoothdata(adv.v,'gaussian',C);
    % w
    adv.w = smoothdata(adv.w,'movmedian',A);
    adv.w = smoothdata(adv.w,'movmedian',B);
    adv.w = smoothdata(adv.w,'gaussian',C);
end

% done
disp('      cleanup_adv done')
end

function adv = find_inout_using_amplitude(adv)
% find in and out of water period using signal amplitudes
disp('         cleanup_adv: using signal amplitude')

% define inuv, inw
N = length(adv.u);
if adv.downlooker
    inuv = zeros(N,1);
    inuv(mean(adv.amp - adv.noise,2) > 10) = 1;
    inw = inuv;
else
    inuv = zeros(N,1);
    inuv(mean(adv.amp(:,1:2) - adv.noise(1:2),2) > 10) = 1;
    inw = zeros(N,1);
    inw(mean(adv.amp(:,3:4) - adv.noise(3:4),2) > 10) = 1;
end

% find dropouts
[adv,inuv,inw] = find_dropouts(adv,inuv,inw);

% find spikes
[inuv,inw] = find_spikes(adv,inuv,inw);

% finish
adv.inuv = inuv;
adv.inw = inw;
end

function adv = find_inout_using_snr(adv)
% find in and out of water period using signal to noise ratios
disp('         cleanup_adv: using signal to noise ratio')

% define inuv, inw
N = length(adv.u);
if adv.downlooker
    inuv = zeros(N,1);
    inuv(mean(adv.snr,2) > 5) = 1;
    inw = inuv;
else
    inuv = zeros(N,1);
    inuv(mean(adv.snr(:,1:2),2) > 5) = 1;
    inw = zeros(N,1);
    inw(mean(adv.snr(:,3:4),2) > 5) = 1;
end

% find dropouts
[adv,inuv,inw] = find_dropouts(adv,inuv,inw);

% find spikes
[inuv,inw] = find_spikes(adv,inuv,inw);

% finish
adv.inuv = inuv;
adv.inw = inw;
end

function [adv,inuv,inw] = find_dropouts(adv,inuv,inw)
% Find dropouts (short out of water periods 1/8sec or less)
disp('         cleanup_adv: find dropouts')
N = length(inuv);
mindropout = floor(0.125*adv.fs);
dropstarts = 1+find(diff(inuv) < 0); % inuv goes from 1 to 0
dropends = 1+find(diff(inuv) > 0); % inuv goes from 0 to 1
for drop=1:length(dropstarts)
    dropstart=dropstarts(drop);
    dropend=dropends(find(dropends>dropstart,1));
    if (~isempty(dropend)) && (dropstart ~= 1) && (dropend ~= N) && (dropend-dropstart <= mindropout)
        inuv(dropstart:dropend) = 1;
        adv.u(dropstart:dropend)=interp1([(dropstart-1) (dropend+1)],[adv.u(dropstart-1)  adv.u(dropend+1)],(dropstart:dropend),'linear');
        adv.v(dropstart:dropend)=interp1([(dropstart-1) (dropend+1)],[adv.v(dropstart-1) adv.v(dropend+1)],(dropstart:dropend),'linear');
    end
end
dropstarts = 1+find(diff(inw) < 0); % inw goes from 1 to 0
dropends = 1+find(diff(inw) > 0); % inw goes from 0 to 1
for drop=1:length(dropstarts)
    dropstart=dropstarts(drop);
    dropend=dropends(find(dropends>dropstart,1));
    if (~isempty(dropend)) && (dropstart ~= 1) && (dropend ~= N) && (dropend-dropstart <= mindropout)
        inw(dropstart:dropend) = 1;
        adv.w(dropstart:dropend)=interp1([(dropstart-1) (dropend+1)],[adv.w(dropstart-1) adv.w(dropend+1)],(dropstart:dropend),'linear');
    end
end
end

function [inuv,inw] = find_spikes(adv,inuv,inw)
% Find spikes (short in water periods less than 1/8 sec)
N = length(inuv);
minspike = floor(0.125*adv.fs);
spikestarts = 1+find(diff(inuv) > 0); % inuv goes from 0 to 1
spikeends = 1+find(diff(inuv) < 0); % inuv goes from 1 to 0
for spike=1:length(spikestarts)
    spikestart=spikestarts(spike);
    spikeend=spikeends(find(spikeends>spikestart,1));
    if (~isempty(spikeend)) && (spikestart ~= 1) && (spikeend ~= N) && (spikeend-spikestart < minspike)
        inuv(spikestart:spikeend) = 0;
    end
end
spikestarts = 1+find(diff(inw) > 0); % inw goes from 0 to 1
spikeends = 1+find(diff(inw) < 0); % inw goes from 1 to 0
for spike=1:length(spikestarts)
    spikestart=spikestarts(spike);
    spikeend=spikeends(find(spikeends>spikestart,1));
    if (~isempty(spikeend)) && (spikestart ~= 1) && (spikeend ~= N) && (spikeend-spikestart < minspike)
        inw(spikestart:spikeend) = 0;
    end
end
end