function fnew = replace_bad_data(f,fs,indices,window)

% REPLACE_BAD_DATA
%
% fnew = replace_bad_data(f,fs,indices,window);
%
% Replaces bad datapoints in 'f' identified at points 'indices' with a 
% moving median & Gaussian filter of length defined by 'window' that
% excludes the points in 'indices'. For any points that aren't handled by
% the robust fit, a -20dB Butterworth filter with stopband frequency at 
% 5Hz is used. If more than 75% of the points in 'f' are bad (as defined 
% by 'indices') then the low pass filter is used to replace all bad points.
%
% SEE ALSO: SMOOTHDATA

% setup
fnew = f;

if ~isempty(indices)
    % we do have bad points
    disp(['         replace_bad_data: starting on ',num2str(length(indices)),' bad points'])
    
    % setup smooth filter options
    A = window + ~mod(window,2); % A (ensured odd) for gaussian window
    B = round(window/2); B = B + ~mod(B,2); % B (ensured odd) for median window

    % setup low pass filtered option
    fmax = 2.5; % maximum frequency to pass
    fN = fs/2; % Nyquist frequency
    [n_filt,Wn_filt] = buttord(fmax/fN,fmax*2/fN,0.01,20);
    [z_filt,p_filt,k_filt] = butter(n_filt,Wn_filt,'low');
    [SOS_filt,G_filt] = zp2sos(z_filt,p_filt,k_filt);
    if length(f) <= 3*(filtord(SOS_filt)) % if f is too short
        f_lowpass = f;
    else
        f_lowpass = filtfilt(SOS_filt,G_filt,f);
    end
    
    if length(indices) >= 3*length(f)/4
        % use lowpass when more than 75% bad points
        fnew(indices) = f_lowpass(indices); 
    else
        % smooth
        f(indices) = NaN;
        fsmooth = (smoothdata(f,'movmedian',B) + smoothdata(f,'gaussian',A))/2;
        fnew(indices) = fsmooth(indices); % assign replaced bad points to fnew
        if any(isnan(fnew)) % we still have bad points
            fnew(isnan(fnew)) = f_lowpass(isnan(fnew)); % use low-passed points for the rest of the bad points
        end
    end   
else
    % no bad points!
    disp('         replace_bad_data: no bad points')
end

% done
end