function channel_cleaned = cleanup_uswg(channel)

% CLEANUP_USWG
%
% channel_cleaned = cleanup_uswg(channel);
%
% Cleans up ultrasonic range finder (ultrasonic wave gage) data collected
% at HWRL.
%
% Identifies bad data using gradient and absolute +/- 5V thresholds.
% Adds 2 samples before and 4 samples after each bad point.  
%
% Calls replace_bad_data for the replacement of bad points.
%
% Cleaned up data is median filtered and then rloess filtered using
% the 'smoothdata' function.
%
% SEE ALSO: PROCESS_RAW_TRIAL, REPLACE_BAD_DATA, SMOOTHDATA

% setup
channel_cleaned = channel; % data channel structure
x = channel.analog; % analog data
fs = channel.fs; % sample rate
window = floor(fs/4); % filter window
window = window + ~mod(window,2); % ensure that window is odd

% identify bad points with a difference threshold
spike_indices = find(abs(diff(x))>0.5); % threshold of 0.5 volts in 1 sample

% identify other bad points outside 5V
high_indices = find(x>5); % threshold of +5V
low_indices = find(x<-5); % threshold of -5V

% unify bad points and spread out around them a bit
bad_indices = union(spike_indices,union(high_indices,low_indices));
indices = union(bad_indices,union(bad_indices - 1,bad_indices + 1)); % flag 1 sample before and after each bad sample
indices = union(indices,union(indices - 1,indices + 1)); % repeat, so we get 2 samples before and after each bad sample
indices = union(indices,union(indices + 1,indices + 2)); % again to get an additional 2 samples after each bad sample
indices = indices(indices>=1 & indices<=length(x));

% replace bad points, if any
if ~isempty(indices)
    y = replace_bad_data(x,fs,indices,window);
else
    y = x;
end

% smooth
A = max(21,round(window/2)+~mod(round(window/2),2));
B = max(5,round(A/4)+~mod(round(A/4),2));
C = max(3,round(B/2)+~mod(round(B/2),2));
y = smoothdata(y,'movmedian',B);
y = smoothdata(y,'movmedian',C);
y = smoothdata(y,'gaussian',A);

% finish up
channel_cleaned.analog = y;

end
