function peakdata = find_peaks(data,signals,definers,Npeaks)

% find_peaks
%
% peakdata = find_peaks(data,signals,definers,Npeaks);
%
% Produces a number (specified by 'Npeaks') ofpeaks  of listed channels in
% 'signals' from the input structure 'data'. Uses the channels listed in
% 'definers' to estimate the period in which to find the maximum (a peak)
% for each of 'signals' and returns that in the structure 'peakdata'.

% find a wave period to filter around
% we assume these signals are wave gauges
N = length(definers);
T = zeros(1,N); % wave periods
for n=1:N
    channelname = definers{n};
    if isfield(data,channelname)
        time = data.(channelname).time; % time
        eta = data.(channelname).analog; % free surface
        x = data.(channelname).x; % cross-shore location of this channel
        h = data.h; % still water depth of this data
        fs = data.(channelname).fs; % sampling rate
        fmin = 0.05; % minimum frequency to pass
        fmax = 2.5; % maximum frequency to pass
        eta = peaks_bandpass(fs,eta,fmin,fmax); % mild bandpass to remove high frequency noise and low frequency seiche
        wrr = data.(channelname).waves_running_range; % waves running indices
        skip = ceil(20*fs); % skip 20s of ramp between start and end
        wrr = wrr(skip:(end-skip)); % crop waves running indices
        time = time(wrr); % time cropped
        eta = eta(wrr); % free surface cropped
        [~,~,T_z,t] = find_waves(time,eta); % zero upcrossings wave periods
        if isfinite(x)
            [~,cg] = phase_speed(h,mean(T_z)); % cg of mean wave period of entire record
            delay = x/cg;
            T_z = T_z(find(t>delay,1,'first'):end); % crop off time prior to waves arriving at channel
        end
        T(n) = mean(T_z); % average the waves to get a wave period estimate
    end
end
T_f = mean(T); % filtering wave period

% now analyze each signal
M = length(signals);
for m=1:M
    channelname = signals{m};
    if isfield(data,channelname)
        % setup
        time = data.(channelname).time; % time
        signal = data.(channelname).analog; % signal
        x = data.(channelname).x; % cross-shore location of this channel
        h = data.h; % still water depth of this channel
        fs = data.(channelname).fs; % sampling rate
        dt = 1./fs; % timestep

        % smooth and bandpass
        window = floor(fs/4); % smoothing window
        window = window + ~mod(window,2); % ensure that window is odd
        A = window; B = round(A/2)+~mod(round(A/2),2); C = round(B/2) + ~mod(round(B/2),2);
        s_s = smoothdata(smoothdata(smoothdata(signal,'movmedian',B),'movmedian',C),'gaussian',A);
        fmin = T_f/sqrt(10); % minimum frequency to pass
        fmax = T_f*sqrt(10); % maximum frequency to pass
        s_f = peaks_bandpass(fs,s_s,fmin,fmax); % bandpass around T_f with a width of 1 order of magnitude
        dsdt = gradient(s_f,dt); % signal gradient

        % choose timeframe to analyze
        wrr = data.(channelname).waves_running_range; % waves running indices
        start_skip = ceil(20*fs); % skip 20s of ramp at start
        end_skip = 2*start_skip; % double skip at end for ramp plus separate wavemaker winddown
        wrr = wrr(start_skip:(end-end_skip)); % crop waves running indices (double end subtraction to account for rundown of wavemaker)
        if isfinite(x)
            [~,cg] = phase_speed(h,T_f); % cg of T_e ensembling wave period
            start = ceil(x/cg)*fs; % samples to skip at start for waves not ariving at channel
            wrr = wrr(start:end); % crop waves running indices
        end
        s_crop = signal(wrr); t_f = time(wrr); dsdt = dsdt(wrr); % crop

        % find peaks
        zdown = find((dsdt(1:end-1).*dsdt(2:end) <=0) & (dsdt(1:end-1) > dsdt(2:end))); % zero downcrossings of slope
        t_zd = t_f(zdown); % corresponding downcrossing times
        N = length(zdown); % number of zero downcrossings
        km = zdown(round(N/2)); % middle downcrossing
        tm = t_f(km); % time of middle downcrossing
        T1 = tm-T_f*ceil(Npeaks/2); T2 = tm+T_f*ceil(Npeaks/2); % start and end times to find highest peaks
        [~,k1] = min(abs(t_zd-T1)); [~,k2] = min(abs(t_zd-T2)); K=k1:k2; % corresponding start and end downcrossings
        j_peaks = zeros(1,Npeaks); peakvalues = zeros(1,Npeaks);  peaktimes = zeros(1,Npeaks); n_peak = 1;
        for k=K(1):K(end)
            T1 = t_zd(k)-T_f/2; T2 = t_zd(k)+T_f/2; % start and end times 1/2 wave period around this downcrossing
            [~,j1] = min(abs(t_f-T1)); [~,j2] = min(abs(t_f-T2)); J=j1:j2; % corresponding indices
            [~,I] = max(s_crop(J)); % maximum signal in this period
            s_p = s_crop(J(I)); % value of this peak
            t_p = t_f(J(I)); % timestamp of this peak
            if ~any(j_peaks == J(I)) % if we haven't seen this peak
                if ~any(abs(peaktimes(peaktimes~=0) - t_p)<=T_f/4) % if there isn't a nearby peak we've already got
                    j_peaks(n_peak) = J(I); % record the index of this peak
                    peakvalues(n_peak) = s_p; % record the value of this peak
                    peaktimes(n_peak) = t_p; % record the timestamp of this peak
                    n_peak = n_peak+1;
                end
            end
        end
        
        % sometimes the above will slightly miss a peak due to overlap
        % adjust each peak to a nearby local maximum
        for n=1:Npeaks
            pval = peakvalues(n); % peak value
            pt = peaktimes(n); % peak time
            T1 = pt-T_f/4; T2 = pt+T_f/2; % start and end times 1/4 wave period around the peak time
            [~,j1] = min(abs(t_f-T1)); [~,j2] = min(abs(t_f-T2)); J=j1:j2; % corresponding indices
            if any(s_crop(J)>pval) % is there a higher value near this peak?
                [pval,jp] = max(s_crop(J)); % higher value
                pt = t_f(J(jp)); % associated timestamp
                peakvalues(n) = pval;
                peaktimes(n) = pt;
            end
        end

        % done with this channel
        peakdata.(channelname).values = peakvalues;
        peakdata.(channelname).times = peaktimes;
    end
end

% done
end

function signal_filt = peaks_bandpass(fs,signal,fmin,fmax)
% mild butterworth bandpass filter

% set up
fN = fs/2; % Nyquist frequency
[n_filt,Wn_filt] = buttord([fmin fmax]/fN,[fmin/2 fmax*2]/fN,0.01,20);
[z_filt,p_filt,k_filt] = butter(n_filt,Wn_filt,'bandpass');
[SOS_filt,G_filt] = zp2sos(z_filt,p_filt,k_filt);

% pad data
M = length(signal); % original length
N = 2^nextpow2(3*M); % padded length
signal_pad = padarray(padarray(signal,floor((N-M)/2),'symmetric','pre'),ceil((N-M)/2),'symmetric','post');

% apply bandpass filter
signal_filt = filtfilt(SOS_filt,G_filt,signal_pad); % filter
j = 1+((round(N/2) - round(M/2)):(round(N/2) - round(M/2) + M - 1));
signal_filt = signal_filt(j); % unpad
end
