function [N,H,T,t] = find_waves(time,eta)

% FIND_WAVES
%
% [N,H,T,t] = find_waves(time,eta);
%
% Zero upcrossing analysis to find waves in the provided signal 'eta' (m)
% as a function of 'time' (s).  Returns number of waves found 'N',
% and each wave height 'H' (m), period 'T' (s), and time 't' (s).
%
% SEE ALSO: WAVE_STATISTICS

% setup indices
indices=1:(length(eta)-1);

% find zero upcrossings
zup = find((eta(indices).*eta(indices+1) <=0) & (eta(indices) < eta(indices+1)));
N = length(zup) - 1;

% setup H,T,t
H = zeros(N,1);
T = zeros(N,1);
t = zeros(N,1);

for n=1:N % for each zero upcrossing
    % H is crest-trough distance between this zero upcrossing and the next
    H(n) = max(eta(zup(n):zup(n+1))) - min(eta(zup(n):zup(n+1)));
    
    % interpolate to estimate zero upcrossing times
    if n == 1
        % estimate this zero upcrossing time
        j2 = zup(n)+1; j1 = zup(n);
        m = (eta(j2)-eta(j1))/(time(j2)-time(j1)); % slope
        b = (eta(j2)+eta(j1) - m*(time(j2)+time(j1)))/2; % y-offset
        t1 = -b/m; % estimate this zero upcrossing time
    else
        t1 = t2; % we estimated this one on the previous iteration
    end
    j2 = zup(n+1)+1; j1 = zup(n+1);
    m = (eta(j2)-eta(j1))/(time(j2)-time(j1)); % slope
    b = (eta(j2)+eta(j1) - m*(time(j2)+time(j1)))/2; % y-offset
    t2 = -b/m; % estimate next zero upcrossing time
        
    % T is the time in seconds between t1 and t2
    T(n) = (t2 - t1);
    
    % t is average timestamp of t1 and t2
    t(n) = 0.5*(t1 + t2);
end

% done
end