function [wg_i,wg_r] = inc_ref_array(x,h,wg,t)

% INC_REF_ARRAY
%
% [wg_i,wg_r] = inc_ref_array(x,h,wg,t);
%
% Function to separate incident and reflected waves using an array of wave
% gages at different cross-shore locations and depths.  Returns estimated
% incident (wg_i) and reflected (wg_r) waves at the most offshore location
% of each possible pairing in the array.
%
% x = cross-shore location (m) of wave gage observations 
% h = water depth (m) of wave gage observations
% wg = wave gage observations of free surface (m) 
% t = time (s)
%
% Calls inc_wave_freq for the core routine in frequency space.

disp('inc_ref_array: starting');
% set up variables
fs = 1/mean(diff(t));
T = length(t);
nfft=2^nextpow2(T);

% process wave gage pairings
n = length(x); % number of wave gages
wg_i = zeros(T,n-1); % incident wave estimates
wg_r = zeros(T,n-1); % reflected wave estimates
for i=1:n-1 % first wave gage in each pair
    % initialize arrays
    inc_zetaf = zeros(1,nfft);
    count = zeros(1,nfft);
    
    % setup inputs for first wave gage
    eta1 = padarray(padarray(wg(:,i),floor((nfft-T)/2),'symmetric','pre'),ceil((nfft-T)/2),'symmetric','post');
    x1 = x(i);
    h1 = h(i);
    
    for j=i+1:n % second wave gage in each pair
        % setup inputs for second wave gage
        eta2 = padarray(padarray(wg(:,j),floor((nfft-T)/2),'symmetric','pre'),ceil((nfft-T)/2),'symmetric','post');
        %[wg(floor((nfft-T)/2):-1:1,i);wg(:,i);wg(T:-1:(T-floor((nfft-T)/2)),i)]
        x2 = x(j);
        h2 = h(j);
                
        % find incident waves in frequency space
        disp(['inc_ref_array: calling inc_wave_freq for x=',num2str(x1),' and x=',num2str(x2)]);
        [~,temp,ok_f] = inc_wave_freq(x1,x2,h1,h2,eta1,eta2,fs,nfft);
        
        % sum the results
        inc_zetaf = inc_zetaf + temp;
        count = count + [ok_f(1:end),ok_f(end-1:-1:2)];
    end % second wave gage in each pair

    % divide sum by count to get average of incident waves
    inc_zetaf(count ~= 0) = inc_zetaf(count ~= 0) ./ count(count ~= 0);
    
    % handle any singularities
    inc_zetaf(count == 0) = 0;

    % inverse fft, and take real part to get incident wave in time domain
    wg_i_pad = real(ifft(inc_zetaf))';
    
    % reflected wave is first wave gage reading minus incident
    eta = padarray(padarray(wg(:,i),floor((nfft-T)/2),'symmetric','pre'),ceil((nfft-T)/2),'symmetric','post');
    wg_r_pad = eta - wg_i_pad;

    % unpad
    indices = 1+((round(length(wg_i_pad)/2) - round(T/2)):(round(length(wg_i_pad)/2) - round(T/2) + T - 1));
    wg_i(:,i) = wg_i_pad(indices);
    wg_r(:,i) = wg_r_pad(indices);
end % wave gage pair loop

% done
disp('inc_ref_array: done');
return