function [f,inc_zeta1_f,ok_f] = inc_wave_freq(x1,x2,h1,h2,eta1,eta2,fs,nfft);

% INC_WAVE_FREQ
%
% [f,inc_zeta1_f,ok_f] = inc_ref_separation(x1,x2,h1,h2,eta1,eta2,fs,nfft);
%
% Function to separate incident and reflected waves in the frequency domain.  
%
% x1,x2 = location of 1st and 2nd wave gages. 2nd gage onshore of first (x2 > x1)
% h1,h2 = still water depth at 1st and 2nd wave gages
% eta1,eta2 = free surface time series at 1st and 2nd wave gages
% fs = sampling frequency
% nfft = FFT length (2^n)
%
% Rewritten by Tim Maddux from Tom Baldock's Ref_slope.m to be 2.3x faster
%
% References:
% Baldock, T. E. and Simmonds, D., 1999. Separation of incident and reflected 
% waves over sloping bathymetry. Coastal Engineering, 38, 167-176.
% 
% Goda, 2010, Random Seas and Design of Maritime Structures, 3rd Ed., World
% Scientific Advanced Series on Ocean Engineering, Vol. 33, p.474.

% setup
disp('inc_wave_freq: starting')
df = fs/nfft; % frequency resolution
fmax = fs/2; % Nyquist frequency
f = 0:df:fmax; % define frequency space
Nf = nfft/2 + 1; % number of frequencies

% discretize the bathymetry
dx = x2 - x1; % distance between x1 and x2
N = 100; % steps between x1 and x2
x = linspace(x1,x2,N); % cross-shore range from x1 to x2
h = linspace(h1,h2,N); % depth range from x1 to x2

% phase speed and group velocity
[c,cg] = phase_speed(h,1./f);

% shoaling coefficient
S = sqrt(cg(1,:) ./ cg(N,:)); 

% travel time and phase shift
t = sum((dx/(N-1))./(0.5*c(1:N-1,:)+0.5*c(2:N,:)),1); % travel time
kdx = 2*pi*f.*t; % physical phase shift, eqn 8 of Baldock 1999
psi1 = kdx + pi/2; % first theoretical phase shift, eqn 5 of Baldock 1999
psi2 = -pi/2; % 2nd theoretical phase shift, eqn 6 of Baldock 1999

% amplification factor C
C = 0.5./cos(-kdx - pi/2); % eqn 7 of Baldock 1999

% effective frequency range per Goda eqns 11.113, p. 474
ok_f = kdx >= pi/10; % wavenumbers above lower limit
ok_f = ok_f & kdx <= 9*pi/10; % wavenumbers below upper limit
ok_f(1) = 0;
C(~ok_f) = 0; % force C outside those values to 0
C(1) = 0;

% crop eta1,eta2 to nfft length, remove mean, FFT
f1 = fft(eta1(1:nfft)' - mean(eta1(1:nfft)),nfft);
f2 = fft(eta2(1:nfft)' - mean(eta2(1:nfft)),nfft);

% apply gain C and phase shift psi1 in frequency domain to f1
y1 = [0, f1(2:Nf-1).*C(2:Nf-1).*exp(1i*psi1(2:Nf-1)), 2*f1(Nf)*C(Nf)*cos(psi1(Nf)), f1(Nf+1:1:nfft).*C(Nf-1:-1:2).*exp(-1i*psi1(Nf-1:-1:2))];

% apply shoaling coefficient S, gain C and phase shift psi2 to f2
y2 = [0, f2(2:Nf-1).*C(2:Nf-1)*exp(1i*psi2)./S(2:Nf-1), 2*f2(Nf)*C(Nf)*cos(psi2)/S(Nf), f2(Nf+1:1:nfft).*C(Nf-1:-1:2)*exp(-1i*psi2)./S(Nf-1:-1:2)];

% sum to get incident wave at x1
inc_zeta1_f = y1+y2;

% done
disp('inc_wave_freq: done')
return