function stats = wave_statistics(h,eta,time)

% WAVE_STATISTICS
%
% stats = wave_statistics(h,eta,time);
%
% Calculates statistics and moments of input free surface elevation 'eta'
% (m) in water depth 'h' (m) as a function of 'time' (s).
%
% The structure 'stats' has subfields as follows:
% stats.N is the number of waves counted
% stats.eta fields: .mean, .std, .skew, .asym.
% stats.H fields: .s, .m0, .bar, .std, .rms, .ten, .two, .median, .min, and .max.
% stats.T fields: .s, .p, .bar, .ten, .two, .median, and .e.
% stats.E is the energy.
% stats.Ef is the energy flux.
% stats.psd fields: .f, .S, .Sc, .fs, .nfft, .noverlap, .K

% setup
N = length(time); % number of data points
dt = mean(diff(time)); % dt in seconds
fs = 1/dt; % sample frequency
p = nextpow2(fs*30); % exponent to get at least a 30-second nfft
if 2^p > length(eta) % nfft is too long
    done = 0;
    while ~done % reduce by factors of 2 until nfft is shorter than eta
        p = p-1;
        done = (2^p <= length(eta));
    end
end
nfft = 2^p; % nfft for pwelch
overlap = 0.5; % 50% overlap
noverlap = nfft*overlap; % number of overlap points
stats.psd.fs = fs;
stats.psd.nfft = nfft;
stats.psd.noverlap = noverlap;

% free surface statistics
stats.eta.mean = mean(eta);
stats.eta.std = sqrt(var(eta - stats.eta.mean));
stats.eta.skew = skewness(eta - stats.eta.mean);
stats.eta.asym = -skewness(imag(hilbert(eta - stats.eta.mean)));

% zero upcrossings to find wave heights, periods, times
[stats.N,H,T,~] = find_waves(time,eta - stats.eta.mean);

% calculate mean wave height and period
stats.H.bar = mean(H);
stats.T.bar = mean(T);

% calculate median wave height and period
stats.H.median = median(H);
stats.T.median = median(T);

% calculate min,max,std and rms wave heights
stats.H.min = min(H);
stats.H.max = max(H);
stats.H.std = std(H);
stats.H.rms = sqrt(stats.H.std^2 + stats.H.bar^2);

% rank by wave height
rankedwaves = sort([H,T],1,'descend');

% calculate significant wave height and period
if round(stats.N/3)>=1
    stats.H.s = mean(rankedwaves(1:round(stats.N/3),1)); % significant height H(1/3)
    stats.T.s = mean(rankedwaves(1:round(stats.N/3),2)); % significant period T(1/3)
else
    stats.H.s = NaN;
    stats.T.s = NaN;
end
if round(stats.N/10)>=1
    stats.H.ten = mean(rankedwaves(1:round(stats.N/10),1)); % 10% height H(1/10)
    stats.T.ten = mean(rankedwaves(1:round(stats.N/10),2)); % 10% period T(1/10)
else
    stats.H.ten = NaN;
    stats.T.ten = NaN;
end
if round(stats.N/50)>=1
    stats.H.two = mean(rankedwaves(1:round(stats.N/50),1)); % 2% height H(1/50)
    stats.T.two = mean(rankedwaves(1:round(stats.N/50),2)); % 2% period T(1/50)
else
    stats.H.two = NaN;
    stats.T.two = NaN;
end

% calculate spectral wave height Hm0
m0 = var(eta - stats.eta.mean);
stats.H.m0 = 4.004*sqrt(m0);

% energy
rho = density_of_water; % water density
g = gravity; % gravity
stats.E = rho*g*m0; % energy

% estimate Welch spectrum
if nfft > 4
    [S,w,S_c] = pwelch(eta - stats.eta.mean,nfft,nfft*overlap,nfft,'ConfidenceLevel',0.95); % Welch spectrum
    f = w*fs/2/pi; % unnormalized frequencies
    S = S(2:end); f=f(2:end); % throw out 0 bin
    stats.psd.f = f;
    scale_factor = m0 / trapz(f,S); % ratio of m0 to integrated psd
    S = S * scale_factor; % rescale to force integral to match
    stats.psd.S = S;
    S_c = S_c * scale_factor; % apply same rescaling as for S_welch
    stats.psd.Sc = S_c;

    % moments, energy period, peak period from spectrum
    m_1 = trapz(f,S./f); % m(-1)
    [~,index] = max(S);
    fp = f(index); % peak frequency
    stats.T.e = m_1/m0; % energy period Te = m(-1)/m0
    stats.T.p = 1/fp; % peak period Tp = 1/fp

    % see welchparse.m for this equation
    stats.psd.K = floor((N - noverlap)/(nfft - noverlap)); % number of overlap segments in pwelch

    % energy flux
    [~,cg] = phase_speed(h,1./f');
    Ef = rho*g*trapz(f,cg'.*S);
    stats.Ef = Ef;
end

% done
end
