function data_calibrated = calibrate_raw_data(data_raw,calibrations,interp_flag)

% CALIBRATE_RAW_DATA
%
% data_calibrated = calibrate_raw_data(data_raw,calibrations);
% data_calibrated = calibrate_raw_data(data_raw,calibrations,interp_flag);
%
% Calibrates the raw data in 'data_raw' with the coefficients in 'calibrations',
% and returns the calibrated data in 'data_calibrated'.
%
% Values of 'interp_flag' are 'ON','OFF','WG_AND_USWG', and 'WG_ONLY' (default).
% When interpolation is enabled, calibrations are linearly interpolated between
% calibrations that bracket the timestamp of the data. When disabled, the most 
% recent calibration is used.
%
% SEE ALSO: LOAD_CALIBRATIONS, PROCESS_RAW_TRIAL

% setup
disp('      calibrate_raw_data: starting')
if nargin < 3
    interp_flag = 'WG_ONLY'; % default
end
switch interp_flag
    case {'ALL','NONE'}
        disp(['      calibrate_raw_data: interp_flag ',interp_flag])
    case 'WG_ONLY'
        disp(['      calibrate_raw_data: interp_flag ',interp_flag])
        wgchannels = find_wgs(data_raw); % find wgs for interpolation
    case 'WG_AND_USWG'
        disp(['      calibrate_raw_data: interp_flag ',interp_flag])
        wgchannels = union(find_wgs(data_raw),find_uswgs(data_raw)); % find wgs and uswgs for interpolation
    otherwise
        disp(['      calibrate_raw_data: interp_flag ',interp_flag,' unrecognized and calibration aborted!'])
        data_calibrated = NaN;
        return
end
data_calibrated = data_raw;

% calibrate every analog channel
if isfield(data_raw,'channelnames')
    Nc = length(data_raw.channelnames); % number of channels
    for nc = 1:Nc
        channelname = data_raw.channelnames{nc}; % channel name
        if isfield(data_raw,channelname) % we have data
            channel_timestamp = data_raw.(channelname).startdatenum + mean(data_raw.(channelname).time)/86400;
            if isfield(calibrations,channelname) % we have a calibration
                % find applicable calibration slope, offset, and units
                if channel_timestamp >= min(calibrations.(channelname).time) && channel_timestamp <= max(calibrations.(channelname).time)
                    % we have calibrations that bracket the trial timestamp
                    [~,cal_index] = find(calibrations.(channelname).time <= channel_timestamp,1,'last'); % most recent cal index
                    switch interp_flag
                        case 'ALL' % interpolate all channels
                            data_calibrated.(channelname).calslope = interp1(calibrations.(channelname).time,calibrations.(channelname).slope,channel_timestamp); % linearly interpolate
                            data_calibrated.(channelname).caloffset = interp1(calibrations.(channelname).time,calibrations.(channelname).offset,channel_timestamp); % linearly interpolate
                            data_calibrated.(channelname).calunits = calibrations.(channelname).units{cal_index};
                        case 'NONE' % use most recent calibration for all channels
                            data_calibrated.(channelname).calslope = calibrations.(channelname).slope(cal_index);
                            data_calibrated.(channelname).caloffset = calibrations.(channelname).offset(cal_index);
                            data_calibrated.(channelname).calunits = calibrations.(channelname).units{cal_index};
                        case {'WG_ONLY','WG_AND_USWG'}
                            if any(wgchannels == nc) % if this is a wave gage
                                % interpolate
                                data_calibrated.(channelname).calslope = interp1(calibrations.(channelname).time,calibrations.(channelname).slope,channel_timestamp); % linearly interpolate
                                data_calibrated.(channelname).caloffset = interp1(calibrations.(channelname).time,calibrations.(channelname).offset,channel_timestamp); % linearly interpolate
                                data_calibrated.(channelname).calunits = calibrations.(channelname).units{cal_index};
                            else
                                % use most recent calibration
                                data_calibrated.(channelname).calslope = calibrations.(channelname).slope(cal_index);
                                data_calibrated.(channelname).caloffset = calibrations.(channelname).offset(cal_index);
                                data_calibrated.(channelname).calunits = calibrations.(channelname).units{cal_index};
                            end
                        otherwise
                            error(['      calibrate_raw_data: interp_flag ',interp_flag,' unrecognized!']);
                    end
                elseif channel_timestamp > max(calibrations.(channelname).time)
                    % timestamp is after the last calibration
                    data_calibrated.(channelname).calslope = calibrations.(channelname).slope(end);
                    data_calibrated.(channelname).caloffset = calibrations.(channelname).offset(end);
                    data_calibrated.(channelname).calunits = calibrations.(channelname).units{end};
                else
                    % timestamp is before the first calibration
                    data_calibrated.(channelname).calslope = calibrations.(channelname).slope(1);
                    data_calibrated.(channelname).caloffset = calibrations.(channelname).offset(1);
                    data_calibrated.(channelname).calunits = calibrations.(channelname).units{1};
                end
                
                % apply the slope and offset
                data_calibrated.(channelname).analog = data_raw.(channelname).analog * data_calibrated.(channelname).calslope + data_calibrated.(channelname).caloffset;
            else
                % we don't have a calibration for this channel
                data_calibrated.(channelname).calslope = NaN;
                data_calibrated.(channelname).caloffset = NaN;
                data_calibrated.(channelname).calunits = 'None';
                data_calibrated.(channelname).analog = data_raw.(channelname).analog;
            end
        end
    end
end

% done
disp('      calibrate_raw_data: done')
end