function data_checked = check_adv_temperatures(data,default_temperature)

% CHECK_ADV_TEMPERATURES
%
% data_checked = check_adv_temperatures(data,default_temperature);
%
% Checks ADV temperatures for values outside 5C to 23C.  Also checks
% for ADVs with temperatures more than 5C outside the mean of all valid
% ADVs.  Adjusts temperature and speed of sound readings accordingly on
% ADVs that it finds fault with.  Returns adjusted/checked data in
% data_checked structure.
%
% If no good temperatures are found then the default_temperature (C) is
% used.
%
% SEE ALSO: SPEED_OF_SOUND, PROCESS_RAW_TRIAL

% setup
lower_limit = 5; % our lowest recorded water temperature since 2008 is 6C
upper_limit = 23; % our highest recorded water temperature since 2008 is 22C
data_checked = data;
if isfield(data,'advnames') && ~isempty(data.advnames)
    Nadv = length(data.advnames);
    temp_ok = zeros(Nadv,1);
    temp_mean = 0;
    
    % find the mean of all good adv temperature readings
    for nadv = 1:Nadv
        advname = data.advnames{nadv};
        if data.(advname).temperature < lower_limit || data.(advname).temperature > upper_limit
            disp(['      check_adv_temperatures: invalid ',advname,' temperature of ',num2str(data.(advname).temperature),' C'])
            temp_ok(nadv) = 0;
        else
            temp_ok(nadv) = 1;
            temp_mean = temp_mean + data.(advname).temperature;
        end
    end % end invalid temperature check
    if ~any(temp_ok)
        % no good temperatures; use the default
        temp_mean = default_temperature;
    else
        % use good adv temperatures to fix bad ones
        temp_mean = temp_mean / sum(temp_ok);
    end
    
    % correct the bad temperature readings and speeds of sound
    for nadv = 1:Nadv
        advname = data.advnames{nadv};
        if ~temp_ok(nadv)
            % invalid temperature, give it the mean value
            disp(['      check_adv_temperatures: invalid ',advname,' temperature adjusted to ',num2str(temp_mean),' C'])
            c_old = data.(advname).c_sound;
            c_new = speed_of_sound(temp_mean);
            disp(['      check_adv_temperatures: invalid ',advname,' sound speed adjusted from ',num2str(c_old),' m/s to ',num2str(c_new),' m/s'])
            data_checked.(advname).temperature = temp_mean;
            data_checked.(advname).c_sound = c_new;
            data_checked.(advname).vel = data.(advname).vel * c_new / c_old;
            data_checked.(advname).temp_ok = 'NO';
        else
            if abs(data.(advname).temperature - temp_mean) > 5
                % outlier temperature, give it the mean value
                disp(['      check_adv_temperatures: outlier ',advname,' temperature of ',num2str(data.(advname).temperature),' C'])
                disp(['      check_adv_temperatures: outlier ',advname,' temperature adjusted to ',num2str(temp_mean),' C'])
                c_old = data.(advname).c_sound;
                c_new = speed_of_sound(temp_mean);
                disp(['      check_adv_temperatures: outlier ',advname,' sound speed adjusted from ',num2str(c_old),' m/s to ',num2str(c_new),' m/s'])
                data_checked.(advname).temperature = temp_mean;
                data_checked.(advname).c_sound = c_new;
                data_checked.(advname).vel = data.(advname).vel * c_new / c_old;
                data_checked.(advname).temp_ok = 'NO';
                temp_ok(nadv) = 0;
            else % neither an outlier nor invalid
                data_checked.(advname).temp_ok = 'YES';
            end % end outlier temperature check and adjust
        end % end invalid temperature adjust
    end
end

% done
disp('   check_adv_temperatures: done')
end