function data = load_adv_data(filename)

% LOAD_ADV_DATA
%
% data = load_adv_data(filename);
%
% Loads data from the Vectrino (ADV) record in the specified file 'filename'.
% Data is returned in the 'data' structure with the following fields:
%
% vel - 4 columns of velocity (m/s)
% snr - 4 columns of beam signal-to-noise ratio (dB)
% corr - 4 columns of beam correlation (%)
% amp - 4 columns of beam amplitude (counts)
% temperature - temperature (C)
% c_sound - speed of sound (m/s)
% head_serial_number - probe head serial number
% filename - filename that the data came from
% name - adv name (typically 'advN' where N is a positive integer)
% number - adv number (typically 'N' from the previous line)
% comments - text of full comments

disp(['      load_adv_data: ',filename]);

if ~exist(filename,'file') % give up if there's no file matching
    disp(['         load_adv_data: cannot find ',filename]);
    data = NaN;
else
    data_array = load(filename);
    if isempty(data_array)
        disp('         load_adv_data: no data')
        data = NaN;
    else
        % load the velocities, amplitudes, correlations
        disp('         load_adv_data: getting velocities,amplitudes,correlations');
        data.vel = data_array(:,2:5);
        data.amp = data_array(:,6:9);
        data.corr = data_array(:,10:13);
        
        % load noise levels
        disp('         load_adv_data: getting noise levels');
        fid=fopen(filename);
        found=0;
        while ~found
            tline=fgetl(fid);
            if tline==-1 % no noise amplitude found
                disp('         load_adv_data: no noise levels - SNR forced to 0');
                data.snr = zeros(size(data.amp));
                found=1;
            else
                if contains(tline,'NoiseAmplitudes:')
                    disp('         load_adv_data: calculating SNR');
                    data.noise = cell2mat(textscan(tline((strfind(tline,'{')+1):(strfind(tline,'}')-1)),'%f %f %f %f','delimiter',','));
                    for nbeam=1:length(data.noise)
                        data.snr(:,nbeam) = 20*log10(data.amp(:,nbeam)/data.noise(nbeam));
                        data.snr(data.snr<0) = 0; % SnR can't be negative by definition
                    end
                    found=1;
                end
            end
        end
        fclose(fid);
        
        % load temperature and speed of sound
        disp('         load_adv_data: getting water temperature and speed of sound');
        data.temperature = read_comment_value_from_data_file(filename,'Temperature:','%f');
        data.c_sound = read_comment_value_from_data_file(filename,'SpeedOfSound:','%f');
        
        % load probe head serial number
        disp('         load_adv_data: getting probe head serial number');
        data.head_serial_number = read_comment_value_from_data_file(filename,'HeadSerialNumber:','%s');
        data.downlooker = is_downlooker(data.head_serial_number);
        
        % filename
        data.filename = filename;
        
        % adv number
        data.number = cell2mat(textscan(filename(strfind(filename,'adv'):length(filename)),'adv%d.txt'));
        
        % adv name
        data.name = ['adv',num2str(data.number,'%d')];
        
        % load comments
        disp('         load_adv_data: loading comments');
        data.comments=load_comments(filename,'');
    end    
end

% done
disp('      load_adv_data done');

