function data = load_analog_data(filename)

% LOAD_ANALOG_DATA
%
% data = load_analog_data(fname);
%
% Loads data from the analog record specified.  If no record is present, returns NaN.
%
% data structure has the following fields:
% analog - timeseries of analog channels
% channelnames - list of channel names for analog channels
% role - 'LEADER', 'FOLLOWER', or 'STANDALONE'
% time - synthesized time
% fs - sample rate
% filename - filename that the data came from
% comments - text of full comments

disp(['      load_analog_data: ',filename]);

if ~exist(filename,'file') % give up if there's no file matching
    disp(['         load_analog_data: cannot find ',filename]);
    data = NaN;
else
    % load the data
    disp('         load_analog_data: loading data');
    data.analog = load(filename);

    if ~isempty(data.analog)
        % filename
        data.filename = filename;
        
        % load channel names
        disp('         load_analog_data: loading channel names');
        fid=fopen(filename);
        found=0;
        while ~found
            tline=fgetl(fid);
            if tline==-1 % no channel names found
                disp('            load_analog_data: can not find channel names');
                found=1;
            else
                if contains(tline,'ChannelNames:')
                    temp = textscan(tline((strfind(tline,':')+1):length(tline)),'%s','delimiter',',');
                    data.channelnames = temp{:};
                    found=1;
                end
            end
        end
        fclose(fid);
      
        % load cutoff frequency
        disp('         load_analog_data: loading low-pass cutoff frequency');
        fc = read_comment_value_from_data_file(filename,'LowpassCutoffFrequency:','%f');
        if isfinite(fc)
            data.fc = fc;
        end
        
        % load start time
        disp('         load_analog_data: loading start time');
        datetime_string = read_comment_value_from_data_file(filename,'StartDateTimeUTC:','%s');
        if isnan(datetime_string) % backwards compatibility with pre-2013 DAQ
            datetime_string = read_comment_value_from_data_file(filename,'StartDateTimeLocal:','%s');
        end
        datetime_value = datetime(datetime_string,'InputFormat','eee yyyy/MM/dd HH:mm:ss.SSS ''UTC''','TimeZone','UTC');
        if isfinite(convertTo(datetime_value,'datenum'))
            if isfinite(fc)
                t = scxi_1143_filter_lag_time(fc)/86400; % in days
            else
                t = 0;
            end
            data.startdatenum = convertTo(datetime_value,'datenum') - t;      
        end

        % load sample rate
        disp('         load_analog_data: loading sample rate');
        fs = read_comment_value_from_data_file(filename,'SampleRate:','%f');
        if isfinite(fs)
            disp('         load_analog_data: calculating time');
            data.fs = fs;
            data.time = (0:size(data.analog,1)-1).'/fs;
        else
            disp('         load_analog_data: cannot calculate time');
        end

        % load LEADER/FOLLOWER/STANDALONE
        disp('         load_analog_data: loading LEADER/FOLLOWER/STANDALONE');
        data.role = strtrim(read_comment_value_from_data_file(filename,'DAQRole:','%s'));
              
        % load comments
        disp('         load_analog_data: loading comments');
        data.comments=load_comments(filename,'');
    else
        disp('         load_analog_data: no data!');
        data = NaN;
    end
end

% done
disp('      load_analog_data done');
end

function t=scxi_1143_filter_lag_time(fc)
% calculates filter delay time 't' (s) for a given cutoff frequency 'fc' (Hz)
% from Maddux #16, p.92, 10-23-2013
V = 1; % wmstart>1 threshold voltage is the HWRL standard for timing
a = 0.002189;
b = -0.01453;
c = 0.104;
d = 0.6721;
e = 0.06983;
G = a*V^3 + b*V^2 + c*V + d + e*log(V);
t0 = -1.5e-5;
t = G/fc + t0;
end
