function calibrations = load_calibrations(sharename)

% LOAD_CALIBRATIONS
%
% calibrations = load_calibrations(sharename,year,projname);
%
% Loads instrument calibrations and/or sets them from known metadata.  Returns
% them in the structure 'calibrations'.
%
% calibrations.channelname - a channel with m calibrations
% calibrations.channelname.time(m) - calibration time (float, in decimal yearday) 
% calibrations.channelname.slope(m) - calibration slope (float)
% calibrations.channelname.offset(m) - calibration offsets (float)
% calibrations.channelnameunitsz(m) - calibration units (string)

disp('load_calibrations: starting');

% find data/calibration/summary/ for the project
% assumes HWRL project share conventions
cals_dirname = fullfile(sharename,'data','calibration','summary');
disp(['   load_calibrations: searching ',cals_dirname]);
if ~exist(cals_dirname,'dir') % give up if there's no directory matching
    disp(['   load_calibrations: cannot find ',cals_dirname])
    calibrations = NaN;
    return
else
    cals_fnames = dir(cals_dirname); % look for xlsx files in here
    if isempty(cals_fnames) % give up if the directory is empty
        disp(['   load_calibrations: cannot find anything in ',cals_dirname])
        calibrations = NaN;
        return
    else
        for cal_filenum=1:length(cals_fnames)
            if cals_fnames(cal_filenum).isdir
                % ignore any directories
            else
                % not a directory so it must be a file.  process it
                fname = cals_fnames(cal_filenum).name;
                if contains(fname,'.xlsx')
                    if contains(fname,'cal_summary')
                        disp(['   load_calibrations: loading ',fullfile(cals_dirname,fname)])
                        
                        % get the time
                        file_timestamp = parse_file_timestamp(fname,'cal_summary');
                        if isnan(file_timestamp) % unrecognized - use the file modification date
                            disp(['   load_calibrations: cannot recognize timestamp of ',fname])
                            disp('   load_calibrations: using filesystem modification date')
                            file_timestamp = convertTo(datetime(cals_fnames(cal_filenum).date),'datenum');
                        end
                        
                        % convert to UTC
                        if is_Daylight_Savings(char(datetime(file_timestamp,'ConvertFrom','datenum'),'MM/dd/yyyy'))
                            file_timestamp = file_timestamp + 7/24; % PDT -> UTC
                        else
                            file_timestamp = file_timestamp + 8/24; % PST -> UTC
                        end
                        
                        % subtract yearday one
                        % removed 12/9/2015
                        % file_timestamp = file_timestamp - datenum([year,'0101000000'],'yyyymmddHHMMSS') + 1;
                                               
                        % load the xlsx file
                        [~,~,raw] = xlsread(fullfile(cals_dirname,fname));
                        for n=1:size(raw,1)-1
                            channelname = raw{n+1,1}; % channel name
                            slope = raw{n+1,6}; % slope
                            offset = raw{n+1,8}; % offset
                            if isnan(offset) % handle empty offset field
                                offset = 0;
                            end 
                            units = raw(n+1,9); % units
                            if ~iscell(units) % handle empty units field
                                units = 'unknown';
                            end 
                            if ~isnan(channelname)
                                if exist('calibrations','var') && isfield(calibrations,channelname)
                                    calibrations.(channelname).time = [calibrations.(channelname).time, file_timestamp]; % timestamp
                                    calibrations.(channelname).slope = [calibrations.(channelname).slope, slope]; % slope
                                    calibrations.(channelname).offset = [calibrations.(channelname).offset, offset]; % offset
                                    calibrations.(channelname).units = [calibrations.(channelname).units, units]; % units
                                else
                                    calibrations.(channelname).time = file_timestamp; % timestamp
                                    calibrations.(channelname).slope = slope; % slope
                                    calibrations.(channelname).offset = offset; % offset
                                    calibrations.(channelname).units = units; % units
                                end
                            end
                        end
                   end % cal_summary
                end % .xlsx
            end
        end % cal_filenum
    end
end

% done
disp('load_calibrations: done');
return