function instr_loc=load_instrument_locations(sharename)

% LOAD_INSTRUMENT_LOCATIONS
%
% instr_loc=load_instrument_locations(sharename,year,projname);
%
% Loads instrument positions and/or sets them from known metadata.  Returns
% them in the structure 'instr_loc', defined as follows:
% instr_loc.channelname - a channel with m locations
% instr_loc.channelname.time(m) - location time (float, in decimal yearday) 
% instr_loc.channelname.x(m) - location x (float)
% instr_loc.channelname.y(m) - location y (float)
% instr_loc.channelname.z(m) - location z (float)

% setup
disp('load_instrument_locations: starting')

% find docs/setup/instm_locations/ for the project
% assumes HWRL project share conventions
instm_locations_dirname = fullfile(sharename,'docs','setup','instm_locations');
disp(['   load_instrument_locations: searching ',instm_locations_dirname]);
if ~exist(instm_locations_dirname,'dir') % give up if there's no directory matching
    disp(['   load_instrument_locations: cannot find ',instm_locations_dirname])
    instr_loc = NaN;
    return
else
    instm_locations_fnames = dir(instm_locations_dirname); % look for xls files in here
    if isempty(instm_locations_fnames) % give up if the directory is empty
        disp(['   load_instrument_locations: cannot find anything in ',instm_locations_dirname])
        instr_loc = NaN;
        return
    else
        for instm_locations_num=1:length(instm_locations_fnames)
            if instm_locations_fnames(instm_locations_num).isdir
                % ignore any directories
            else
                % not a directory so it must be a file.  process it
                fname = instm_locations_fnames(instm_locations_num).name;
                if contains(fname,'.xls') && ~contains(fname,'~$')
                    disp(['   load_instrument_locations: loading ',fullfile(instm_locations_dirname,fname)])
                    
                    % get the time
                    file_timestamp = parse_file_timestamp(fname,'instm_locations');
                    if isnan(file_timestamp) % unrecognized - use the file modification date
                        disp(['   load_instrument_locations: cannot recognize timestamp of ',fname])
                        disp('   load_instrument_locations: using filesystem modification date')
                        file_timestamp = convertTo(datetime(cals_fnames(cal_filenum).date),'datenum');
                    end
                    
                    % convert to UTC
                    if is_Daylight_Savings(char(datetime(file_timestamp,'ConvertFrom','datenum'),'MM/dd/yyyy'))
                        file_timestamp = file_timestamp + 7/24; % PDT -> UTC
                    else
                        file_timestamp = file_timestamp + 8/24; % PST -> UTC
                    end
                    
                    % subtract yearday one
                    % removed 7/25/2017
                    %file_timestamp = file_timestamp - datenum([year,'0101000000'],'yyyymmddHHMMSS') + 1;
                    
                    % load the xls file
                    [~,~,raw] = xlsread(fullfile(instm_locations_dirname,fname));
                    for n=1:size(raw,1)-1
                        channelname = raw{n+1,1}; % channel name
                        if exist('instr_loc','var') && isfield(instr_loc,channelname)
                            % existing channel
                            instr_loc.(channelname).time = [instr_loc.(channelname).time, file_timestamp]; % timestamp
                            instr_loc.(channelname).x = [instr_loc.(channelname).x, raw{n+1,4}]; % x
                            instr_loc.(channelname).y = [instr_loc.(channelname).y, raw{n+1,5}]; % y
                            instr_loc.(channelname).z = [instr_loc.(channelname).z, raw{n+1,6}]; % z
                            if contains(channelname,'adv') % this is an adv
                                instr_loc.(channelname).i = [instr_loc.(channelname).i, raw{n+1,7}]; % i
                                instr_loc.(channelname).j = [instr_loc.(channelname).j, raw{n+1,8}]; % j
                                instr_loc.(channelname).k = [instr_loc.(channelname).k, raw{n+1,9}]; % k
                            end                               
                        else
                            % new channel
                            instr_loc.(channelname).time = file_timestamp; % timestamp
                            instr_loc.(channelname).x = raw{n+1,4}; % x
                            instr_loc.(channelname).y = raw{n+1,5}; % y
                            instr_loc.(channelname).z = raw{n+1,6}; % z
                            if contains(channelname,'adv') % this is an adv
                                instr_loc.(channelname).i = raw{n+1,7}; % i
                                instr_loc.(channelname).j = raw{n+1,8}; % j
                                instr_loc.(channelname).k = raw{n+1,9}; % k
                            end     
                        end
                    end
                else
                    disp(['   load_instrument_locations: unrecognized file ',fname])
                end
            end
        end
        if ~exist('instr_loc','var')
            % no instrument locations present
            instr_loc = NaN;
        end
    end
end

% done
disp('load_instrument_locations done');

