function data = load_mta_data(lrf_fname,mta_fname)

% LOAD_MTA_DATA
%
% data = load_mta_data(lrf_fname,mta_fname);
%
% Loads laser range finder (lrf_fname) and mta survey data (mta_name) from
% the specified files.  Interpolates laser range finder output to the times
% at which MTA surveys were collected, resulting in a bathymetric survey
% with a single distance column and multiple range columns.  Converts MTA
% survey data from cm to m.  Returns the results in the variable "data".

% load from laser range finder file
disp(['load_mta_data: loading ',lrf_fname])
lrf_data = load(lrf_fname);
lrf_time = lrf_data(:,1);
lrf_range = lrf_data(:,2);

% load from mta file
disp(['load_mta_data: loading ',mta_fname])
mta_data = load(mta_fname);
mta_time = mta_data(:,1); % column 1 is time
mta_range = mta_data(:,3:end); % column 2 is an HH:MM:SS timestamp that we ignore

% interpolate laser range finder values onto mta timestamps
disp('load_mta_data: interpolating laser range finder to mta')
lrf_range_interp = interp1(lrf_time,lrf_range,mta_time,'linear','extrap');

% identify bad (0) values in mta_range and interpolate them
disp('load_mta_data: cleaning up mta')
x=[];
y=[];
z=[];
M = size(mta_range,1); % number of timesteps
N = size(mta_range,2); % number of channels
for n=1:N % build new array of nonzero values
    for m=1:M
        if mta_range(m,n) > eps % keep nonzero values
            x=[x; m]; %#ok<AGROW>
            y=[y; n]; %#ok<AGROW>
            z=[z; mta_range(m,n)]; %#ok<AGROW>
        end
    end
end
F = scatteredInterpolant(x,y,z);
[qx,qy]=meshgrid(1:M,1:N);
mta_range_clean=F(qx,qy);

% build data array
disp('load_mta_data: finishing')
data(1,:) = lrf_range_interp;
data(1+(1:N),:) = mta_range_clean*0.01; % convert to meters

% done
disp('load_mta_data: done')
return