function data = load_raw_trial(sharename,trialname)

% LOAD_RAW_TRIAL
%
% data = load_raw_trial(sharename,year,projname,expname,trialname);
%
% Loads data from the trial specified.  Includes leader,follower,ADV data.
% Requires coherency among all the channels sampled. Data files of
% differing lengths will be dropped. Also averages together any timestamps
% that are present. Best if all channel names are unique; if they aren't
% then bad things will happen during processing.

% build the directory.  assumes HWRL project share conventions
dirname = fullfile(sharename,trialname);
disp(['   load_raw_trial: ',dirname]);

if ~exist(dirname,'dir') % give up if there's no directory matching
    disp(['      load_raw_trial: cannot find ',dirname])
    data=NaN;
else
    fnames=dir(dirname); % look for files in here
    if isempty(fnames) % give up if the directory is empty
        disp(['      load_raw_trial: cannot find anything in ',dirname])
        data=NaN;
    else
        % load all data
        for nf=1:length(fnames)
            if fnames(nf).isdir
                % directory so ignore it
            else
                % load this file
                fname=fullfile(dirname,fnames(nf).name);
                if strfind(fnames(nf).name,'Wave')
                    % analog file
                    disp(['      load_raw_trial: found analog file ',fname])
                    analog_data = load_analog_data(fname);
                    if ~isstruct(analog_data)
                        disp('      load_raw_trial: no data, skipping')
                    else
                        % put everything into the 'data' structure
                        if exist('data','var')
                            data = add_analog_data(analog_data,data);
                        else
                            data = add_analog_data(analog_data);
                        end
                    end
                elseif strfind(fnames(nf).name,'_adv')
                    % adv file
                    disp(['      load_raw_trial: found ADV file ',fname])
                    adv_data = load_adv_data(fname);
                    if ~isstruct(adv_data)
                        disp('      load_raw_trial: no data, skipping')
                    else
                        % put everything into the 'data' structure
                        if exist('data','var')
                            data = add_adv_data(adv_data,data);
                        else
                            data = add_adv_data(adv_data);
                        end
                    end
                else % I don't know what it is
                    disp(['      load_raw_trial: huh?  what is ',fname])
                end
            end
        end
        
        % post processing
        if ~exist('data','var')
            % we didn't get any data
            beep;
            disp(['      load_raw_trial: no data loaded from ',dirname])
            data = NaN;
        elseif ~isstruct(data)
            % we didn't get any data
            beep;
            disp(['      load_raw_trial: no data loaded from ',dirname])
            data = NaN;
        else
            % we got data
            data = consolidate_timing(data); % consolidate timing
            data = group_data(data); % identify data groupings
        end
    end
end

% done
disp('    load_raw_trial done');
end

function data = add_analog_data(varargin)
% adds analog data to the 'data' structure and returns 'data'
%
% if you have no 'data' structure, use:
% data = add_analog_data(analog_data);
%
% to add to existing 'data' structure, use:
% data = add_analog_data(analog_data,data);
if isempty(varargin)
    % no inputs, what the --?
    data = NaN;
else
    % get analog data
    analog_data = varargin{1};
    
    % find wavemaker start
    Nc = length(analog_data.channelnames); % number of channels
    found_wmstart = 0;
    nc = 1;
    while ~found_wmstart && nc<=Nc
        if ~isempty(strfind(analog_data.channelnames{nc}, 'wmstart'))
            wmstart = analog_data.analog(:,nc); % save analog record as 'wmstart'
            found_wmstart = 1;
        end
        nc = nc + 1;
    end     
    
    % put channelnames and filenames in 'data'
    if length(varargin) > 1
        % we have a 'data' structure to add to
        data = varargin{2};
        if isfield(data,'channelnames') % we have some channelnames; append new ones
            data.channelnames = [data.channelnames; analog_data.channelnames];
        else % we don't have any channelnames yet yet, record these
            data.channelnames = analog_data.channelnames;
        end
        if isfield(data,'filenames') % we have some filenames
            if ~any(contains(data.filenames,analog_data.filename)) % we don't have this one, record it
                data.filenames = [data.filenames; {analog_data.filename}];
            end
        else % we don't have any filenames yet, record this one
            data.filenames = {analog_data.filename};
        end
    else
        % new (empty) 'data' structure
        data.channelnames = analog_data.channelnames; % channelnames
        data.filenames = {analog_data.filename}; % filename
    end
    
    % make every channelname a field name of 'data'
    for nc=1:Nc
        channelname = analog_data.channelnames{nc}; % new field with this channelname
        data.(channelname) = rmfield(analog_data,'channelnames'); % assign everything but 'channelnames' field to it
        data.(channelname).analog = analog_data.analog(:,nc); % cut back to just the one channel of data
        if found_wmstart
            data.(channelname).wmstart = wmstart; % record wmstart for this channel if we found it
        end
    end
end

end

function data = add_adv_data(varargin)
% adds adv data to the 'data' structure and returns 'data'
%
% if you have no 'data' structure, use:
% data = add_adv_data(adv_data);
%
% to add to existing 'data' structure, use:
% data = add_adv_data(adv_data,data);
if isempty(varargin)
    % no inputs, what the --?
    data = NaN;
else
    adv_data = varargin{1};
    if length(varargin) > 1
        % we have a 'data' structure to add to
        data = varargin{2};
        if isfield(data,'advnames') % we have some advnames; append this one
            data.advnames = [data.advnames; {adv_data.name}];
        else % we don't have any advnames yet yet, record these
            data.advnames = {adv_data.name};
        end
    else
        % new (empty) 'data' structure
        data.advnames = {adv_data.name}; % adv name
    end
    
    % this adv name is a field name, with all adv_data in it
    data.(adv_data.name) = adv_data;
end

end

function data = consolidate_timing(data_old)
% consolidates timing of the different parts of 'data'

% keep copy of the old data for debug
data = data_old;

% find a sample rate, record length and role for each file
Nf = length(data.filenames); % number of files
if Nf > 0 && isfield(data,'channelnames')
    Nc = length(data.channelnames); % number of analog channels
    sample_rates = zeros(size(data.filenames)); % sample rate for every file
    record_lengths = zeros(size(data.filenames)); % record length for every file
    roles = cell(size(data.filenames)); % role for every file
    for nf=1:Nf
        found = 0;
        nc = 1;
        while ~found && nc<=Nc
            channelname = data.channelnames{nc};
            if isfield(data,channelname)
                if strcmp(data.(channelname).filename,data.filenames{nf})
                    sample_rates(nf) = data.(channelname).fs;
                    record_lengths(nf) = length(data.(channelname).analog);
                    roles(nf) = {data.(channelname).role};
                    found = 1;
                end
            end
            nc=nc+1; % increment channel number
        end
        if ~found
            sample_rates(nf) = NaN;
            record_lengths(nf) = NaN;
        end
    end
end

% handle multiple analog files
if Nf > 1 && isfield(data,'channelnames')
    % find files with same sample rate, length, and not STANDALONE role and
    % average 'startdatenum' and 'wmstart' of corresponding channels
    done = zeros(size(sample_rates)); % no files are done yet, by default
    nf = 1;
    while all(~done) && nf<Nf
        if ~done(nf) % if we haven't done this one yet
            % current sample rate, record length, role, filename
            fs = sample_rates(nf);
            rl = record_lengths(nf);
            role = roles{nf};
            if ~strcmp(role,'STANDALONE') % no match for an 'STANDALONE' role
                % find files not done yet with matching fs, record
                % length, that aren't 'STANDALONE' role
                matches = ~done(nf+1:end) && (fs == sample_rates(nf+1:end)) && ...
                    (rl == record_lengths(nf+1:end)) && ~strcmp(role(nf+1:end),'STANDALONE');
                if any(matches) % we have a match
                    % get a startdatenum and wmstart from this file
                    found = 0;
                    nc = 1;
                    while ~found && nc<=Nc
                        channelname = data.channelnames{nc};
                        if isfield(data,channelname)
                            if strcmp(data.(channelname).filename,data.filenames{nf})
                                new_startdatenum = data.(channelname).startdatenum;
                                new_startdatenum_count = 1;
                                if isfield(data.(channelname),'wmstart')
                                    new_wmstart = data.(channelname).wmstart;
                                    new_wmstart_count = 1;
                                end
                                found = 1;
                            end
                        end
                        nc=nc+1; % increment channel number
                    end
                    
                    % get time and wmstart from matching files
                    matching_indexes = find(matches)+nf;
                    matching_filenames = data.filenames(matching_indexes);
                    Nmf = length(matching_filenames);
                    for m=1:Nmf
                        found = 0;
                        nc = 1;
                        while ~found && nc<=Nc
                            channelname = data.channelnames{nc};
                            if isfield(data,channelname)
                                if strcmp(data.(channelname).filename,matching_filenames{m})
                                    if exist('new_startdatenum','var')
                                        new_startdatenum = new_startdatenum + data.(channelname).startdatenum;
                                        new_startdatenum_count = new_startdatenum_count + 1;
                                    else
                                        new_startdatenum = data.(channelname).startdatenum;
                                        new_startdatenum_count = 1;
                                    end
                                    if isfield(data.(channelname),'wmstart')
                                        if exist('new_wmstart','var')
                                            new_wmstart = new_wmstart + data.(channelname).wmstart;
                                            new_wmstart_count = new_wmstart_count + 1;
                                        else
                                            new_wmstart = data.(channelname).wmstart;
                                            new_wmstart_count = 1;
                                        end
                                    end
                                    found = 1;
                                end
                            end
                            nc=nc+1; % increment channel number
                        end
                    end
                    
                    % average time and wmstart
                    new_startdatenum = new_startdatenum/new_startdatenum_count;
                    if exist('new_wmstart','var')
                        new_wmstart = new_wmstart/new_wmstart_count;
                    end
                    
                    % apply average time and wmstart to this file
                    for nc=1:Nc
                        channelname = data.channelnames{nc};
                        if isfield(data,channelname)
                            if strcmp(data.(channelname).filename,data.filenames{nf})
                                data.(channelname).startdatenum = new_startdatenum;
                                if exist('new_wmstart','var')
                                    data.(channelname).wmstart = new_wmstart;
                                end
                            end
                        end
                    end
                    
                    % apply average time to matching files
                    for m=1:Nmf
                        for nc=1:Nc
                            channelname = data.channelnames{nc};
                            if isfield(data,channelname)
                                if strcmp(data.(channelname).filename,matching_filenames{m})
                                    data.(channelname).startdatenum = new_startdatenum;
                                    if exist('new_wmstart','var')
                                        data.(channelname).wmstart = new_wmstart;
                                    end
                                end
                            end
                        end
                    end
                    
                    % we are done with matching files
                    done(matching_indexes) = 1;
                end
            end
            
            % we are done with this file
            done(nf) = 1;
            nf = nf+1;
        end
    end
end

% finally match advs to a leader file with the same length
if isfield(data,'advnames') && ~isempty(data.advnames) ... % we have advs
        && Nf > 0 && Nc > 0 ... % we have a nonzero number of analog files and channels
        && isfield(data,'channelnames') && ~isempty(data.channelnames) % we have analog files & channels
    % check advs and remove any that do not match
    Nadv = length(data.advnames);
    for nadv = 1:Nadv
        advname = data.advnames{nadv}; % adv name
        if isfield(data,advname) % matching field in data for this adv name
            advlength = size(data.(advname).vel,1); % length of this adv record
            N_max_overlength = 25; % no more than 25 extra adv samples
            if any(strcmp(roles,'LEADER') ...
                    & advlength >= record_lengths ...
                    & advlength - record_lengths <= N_max_overlength) 
                % we have a match or close enough
                if ~exist('new_advnames','var')
                    % create a list of new adv names
                    new_advnames = {advname};
                else
                    % add to list of new adv names
                    new_advnames = [new_advnames; {advname}]; %#ok<AGROW>
                end
                
                % check length of this adv record and crop to match
                if ~any(strcmp(roles,'LEADER') & record_lengths == advlength)
                    % we don't have any perfect length matches
                    if any(strcmp(roles,'LEADER') & record_lengths == advlength - 1)
                        % adv has one extra sample at the START (this is common)
                        % Throw out the extra sample and keep the rest
                        disp(['      load_raw_trial: ',advname,' one extra sample REMOVED'])
                        data.(advname).vel = data.(advname).vel(2:end,:);
                        data.(advname).amp = data.(advname).amp(2:end,:);
                        data.(advname).snr = data.(advname).snr(2:end,:);
                        data.(advname).corr = data.(advname).corr(2:end,:);
                        advlength = advlength - 1;
                    else
                        % throw out first sample from start
                        data.(advname).vel = data.(advname).vel(2:end,:);
                        data.(advname).amp = data.(advname).amp(2:end,:);
                        data.(advname).snr = data.(advname).snr(2:end,:);
                        data.(advname).corr = data.(advname).corr(2:end,:);
                        advlength = advlength - 1;
                        % throw out remaining samples from end
                        found = 0;
                        n_overlength = 1;
                        while ~found && n_overlength<N_max_overlength
                            if any(strcmp(roles,'LEADER') & record_lengths == advlength - n_overlength)
                                % throw out remaining samples from end
                                disp(['      load_raw_trial: ',advname,' ',num2str(n_overlength+1),' extra samples REMOVED'])
                                data.(advname).vel = data.(advname).vel(1:end-n_overlength,:);
                                data.(advname).amp = data.(advname).amp(1:end-n_overlength,:);
                                data.(advname).snr = data.(advname).snr(1:end-n_overlength,:);
                                data.(advname).corr = data.(advname).corr(1:end-n_overlength,:);
                                advlength = advlength - n_overlength;
                                found = 1;
                            else
                                n_overlength = n_overlength + 1; % increment to next possible overlength of ADV record
                            end
                        end
                    end
                end
                                        
                % now find a length-matching LEADER file
                if any((record_lengths == advlength) & strcmp(roles,'LEADER'))
                    nf = find((record_lengths == advlength) & strcmp(roles,'LEADER'),1,'first');
                    leader_filename = data.filenames{nf};
                    found = 0;
                    nc = 1;
                    while ~found && nc<=Nc
                        channelname = data.channelnames{nc};
                        if isfield(data,channelname)
                            if strcmp(data.(channelname).filename,leader_filename)
                                data.(advname).startdatenum = data.(channelname).startdatenum;
                                data.(advname).time = data.(channelname).time;
                                data.(advname).fs = data.(channelname).fs;
                                if isfield(data.(channelname),'wmstart')
                                    data.(advname).wmstart = data.(channelname).wmstart;
                                end
                                found = 1;
                            end
                        end
                        nc=nc+1; % increment channel number
                    end
                end
            else
                % reject, remove from data, don't add to new_advnames
                beep;
                disp(['      load_raw_trial: ',advname,' size mismatch - REJECTING entire adv record'])
                data = rmfield(data,advname);
            end
        end
    end
    if exist('new_advnames','var') % if we kept any advs
        data.advnames = new_advnames;
    else
        data = rmfield(data,'advnames');
    end
end

end

function data = group_data(data_old)
% identifies different groups in the data
% a 'group' is a bunch of channels that have the same:
% 1) start time
% 2) sample rate (sampling frequency)
% 3) record length (number of samples)

% keep copy of the old data for debug
data = data_old;

if isstruct(data) % we have data
    if isfield(data,'channelnames')
        Nc = length(data.channelnames); % number of analog channels
        analog_group_assigned = zeros(size(data.channelnames)); % track which analog channel has a group assignment
    else
        Nc = 0; % no analog channels
        analog_group_assigned = [];
    end
    if isfield(data,'advnames')
        Nadv = length(data.advnames); % number of advs
        adv_group_assigned = zeros(size(data.advnames)); % track which adv has a group assignment
    else
        Nadv = 0; % no advs
        adv_group_assigned = [];
    end
    if ~Nc && ~Nadv
        Ngroups = 0; % we have no analog channels AND no advs
    else
        Ngroups = 1; % we have advs and/or analog channels so we have at least 1 group
        ng = 1; % first group number
        if Nc>0
            for nc=1:Nc % go through each channel
                if ~analog_group_assigned(nc) % this channel doesn't have a group
                    channelname = data.channelnames{nc}; % this channel name
                    data.(channelname).groupnumber = ng; % assign group number ng to this channel
                    analog_group_assigned(nc) = 1; % this channel now has a group assignment
                    for nc2 = (nc+1):Nc % go through the rest of the channels
                        if ~analog_group_assigned(nc2) % next channel doesn't have a group
                            channelname2 = data.channelnames{nc2}; % next channel name
                            if same_group(data.(channelname),data.(channelname2)) % these two channels are the same group
                                data.(channelname2).groupnumber = ng; % assign group number ng to next channel
                                analog_group_assigned(nc2) = 1; % next channel now has a group assignment
                            end
                        end
                    end % done checking rest of channels
                    if Nadv>0 % if we have advs
                        for nadv=1:Nadv % go through each adv
                            if ~adv_group_assigned(nadv) % next adv doesn't have a group
                                advname2 = data.advnames{nadv}; % next advname
                                if same_group(data.(channelname),data.(advname2)) % this channel and next adv are the same group
                                    data.(advname2).groupnumber = ng; % assign group number ng to next adv
                                    adv_group_assigned(nadv) = 1; % next adv now has a group assignment
                                end
                            end
                        end
                    end % done checking rest of advs
                    if ~all(analog_group_assigned) || ~all(adv_group_assigned) % we have unassigned channels or advs remaining
                        Ngroups = Ngroups+1; % increment the number of groups
                        ng = ng+1; % next group number
                    end
                end
            end
        end
        if Nadv>0
            for nadv=1:Nadv % go through each adv
                if ~adv_group_assigned(nadv) % this adv doesn't have a group
                    advname = data.channelnames{nadv}; % this adv name
                    data.(advname).groupnumber = ng; % assign group number ng to this adv
                    adv_group_assigned(nadv) = 1; % this adv now has a group assignment
                    for nadv2 = (nadv+1):Nadv % go through the rest of the advs
                        if ~adv_group_assigned(nadv2) % next adv doesn't have a group
                            advname2 = data.advnames{nadv2}; % next adv name
                            if same_group(data.(advname),data.(advname2)) % these two advs are the same group
                                data.(advname2).groupnumber = ng; % assign group number ng to next adv
                                adv_group_assigned(nadv2) = 1; % next adv now has a group assignment
                            end
                        end
                    end % done checking rest of advs
                    if Nc>0 % if we have analog channels
                        for nc2=1:Nc % go through each channel
                            if ~analog_group_assigned(nc2) % next channel doesn't have a group
                                channelname2 = data.channelnames{nc2}; % next channel name
                                if same_group(data.(advname),data.(channelname2)) % this adv and next channel are the same group
                                    data.(channelname2).groupnumber = ng; % assign group number ng to next channel
                                    analog_group_assigned(nc2) = 1; % next channel now has a group assignment
                                end
                            end
                        end
                    end % done checking rest of advs
                    if ~all(analog_group_assigned) || ~all(adv_group_assigned) % we have unassigned channels or advs remaining
                        Ngroups = Ngroups+1; % increment the number of groups
                        ng = ng+1; % next group number
                    end
                end
            end
        end
    end
    data.Ngroups = Ngroups; % assign Ngroups to data
end
end

function output = same_group(channel1,channel2)
% returns true if channel1 and channel2 have the same:
% 1) start time
% 2) sample rate (sampling frequency)
% 3) record length (number of samples)
if isfield(channel1,'vel') % adv
    N1 = size(channel1.vel,1); % record length 1
elseif isfield(channel1,'analog') % analog
    N1 = length(channel1.analog); % record length 1
else % unknown
    N1 = NaN;
end
if isfield(channel2,'vel') % adv
    N2 = size(channel2.vel,1); % record length 1
elseif isfield(channel2,'analog') % analog
    N2 = length(channel2.analog); % record length 1
else % unknown
    N2 = NaN;
end
if isfinite(N1) && isfinite(N2)
    output = (N1==N2) && (channel1.fs == channel2.fs) && (channel1.startdatenum == channel2.startdatenum);
else
    output = 0;
end
end