function process_raw_experiment(options)

% PROCESS_RAW_EXPERIMENT
%
% process_raw_experiment(options);
%
% Processes all the raw trials found for the specified options.
%
% This function does the prep work of finding instrument locations,
% calibration slopes and offsets, and then calls a raw trial processing
% subroutine.  The bulk of the work is done in PROCESS_RAW_TRIAL.
%
% SEE ALSO: PROCESS_RAW_OPTIONS, PROCESS_RAW_TRIAL

diary off
sharename = options.ShareName;
year = options.Year;
projname = options.ProjectName;
expname = options.ExperimentName;

% build the directory structure. assumes HWRL project share conventions
if ~exist(sharename,'dir') % give up if no HWRL share
    disp(['process_raw_experiment: cannot find ',sharename])
elseif ~exist(fullfile(sharename,'projects'),'dir') % give up if no projects folder
    disp(['process_raw_experiment: cannot find ',fullfile(sharename,'projects')])
elseif ~exist(fullfile(sharename,'projects',year),'dir') % give up if no projects/year folder
    disp(['process_raw_experiment: cannot find ',fullfile(sharename,'projects',year)])
elseif ~exist(fullfile(sharename,'projects',year,projname),'dir') % give up if no project folder
    disp(['process_raw_experiment: cannot find ',fullfile(sharename,'projects',year,projname)])
else % we found the project folder and can proceed
    projdirname = fullfile(sharename,'projects',year,projname); % we know this exists already
    if ~exist(fullfile(projdirname,'docs'),'dir') % make sure we have a docs directory
        % no docs directory (!); make one
        mkdir(fullfile(projdirname,'docs'));
    end
    
    if strcmp(options.Diary,'ON')
        % we are logging this
        logdirname = fullfile(projdirname,'docs','logs');
        if ~exist(logdirname,'dir') % create a logging directory if it doesn't exist yet
            mkdir(logdirname);
        end
        
        % start a logging diary
        diaryname = fullfile(logdirname,['process_raw_experiment_',expname,'.log']);
        if exist(diaryname,'file')
            delete(diaryname)
        end
        diary(diaryname);
    end
    
    disp('process_raw_experiment started')
    if strcmp(options.Diary,'ON')
        % we are logging this
        disp(['   process_raw_experiment: logging to ',diaryname])
    end
        
    % load instrument locations
    instr_loc = load_instrument_locations(sharename,year,projname);
    
    % load calibration slopes offsets and units
    calibrations = load_calibrations(sharename,year,projname);
    
    % build the directory.  assumes HWRL project share conventions
    expdirname = fullfile(projdirname,'data','raw',expname);
    disp(['   process_raw_experiment: ',expdirname]);
    if ~exist(expdirname,'dir') % give up if there's no directory matching
        disp(['   process_raw_experiment: cannot find ',expdirname])
        return
    else
        trialdirnames=dir(expdirname); % look for trials in here
        if isempty(trialdirnames) % give up if the directory is empty
            disp(['   process_raw_experiment: cannot find anything in ',expdirname])
            return
        else
            for trialnum=1:length(trialdirnames)
                if trialdirnames(trialnum).isdir
                    % each trial is in a directory of its own. process it
                    trialname=trialdirnames(trialnum).name;
                    if ~strcmp(trialname, '.') && ~strcmp(trialname, '..') && ~strcmp(trialname, '.AppleDouble')
                        pause(0.1);
                        process_raw_trial(trialname,calibrations,instr_loc,options);
                    end
                else
                    % not a directory so it must be a file.  ignore any files
                end
            end
        end
    end
    
    % done
    disp('process_raw_experiment done')
    diary off
end


end