function data = process_raw_trial(trialname,calibrations,instr_loc,options)

% PROCESS_RAW_TRIAL
%
% data=process_raw_trial(trialname,calibrations,instr_loc,options);
%
% Loads raw data for the specified trial.  Processes analog data, removing
% wave gage offsets and calibrating all analog channels.  Cleans up ADV
% data.  Records all processed trial data back out to specified locations
% and formats (based on options) for the given trial.

% setup
sharename = options.ShareName;
%year = options.Year;
%projname = options.ProjectName;
%expname = options.ExperimentName;
disp(['process_raw_trial for project ',trialname]);

% load trial
data = load_raw_trial(sharename,trialname);

% make sure we got data
if isstruct(data)
    if isfield(data,'channelnames') && ~isempty(data.channelnames)
        % process analog
        disp('   process_raw_trial: processing analog data')
        data = find_wmstart(data); % find zero_range, waves_running_range
        
        if strcmp(options.CleanUpUltrasonicWaveGages,'ON')
            % clean up any uswgs
            uswgchannels = find_uswgs(data); % find uswgs for cleanup
            if ~isempty(uswgchannels)
                disp('   process_raw_trial: cleaning up sonic gages')
                Nuswg = length(uswgchannels);
                for nuswg = 1:Nuswg
                    nc = uswgchannels(nuswg);
                    channelname = data.channelnames{nc};
                    disp(['   process_raw_trial: cleaning up ',channelname])
                    data.(channelname) = cleanup_uswg(data.(channelname));
                end
            end
        end
        
        if strcmp(options.CalRemoveZeroOffsets,'WG_ONLY') || strcmp(options.CalRemoveZeroOffsets,'WG_AND_USWG')
            % remove offsets on wgchannels from still water period
            wgchannels = find_wgs(data); % find wgs for zero offset removal
            if ~isempty(wgchannels)
                disp('   process_raw_trial: removing wave gauge still water offsets')
                Nwg = length(wgchannels);
                for nwg = 1:Nwg
                    nc = wgchannels(nwg);
                    channelname = data.channelnames{nc};
                    zero_offset = mean(data.(channelname).analog(data.(channelname).zero_range)); % average value during zero_range
                    data.(channelname).zero_offset = zero_offset; % save the zero offset
                    data.(channelname).analog = data.(channelname).analog - zero_offset; % subtract from full timeseries
                end
            end
            if strcmp(options.CalRemoveZeroOffsets,'WG_AND_USWG')
                % remove offsets on uswgchannels from still water period
                if ~exist('uswgchannels','var')
                    uswgchannels = find_uswgs(data); % find uswgs
                end
                if ~isempty(uswgchannels)
                    disp('   process_raw_trial: cleaning up ultrasonic wave gauge still water offsets')
                    Nuswg = length(uswgchannels);
                    for nuswg = 1:Nuswg
                        nc = uswgchannels(nuswg);
                        channelname = data.channelnames{nc};
                        zero_offset = mean(data.(channelname).analog(data.(channelname).zero_range)); % average value during zero_range
                        data.(channelname).zero_offset = zero_offset; % save the zero offset
                        data.(channelname).analog = data.(channelname).analog - zero_offset; % subtract from full timeseries
                    end
                end
            end
        elseif strcmp(options.CalRemoveZeroOffsets,'ALL')
            disp('   process_raw_trial: removing all still water offsets')
            Nc = length(data.channelnames);
            for nc=1:Nc
                channelname = data.channelnames{nc};
                zero_offset = mean(data.(channelname).analog(data.(channelname).zero_range)); % average value during zero_range
                data.(channelname).zero_offset = zero_offset; % save the zero offset
                data.(channelname).analog = data.(channelname).analog - zero_offset; % subtract from full timeseries
            end
        elseif strcmp(options.CalRemoveZeroOffsets,'NONE')
            disp('   process_raw_trial: this is probably a bad idea, but')
            disp('   process_raw_trial: NO zero offsets removed!!!')
        else
            % invalid option
            % don't remove any zero offsets
            beep
            disp(['   process_raw_trial: option CalRemoveZeroOffsets value ',options.CalRemoveZeroOffsets,' NOT recognized!!!'])
            disp('   process_raw_trial: NO zero offsets removed!!!')
        end
        
        % calibrate
        data = calibrate_raw_data(data,calibrations,options.CalInterpolation);
        
        % locate
        data = locate_raw_data(data,instr_loc);
        
        % still water depth
        data.h = find_still_water(data);
        
        % crop
        if strcmp(options.CropData,'WAVES_RUNNING') % crop to only when wavemaker start is high
            disp('   process_raw_trial: saving only when waves are running');
        elseif strcmp(options.CropData,'EXCLUDE_STILL_WATER') % crop from wavemaker start to end of record
            disp('   process_raw_trial: saving from wavemaker start to end of record');
        elseif strcmp(options.CropData,'OFF') % save entire record
            disp('   process_raw_trial: saving entire record');
        else % not recognized; saving entire record
            beep
            disp(['   process_raw_trial: option CropData, value ',options.CropData,' not recognized!!!']);
            disp('   process_raw_trial: saving entire record');
        end
        Nc = length(data.channelnames);
        for nc=1:Nc
            channelname = data.channelnames{nc};
            if strcmp(options.CropData,'WAVES_RUNNING') % crop to only when wavemaker start is high
                save_index = data.(channelname).waves_running_range;
                data.(channelname).time = data.(channelname).time(save_index);
                data.(channelname).startdatenum = data.(channelname).startdatenum + data.(channelname).time(1)/86400;
                data.(channelname).time = data.(channelname).time - data.(channelname).time(1);
                data.(channelname).analog = data.(channelname).analog(save_index);
            elseif strcmp(options.CropData,'EXCLUDE_STILL_WATER') % crop from wavemaker start to end of record
                save_index = data.(channelname).waves_running_range(1):length(data.(channelname).analog);
                data.(channelname).time = data.(channelname).time(save_index);
                data.(channelname).startdatenum = data.(channelname).startdatenum + data.(channelname).time(1)/86400;
                data.(channelname).time = data.(channelname).time - data.(channelname).time(1);
                data.(channelname).analog = data.(channelname).analog(save_index);
            end
        end
    end % process analog data
    
    % process advs
    if isfield(data,'advnames') && ~isempty(data.advnames)
        % check adv temperatures
        data = check_adv_temperatures(data,options.VectrinoDefaultTemperature);
        
        % process advs
        Nadv = length(data.advnames);
        for nadv = 1:Nadv
            % setup for this adv
            advname = data.advnames{nadv};
            disp(['   process_raw_trial: processing ',advname])
            
            % clean up
            if data.(advname).z < data.h
                % advs are in water
                flags.use_amp = 0;
                flags.use_snr = 0;
            else
                % advs are in and out of water
                if strcmp(options.VectrinosUseAmplitudes,'ON')
                    flags.use_amp = 1; % use beam amplitudes to find when ADVs are in or out of water
                else
                    flags.use_amp = 0; % do not use beam amplitudes
                end
                if strcmp(options.VectrinosUseSNR,'ON')
                    flags.use_snr = 1; % use beam signal-to-noise ratio to find when ADVs are in or out of water
                else
                    flags.use_snr = 0; % do not use beam signal-to-noise ratio
                end
            end
            if strcmp(options.VectrinosUseCorrelations,'ON')
                flags.use_corr = 1; % use beam correlation to identify bad points
            else
                flags.use_corr = 0; % do not use beam correlation
            end
            if strcmp(options.VectrinosUsePhaseSpace,'ON')
                flags.use_phase = 1; % use phase space despiking algorithm to identify bad points
            else
                flags.use_phase = 0; % do not phase space despiking algorithm
            end
            switch(options.VectrinosSmoothing)
                case 'OFF'
                    flags.smoothing = 0;
                case 'MILD'
                    flags.smoothing = 1;
                case 'MEDIUM'
                    flags.smoothing = 2;
                case 'HEAVY'
                    flags.smoothing = 3;
                case 'EXTREME'
                    flags.smoothing = 4;
                otherwise
                    flags.smoothing = 0;
            end
            cleanadv = cleanup_adv(data.(advname),flags);
            
            % rotate into HWRL coordinates
            % uses i,j,k in location plans
            % common examples
            % u=uadv, v=vadv, w=wadv => i=1,j=2,k=3 (basic alignment)
            % u=uadv, v=-wadv, w=vadv => i=1,j=-3,k=2 (downlooking probe oriented sideways)      
            % u=uadv, v=wadv, w=-vadv => i=1,j=3,k=-2 (downlooking probe oriented sideways)      
            uvw = 'uvw';
            data.(advname).u = sign(data.(advname).i)*cleanadv.(uvw(abs(data.(advname).i)));
            data.(advname).v = sign(data.(advname).j)*cleanadv.(uvw(abs(data.(advname).j)));
            data.(advname).w = sign(data.(advname).k)*cleanadv.(uvw(abs(data.(advname).k)));
            
            % crop
            if strcmp(options.CropData,'WAVES_RUNNING') % crop to only when wavemaker start is high
                save_index = data.(advname).waves_running_range;
                data.(advname).time = data.(advname).time(save_index);
                data.(advname).startdatenum = data.(advname).startdatenum + data.(advname).time(1)/86400;
                data.(advname).time = data.(advname).time - data.(advname).time(1);
                data.(advname).u = data.(advname).u(save_index);
                data.(advname).v = data.(advname).v(save_index);
                data.(advname).w = data.(advname).w(save_index);
           elseif strcmp(options.CropData,'EXCLUDE_STILL_WATER') % crop from wavemaker start to end of record
                save_index = data.(advname).waves_running_range(1):length(data.(advname).u);
                data.(advname).time = data.(advname).time(save_index);
                data.(advname).startdatenum = data.(advname).startdatenum + data.(advname).time(1)/86400;
                data.(advname).time = data.(advname).time - data.(advname).time(1);
                data.(advname).u = data.(advname).u(save_index);
                data.(advname).v = data.(advname).v(save_index);
                data.(advname).w = data.(advname).w(save_index);
            end

            disp(['   process_raw_trial: done with ',advname])
        end
    end % process advs
    
    % save data
    save_processed_trial(data,trialname,options);
end

% done
disp('process_raw_trial done')
end

function save_processed_trial(data,trialname,options)
% save processed trial data according to options
% SaveTXT = 'ON' for ASCII .TXT files, one per channel
% SaveMAT = 'ON' for MATLAB .MAT files, one per trial
% SaveWaveLab = 'ON' for WaveLab3-compatible .DAT files, one per trial
sharename = options.ShareName;
%year = options.Year;
%projname = options.ProjectName;
%expname = options.ExperimentName;
if strcmp(options.SaveTXT,'ON')
    % save ASCII text files
    targetdir=fullfile('~','Downloads','txt',expname,trialname);
    if ~exist(targetdir,'dir')
        mkdir(targetdir);
    end
    eol = '\r\n'; % end of line

    % analog data
    if isfield(data,'channelnames') && ~isempty(data.channelnames)
        Nc = length(data.channelnames);
        for nc = 1:Nc
            channelname = data.channelnames{nc};
            channel_start_string = char(datetime(data.(channelname).startdatenum,'ConvertFrom','datenum'),'eee yyyy/MM/dd HH:mm:ss.SSS');
            if strcmp(channelname,'disp') % disp is a function
                fname = fullfile(targetdir,'displ.txt');
            elseif strcmp(channelname,'cond') % cond is a function too
                fname = fullfile(targetdir,'condctvty.txt');
            else
                fname = fullfile(targetdir,[channelname,'.txt']);
            end
            disp(['   process_raw_trial: saving data to ',fname]);
            fid=fopen(fname,'w');
            fprintf(fid,'%s',data.(channelname).comments);
            fprintf(fid,['%% [Calibrated Metadata]',eol]);
            fprintf(fid,['%% StillWaterDepth: %.4e',eol], data.h);
            fprintf(fid,['%% ChannelName: %s',eol],channelname);
            if isfield(data.(channelname),'zero_offset')
                fprintf(fid,['%% ChannelZeroOffsetVoltageRemoved: %.5e',eol], data.(channelname).zero_offset);
            end
            fprintf(fid,['%% CalibrationSlope: %.5e',eol], data.(channelname).calslope);
            fprintf(fid,['%% CalibrationOffset: %.5e',eol], data.(channelname).caloffset);
            fprintf(fid,['%% CalibrationUnits: %s',eol], data.(channelname).calunits);
            fprintf(fid,['%% X: %5.3f',eol],data.(channelname).x);
            fprintf(fid,['%% Y: %5.3f',eol],data.(channelname).y);
            fprintf(fid,['%% Z: %5.3f',eol],data.(channelname).z);
            fprintf(fid,['%% CalibratedStartDateTimeUTC: %s UTC',eol],channel_start_string);
            fprintf(fid,['%% CalibratedStartDateNumUTC: %.10f',eol],data.(channelname).startdatenum);
            fprintf(fid,['%%',eol]);
            fprintf(fid,['%% [Data]',eol]);
            fprintf(fid,['%.8e',eol],data.(channelname).analog);
            fclose(fid);
        end
    end
    
    % Vectrino data
    if isfield(data,'advnames') && ~isempty(data.advnames)
        Nadv = length(data.advnames);
        for nadv = 1:Nadv
            advname = data.advnames{nadv};
            advnum = cell2mat(textscan(advname,'adv%d'));
            adv_start_string = char(datetime(data.(advname).startdatenum,'ConvertFrom','datenum'),'eee yyyy/MM/dd HH:mm:ss.SSS');
            uvw = 'uvw';
            for ijk=1:3
                fname = fullfile(targetdir,[uvw(ijk),num2str(advnum),'.txt']);
                disp(['   process_raw_trial: saving data to ',fname])
                fid=fopen(fname,'w');
                fprintf(fid,'%s',data.(advname).comments);
                fprintf(fid,['%% [Calibrated Metadata]',eol]);
                fprintf(fid,['%% StillWaterDepth: %.4e',eol], data.h);
                fprintf(fid,['%% MeanSNR: %5.3f',eol],mean(mean(data.(advname).snr)));
                fprintf(fid,['%% MeanCorrelation: %5.3f',eol],mean(mean(data.(advname).corr)));
                fprintf(fid,['%% TemperatureOK: ',data.(advname).temp_ok,eol]);
                if strcmp(data.(advname).temp_ok,'NO')
                    fprintf(fid,['%% AdjustedTemperature: %5.3f',eol],data.(advname).temperature);
                    fprintf(fid,['%% AdjustedSpeedOfSound: %5.3f',eol],data.(advname).c_sound);
                end
                fprintf(fid,['%% ChannelName: %s',eol],[uvw(ijk),num2str(advnum)]);
                fprintf(fid,['%% CalibrationSlope: 1',eol]);
                fprintf(fid,['%% CalibrationOffset: 0',eol]);
                fprintf(fid,['%% CalibrationUnits: m/s',eol]);
                fprintf(fid,['%% X: %5.3f',eol],data.(advname).x);
                fprintf(fid,['%% Y: %5.3f',eol],data.(advname).y);
                fprintf(fid,['%% Z: %5.3f',eol],data.(advname).z);
                fprintf(fid,['%% CalibratedStartDateTimeUTC: %s UTC',eol],adv_start_string);
                fprintf(fid,['%% CalibratedStartDateNumUTC: %.9f',eol],data.(advname).startdatenum);
                fprintf(fid,['%%',eol]);
                fprintf(fid,['%% [Data]',eol]);
                fprintf(fid,['%.4e',eol],data.(advname).(uvw(ijk)));
                fclose(fid);
            end
            disp(['   process_raw_trial: done with ',advname])
        end
    end
end % save ASCII text files

if strcmp(options.SaveMAT,'ON')
    % save MATLAB .MAT files
    targetdir=fullfile('~','Downloads','mat');
    if ~exist(targetdir,'dir')
        mkdir(targetdir);
    end
    fname = fullfile(targetdir,[trialname,'.mat']);
    disp(['   process_raw_trial: saving data to ',fname])
    save(fname,'data');
end % save MATLAB .MAT files

if strcmp(options.SaveWaveLab,'ON')
    % save WaveLab3-compatible .DAT files
    targetdir=fullfile('~','Downloads','dat',expname);
    if ~exist(targetdir,'dir')
        mkdir(targetdir);
    end
    eol = '\r\n'; % end of line
    
    % for each group
    for ng=1:data.Ngroups
        groupchannels = []; % find all analog channels in this group
        if isfield(data,'channelnames')
            Nc = length(data.channelnames); % number of analog channels
            for nc=1:Nc
                channelname = data.channelnames{nc};
                if data.(channelname).groupnumber == ng % this channel is in the group
                    groupchannels = [groupchannels,nc]; %#ok<AGROW>
                end
            end
            Ngc = length(groupchannels); % number of analog channels in this group
        else
            Ngc = 0; % no analog channels in this group
        end
        groupadvs = []; % find all advs in this group
        if isfield(data,'advnames')
            Nadv = length(data.advnames); % number of advs
            for nadv=1:Nadv
                advname = data.advnames{nadv};
                if data.(advname).groupnumber == ng % this adv is in the group
                    groupadvs = [groupadvs,nadv]; %#ok<AGROW>
                end
            end
            Ngadv = length(groupadvs); % number of advs in this group
        else
            Ngadv = 0; % no advs in this group
        end
        N = Ngc+3*Ngadv; % number of data columns in the group
        if N>0 % if we have data for this group
            if Ngc>0 % if we have analog channels in this group
                nc = groupchannels(1); % first analog channel in this group
                channelname = data.channelnames{nc}; % name of first analog channel in this group
                M = length(data.(channelname).analog); % number of samples in this group (all members of a group have the same number of samples)
                fs = data.(channelname).fs; % sample frequency
                startdatenum = data.(channelname).startdatenum; % start time
            elseif Ngadv>0 % if we have advs in this group but no analog channels
                nadv = groupadvs(1); % first adv in this group
                advname = data.advnames{nadv}; % name of first adv in this group
                M = length(data.(advname).u); % number of samples in this group (all members of a group have the same number of samples)
                if isfield(data.(advname),'fs')
                    fs = data.(advname).fs; % sample frequency
                else
                    fs = NaN;
                end
                if isfield(data.(advname),'startdatenum')
                    startdatenum = data.(advname).startdatenum; % start time
                else
                    startdatenum = NaN;
                end
            else % huh?
                error('   process_raw_trial: data group with no analog channels and no advs')
            end
            
            % open file and write top headers
            fname = fullfile(targetdir,[trialname,'_',num2str(ng,'%02d'),'.dat']); % filename
            disp(['   process_raw_trial: saving data to ',fname])
            fid = fopen(fname,'w');
            startdatetime = datetime(startdatenum,'ConvertFrom','datenum');
            startdatestring = [char(startdatetime,'eeee'),' the ',...
                char(startdatetime,'MM/dd/yyyy'),' at ',...
                char(startdatetime,'HH:mm:ss')];
            fprintf(fid,['WaveLab 3',eol]);
            fprintf(fid,['The data is generated on ',startdatestring,eol]); % start date and time header
            fprintf(fid,['Measures includes calibration coefficient and offset',eol]);
            fprintf(fid,['Test description: Project ',projname,', Experiment ',expname,', ',trialname,eol]);
            fprintf(fid,['Water depth: %f m',eol],data.h); % water depth
            fprintf(fid,['Water depth (reference): %f m',eol],data.h); % water depth
            fprintf(fid,['Sample frequency: %f Hz',eol],fs); % sample rate
            fprintf(fid,['Sample duration: %f',eol],fs*M); % sample duration (seconds)
            fprintf(fid,eol);
            
            % build data array and write channel headers
            groupdata = zeros(M,N); % preallocate
            n=1; % group data column counter
            ngc=1; % group analog channel counter
            ngadv=1; % group adv counter
            while n<=N 
                if ngc<=Ngc
                    nc = groupchannels(ngc); % channel number
                    channelname = data.channelnames{nc}; % channel name
                    fprintf(fid,['COLUMN %d:',eol],n); % metadata header for this column
                    fprintf(fid,['---------------------------',eol]);
                    fprintf(fid,['Channel Description: ',channelname,eol]); % channel name
                    fprintf(fid,['Calibration Coefficient: 1.00 Unit/Volt',eol]); % no coefficient; we have already calibrated the data
                    fprintf(fid,['Offset: 0.00 V',eol]); % no offset
                    if isfinite(data.(channelname).x)
                        fprintf(fid,['Position (X-Coord.): %.3f m',eol],data.(channelname).x); % x
                    end
                    if isfinite(data.(channelname).y)
                        fprintf(fid,['Position (Y-Coord.): %.3f m',eol],data.(channelname).y); % y
                    end
                    if isfinite(data.(channelname).z)
                        fprintf(fid,['Position (Z-Coord.): %.3f m',eol],data.(channelname).z); % z
                    end
                    fprintf(fid,eol);
                    groupdata(:,n) = data.(channelname).analog;
                    ngc = ngc+1; % increment analog channel counter
                    n = n+1; % increment column counter
                elseif ngadv<=Ngadv
                    nadv = groupadvs(ngadv); % adv number
                    advname = data.advnames{nadv}; % adv name
                    fprintf(fid,['COLUMN %d:',eol],n); % metadata header for this column
                    fprintf(fid,['---------------------------',eol]);
                    fprintf(fid,['Channel Description: ',advname,' u',eol]); % channel name
                    fprintf(fid,['Calibration Coefficient: 1.00 Unit/Volt',eol]); % no coefficient; we have already calibrated the data
                    fprintf(fid,['Offset: 0.00 V',eol]); % no offset
                    if isfinite(data.(advname).x)
                        fprintf(fid,['Position (X-Coord.): %.3f m',eol],data.(advname).x); % x
                    end
                    if isfinite(data.(advname).y)
                        fprintf(fid,['Position (Y-Coord.): %.3f m',eol],data.(advname).y); % y
                    end
                    if isfinite(data.(advname).z)
                        fprintf(fid,['Position (Z-Coord.): %.3f m',eol],data.(advname).z); % z
                    end
                    fprintf(fid,eol);
                    fprintf(fid,['COLUMN %d:',eol],n+1); % metadata header for this column
                    fprintf(fid,['---------------------------',eol]);
                    fprintf(fid,['Channel Description: ',advname,' v',eol]); % channel name
                    fprintf(fid,['Calibration Coefficient: 1.00 Unit/Volt',eol]); % no coefficient; we have already calibrated the data
                    fprintf(fid,['Offset: 0.00 V',eol]); % no offset
                    if isfinite(data.(advname).x)
                        fprintf(fid,['Position (X-Coord.): %.3f m',eol],data.(advname).x); % x
                    end
                    if isfinite(data.(advname).y)
                        fprintf(fid,['Position (Y-Coord.): %.3f m',eol],data.(advname).y); % y
                    end
                    if isfinite(data.(advname).z)
                        fprintf(fid,['Position (Z-Coord.): %.3f m',eol],data.(advname).z); % z
                    end
                    fprintf(fid,eol);
                    fprintf(fid,['COLUMN %d:',eol],n+2); % metadata header for this column
                    fprintf(fid,['---------------------------',eol]);
                    fprintf(fid,['Channel Description: ',advname,' w',eol]); % channel name
                    fprintf(fid,['Calibration Coefficient: 1.00 Unit/Volt',eol]); % no coefficient; we have already calibrated the data
                    fprintf(fid,['Offset: 0.00 V',eol]); % no offset
                    if isfinite(data.(advname).x)
                        fprintf(fid,['Position (X-Coord.): %.3f m',eol],data.(advname).x); % x
                    end
                    if isfinite(data.(advname).y)
                        fprintf(fid,['Position (Y-Coord.): %.3f m',eol],data.(advname).y); % y
                    end
                    if isfinite(data.(advname).z)
                        fprintf(fid,['Position (Z-Coord.): %.3f m',eol],data.(advname).z); % z
                    end
                    fprintf(fid,eol);
                    groupdata(:,n) = data.(advname).u;
                    groupdata(:,n+1) = data.(advname).v;
                    groupdata(:,n+2) = data.(advname).w;
                    ngadv = ngadv+1; % increment adv channel counter
                    n = n+3; % increment column counter
                end
            end
            % write data array
            fprintf(fid,['----------------------------------------------------------',eol]);
            for m=1:M % for each row
                fprintf(fid,'%.8e',groupdata(m,1)); % first column
                for n=2:N % for each column
                    fprintf(fid,'\t'); % tab delimiter
                    fprintf(fid,'%.8e',groupdata(m,n)); % bext column
                end
                fprintf(fid,eol); % end of line before next row
            end
        end
    end
end % save WaveLab3-compatible .DAT files
end