function [t,s,eta] = cnoidal_displacement(H,T,h,duration)

% CNOIDAL_DISPLACEMENT
%
% [t,s,eta] = cnoidal_displacement(H,T,h,duration);
%
% Generates a time series 't' (s) of board stroke 's' (m) and free surface
% elevation 'eta' (m) for a 5th order cnoidal wave train of input wave
% height 'H' (m), wave period 'T' (s), and water depth 'h' (m), for the 
% specified 'duration' (s).
%
% References:
% Hughes S.A., 1993. Physical models and laboratory techniques 
% in coastal engineering. Advanced Series on Ocean Engineering, Vol. 7, 
% World Scientific Publishing, Singapore.
%
% Sobey R.J., 2012. Steep progressive waves in deep and shallow water, 
% Engineering and Computational Mechanics, 165(1), 2012.
%
% Svendsen I.A., 2006. Introduction to nearshore hydrodynamics.
% Advanced Series on Ocean Engineering, Vol. 24, World Scientific Publishing,
% Singapore.

% start
disp('cnoidal_displacement: starting')
g = gravity; % gravity
epsilon = H/h; % equation 52

% check limit
L0 = g*T^2/2/pi; % equivalent deep water wavelength
if h/L0 > 3/8/pi % Svendsen equation 9.6.56
    disp('cnoidal_displacement: cannot solve linear dispersion relation; aborting')
    t = NaN;
    s = NaN;
    eta = NaN;
    return
end

% find m
m = cnoidal_m(H,T,h);

% find solution coefficients
cn = cnoidal_coefficients(m); % Appendix C

% wavelength
i = 0:5; % indices
[K,~] = ellipke(m); % elliptic integral of 1st kind
alpha = sqrt(sum(cn.Fi.*epsilon.^i)); % equation 51
L = 2*K*h/alpha; % equation 50

% phase speed
Q = sqrt(g*h^3)*sum(cn.Ei.*epsilon.^i); % equation 49
c = Q/h; % equation 16

% check Ursell number
U = H*L^2/h^3; % Svendsen equation 7.5.13, among others
disp(['cnoidal_displacement: Ursell number, U=',num2str(U)])
if U < 40
    disp('cnoidal_displacement: warning, Ursell number less than 40!')
end

% cnoidal wave free surface function handle
f_eta = @cnoidal_wave;

% shallow water wavemaker equation, Hughes equation 7.145
% s' = c*eta/(h+eta)
disp('cnoidal_displacement: solving shallow water wavemaker equation')
f_dsdt = @(t,s) c*f_eta(t,s,H,h,m)./(h+f_eta(t,s,H,h,m)); 

% solving 1/2 period in two directions
f_min = 50; % at least 50 Hz
n = 2^nextpow2(T/2*f_min); % number of timesteps for 1/2 period
dt = T/2/n; % timestep

% forward from 0 to T/2
tspan = linspace(0,T/2,n+1);
s0 = 0; % initial stroke is 0
options = odeset('RelTol',1e-9,'AbsTol',1e-12); % require higher accuracy
[t_f,s_f] = ode45(f_dsdt,tspan,s0,options);

% solve backward from T/2 to 0
tspan = linspace(T/2,0,n+1); % running time backwards
s0 = 0; % initial stroke is 0
options = odeset('RelTol',1e-9,'AbsTol',1e-12); % require higher accuracy
[~,s_b] = ode45(f_dsdt,tspan,s0,options);

% merge forward and backward solutions to eliminate drift in board stroke
% result is a solution from 0 to T/2
coeff = 0.5*(cos(2*pi*t_f/T)+1);
s_m = s_f.*coeff + s_b(end:-1:1).*(1-coeff); % ensure start and end at 0

% flip, negate, concatenate to run solution from 0 to T-dt
t = [t_f;t_f(end)+t_f(2:end-1)];
s = [s_m;-s_m(end-1:-1:2)];

% now solve for eta
disp('cnoidal_displacement: solving for eta at paddle')
eta = cnoidal_wave(t,s,H,h,m);

% pad out to full duration
t = (0:dt:duration);
s = padarray(s,length(t) - length(s),'circular','post')';
eta = padarray(eta,length(t) - length(eta),'circular','post')';

% done
disp('cnoidal_displacement: done')