function m = cnoidal_m(H,T,h)

% CNOIDAL_M
%
% m = cnoidal_m(H,T,h);
%
% Finds elliptic parameter 'm' for a 5th order cnoidal wave of input
% wave height 'H' (m), wave period 'T' (s), and water depth 'h' (m).
%
% Reference:
% Sobey R.J., 2012, Steep progressive waves in deep and shallow water, 
% Engineering and Computational Mechanics, 165(1).

% build arrays of m and z
m_array = build_m_array;
z_array = zeros(size(m_array));
for i=1:length(m_array)
    z_array(i) = z_cn_m(m_array(i),H,T,h);
end

% bracket zero-crossing of z closest to m=1
zero_crossing_index = find(z_array(1:end-1).*z_array(2:end) < 0,1,'last');
m1 = m_array(zero_crossing_index);
m2 = m_array(zero_crossing_index+1);

% solve cn_m for m using m1 and m2
m = fzero(@(m) z_cn_m(m,H,T,h),[m1,m2]);
end

function z = z_cn_m(m,H,T,h)
% Z_CN_M
%
% z = z_cn_m(m,H,T,h)
%
% Used to find  elliptic parameter 'm' for a 5th-order cnoidal wave of
% input  wave height 'H' (m), wave period 'T' (s), and water depth 'h' (m).
%
% Reference:
% Sobey R.J., 2012. Steep progressive waves in deep and shallow water, 
% Engineering and Computational Mechanics, 165(1), 2012.

cn = cnoidal_coefficients(m); % Appendix C
g = gravity; % gravity
epsilon = H/h; % equation 52
i = 0:5;
[K,~] = ellipke(m); % elliptic integral of 1st kind
alpha = sqrt(sum(cn.Fi.*epsilon.^i)); % equation 51
L = 2*K*h/alpha; % equation 50
Q = sqrt(g*h^3)*sum(cn.Ei.*epsilon.^i); % equation 49
z = L/T - Q/h; % equation 68
end

function m_array = build_m_array
% build array of m-values from 1e-15 to almost 1e-3
m_array = 0;
m_start = 1e-15;
for p=15:-1:4
    dm = 0.1^p;
    m_end = 9*dm;
    m_array = [m_array,m_start:dm:m_end]; %#ok<AGROW>
    m_start = m_end + dm;
end

% append array from 0.001 to 0.999
m_array = [m_array,0.001:0.001:0.999];

% append array of m-values from 0.9991 to 1-1e-15
m_end = 0.999;
for p=4:15
    dm = 0.1^p;
    m_start = m_end + dm;
    m_end = 1-dm;
    m_array = [m_array,m_start:dm:m_end]; %#ok<AGROW>
end

% done
end
