function eta = cnoidal_wave(t,x,H,h,m)

% CNOIDAL_WAVE
%
% eta = cnoidal_wave(t,x,H,h,m);
%
% 5th order cnoidal wave free surface elevation timeseries, as a function 
% of time 't' (s), cross-shore position 'x' (m), wave height 'H' (m), water
% depth 'h' (m) and elliptic parameter 'm'.
%
% References: 
% Sobey R.J., 2012. Steep progressive waves in deep and shallow water, 
% Engineering and Computational Mechanics, 165(1), 2012.
%
% Svendsen I.A., 2006. Introduction to nearshore hydrodynamics.
% Advanced Series on Ocean Engineering, Vol. 24, World Scientific Publishing,
% Singapore.
%
% SEE ALSO: CNOIDAL_DISPLACEMENT

% find solution coefficients
cn = cnoidal_coefficients(m); % Appendix C

% phase speed
g = gravity; % gravity
epsilon = H/h; % equation 52
i = 0:5; % indices
Q = sqrt(g*h^3)*sum(cn.Ei.*epsilon.^i); % equation 49
c = Q/h; % equation 13, no mass transport

% Cn
alpha = sqrt(sum(cn.Fi.*epsilon.^i)); % equation 51
k = alpha/h; % equation 43
chi = k*(x - c*t);
[~,Cn,~] = ellipj(chi,m); % Cn from Jacobian elliptic function

eta = zeros(size(Cn));
for i = 0:5
    for j = 0:i
        eta = eta + h*epsilon^i*cn.Bij(i+1,j+1)*Cn.^(2*j); % equation 46
    end
end
    