function k = dispersion_e(h,omega,N_evanescent)

% DISPERSION_E
%
% k = dispersion_e(h,omega,N_evanescent);
%
% Numerically solves the linear dispersion relationship for evanescent
% water waves: omega^2=-gk*tan(kh); where omega = 2*pi/T; k=2*pi/L; g is
% gravity; omega (scalar, vector, or matrix) and h (scalar) provided as inputs.
%
% Iterates using Halley's method initialized with a brute-force guess.
% Uses a nondimensional formulation by Soulsby (2006) where x=omega.^2*h/g;
% y=k*h; therefore x = -y * tan(y)
%
% Reference:
% Soulsby R.L., 2006, Simplified calculation of wave orbital velocities,
% TR-155. HR Wallingford Ltd., Wallingford, 12pp.+figures.
%
% See also: DISPERSION_COMPLEX, DISPERSION_P

% setup
g = gravity;
om = repmat(omega,N_evanescent,1);
x = om.^2*h/g; % nondimensional frequency

% initial guess
N_f = length(omega);
y = zeros(N_evanescent,N_f);
N_guess = 6;
d_guess = pi/2/(N_guess-1);
base_range = pi/2:d_guess:pi; base_range = base_range(2:end-1);
N_guess = 51;
d_guess = pi/2/(N_guess-1);
br1 = pi/2:d_guess:base_range(1); br1 = br1(1:end-1);
br2 = base_range(end):d_guess:pi; br2 = br2(2:end);
base_range = [br1,base_range,br2]; base_range = base_range(2:end-1);
N_guess = 501;
d_guess = pi/2/(N_guess-1);
br1 = pi/2:d_guess:base_range(1); br1 = br1(1:end-1);
br2 = base_range(end):d_guess:pi; br2 = br2(2:end);
base_range = [br1,base_range,br2]; base_range = base_range(2:end-1);
for i=1:N_f
    for j=1:N_evanescent
        % range between y = j*pi - pi/2 and y = j*pi
        y_range = base_range+(j-1)*pi;
        
        % find the minimum of f=x+y*tan(y)
        [~,guess_index] = min(abs(x(j,i) + y_range.*tan(y_range)));
        
        % that's our guess for y
        y(j,i) = y_range(guess_index);
    end
end

% iterate
t = tan(y);
f = x + y.*t;
dfdy = t + y.*(1+t.^2);
d2fdy2 = 2*(1+y.*t).*(1+t.^2);
y1 = y - 2*f.*dfdy./(2*dfdy.^2 - f.*d2fdy2);
dy = y1-y;
y = y1;
count = 1;
while any(abs(dy(:))>eps(y(:))) && any(abs(f(:))>eps(x(:))) && count < 10
    t = tan(y);
    f = x + y.*t;
    dfdy = t + y.*(1+t.^2);
    d2fdy2 = 2*(1+y.*t).*(1+t.^2);
    y1 = y - 2*f.*dfdy./(2*dfdy.^2 - f.*d2fdy2);
    dy = y1-y;
    y = y1;
    count = count+1;
end

% done
y(~x) = 0; % handle any zero frequencies
k = y/h; % y = kh
end
