function k = dispersion_p(h,omega)

% DISPERSION_P
%
% k = dispersion_p(h,omega);
%
% Numerically solves the linear dispersion relationship for progressive
% water waves: omega^2=gk*tanh(kh); where omega = 2*pi/T; k=2*pi/L; 
% g is gravity; omega (scalar or vector) and h (scalar) provided as inputs.
%
% Iterates using Newton-Raphson. Uses a nondimensional formulation by 
% Soulsby (2006) where x=omega.^2*h/g; y=k*h; therefore x = y * tanh(y)
%
% Reference:
% Soulsby R.L., 2006, Simplified calculation of wave orbital velocities,
% TR-155. HR Wallingford Ltd., Wallingford, 12pp.+figures.
%
% See also: DISPERSION_COMPLEX, DISPERSION_E

% setup
g = gravity; % gravity
x = omega.^2*h/g; % nondimensional frequency

% initial guess at y=kh
y = sqrt(x).*(x<1) + x.*(x>=1);

% iterate
t = tanh(y);
f = y.*t - x;
dfdy = t + y.*(1-t.^2);
y1 = y - f./dfdy;
dy = y1-y;
y = y1;
count = 1;
while any(abs(dy)>3*eps(y)) && any(abs(f)>3*eps(x)) && count < 10
    t = tanh(y);
    f = y.*t - x;
    dfdy = t + y.*(1-t.^2);
    y1 = y - f./dfdy;
    dy = y1-y;
    y = y1;
    count = count + 1;
end

% done
y(~x) = 0; % handle any zero frequencies
k = y/h; % y = kh
end