function spec = generate_spectrum(conditions,varargin)

% GENERATE_SPECTRUM
%
% spec_out = generate_spectrum(conditions);
% spec_out = generate_spectrum(conditions,options);
% spec_out = generate_spectrum(conditions,options,spec_in);
%
% Generates a spectrum structure (spec_out) from the input structures
% 'conditions', 'options', and 'spec_in'. The conditions structure has
% fields as follows to define the desired spectrum:
%
% wave_type (-). Can be 'CHIRP','CUSTOM','FRF','JS','PINK','PM','TMA','WHITE'.
%                Also 'UPCHIRP' or 'DOWNCHIRP' to define chirp_direction.
% H (m). Wave height (optional). For all but 'CHIRP' this is spectral 
%        significant wave height (Hm0), and for 'CHIRP' this is the height 
%        of the waves (H). Ignored for 'CUSTOM'.
% T (s). Wave period(s) (optional). For 'FRF','JS','PM','TMA" this is spectral 
%        peak wave period (Tp) and for 'CHIRP','PINK','WHITE' this has two 
%        values that define the bounds of the desired spectrum. Ignored for
%        'CUSTOM'.
% h (s). Still water depth at the wavemaker paddle(s).
% duration (s). Length of time series to be generated. Defaults to 600s.
% gm (-). Spectral shape factor for 'FRF','JS' or 'TMA'. Defaults to 3.3.
% chirp_direction (-). Frequency shifts 'UP' or 'DOWN' from start to end. 
%                      Defaults to 'UP'. Specifiable in wave_type above.
% eta (m). Free surface elevation. Required for 'CUSTOM'. Ignored otherwise.
% t (s). Time in seconds. Optional for 'CUSTOM'. Ignored otherwise.
% dt (s). Timestep in seconds. Optional for 'CUSTOM'. Ignored otherwise. 
%
% The 'options' structure should be defined using GENERATE_SPECTRUM_OPTIONS
% with fields designed to treat the input structure. Options default to a
% newly-generated spectrum with random phases and angles. Options are
% ignored for 'CHIRP' and 'CUSTOM' wave types.
%
% Generates a spectrum structure (spec_out) of the specified type with the 
% input fields from 'conditions' and input values from 'spec_in' where 
% appropriate, such as:
% A (m). Complex spectral amplitudes, A=sqrt(2*S*df).*exp(1i*phi)
% phi (rad). Spectral phases used to form A
% S (m^2/Hz). Power spectral density used to form A
% f (Hz). Frequencies
% omega (rad/s). Angular frequencies, omega=2*pi*f
% df (Hz). Frequency resolution
% Nf (-). Number of frequencies
% nf (-). Frequency indexes used for second-order wavemaker theory
%
% Assumes SI units and a piston wavemaker.
%
% References:
% Bouws E. et al, Similarity of the wind wave spectrum in finite depth
% water 1. Spectral form, 1985, Journal of Geophysical Research, V 90, N
% C1, pp. 975-986.
% 
% Goda, 2010, Random Seas and Design of Maritime Structures, 3rd Ed., World
% Scientific Advanced Series on Ocean Engineering, Vol. 33, p.474.
%
% H. Mitsuyasu, 1975, Observations of the directional spectrum of ocean
% waves using a cloverleaf buoy, Journal of Physical Oceanography, v5, 750-760.
%
% Schaffer H.A., 1996, Second-order wavemaker theory for irregular waves,
% Ocean Engineering, v23, n1, pp. 47-88.
%
% Stanserg C.T., The specialist committee on waves final report and
% recommendations to the 23rd ITTC, 2002, Proceedings of the 23rd
% Interational Towing Tank Conference, Volume II.
%
% SEE ALSO: DISPERSION_P, WAVEMAKER, WAVEMAKER_NL

disp('generate_spectrum: starting')
% handle inputs
switch nargin
    case 1
        % no options, no input spectrum
        options = generate_spectrum_options;
        spec = generate_blank_spectrum(conditions);
    case 2
        % no input spectrum
        options = varargin{1};
        spec = generate_blank_spectrum(conditions);
    otherwise
        options = varargin{1};
        spec = varargin{2};
end
conditions.wave_type(~isstrprop(conditions.wave_type,'alpha')) = ''; % strip non-alphabet
conditions.wave_type = upper(conditions.wave_type); % force upper-case
disp(['generate_spectrum: wave type ',conditions.wave_type])
if isfield(conditions,'H')
    disp(['generate_spectrum: H=',num2str(conditions.H),' m'])
end
if isfield(conditions,'T')
    Tstring = '';
    for n=1:length(conditions.T)
        if n>1
            Tstring = [Tstring,',']; %#ok<AGROW>
        end
        Tstring = [Tstring,num2str(conditions.T(n))]; %#ok<AGROW>
    end
    disp(['generate_spectrum: T=',Tstring,' s'])
end
disp(['generate_spectrum: h=',num2str(conditions.h),'m'])

% check to see if this is a new spectrum
if ~isfield(conditions,'duration')
    conditions.duration = 600; % default
end
if options.new
    new_spectrum = 1; % we've chosen an entirely new spectrum
elseif any(strcmp(conditions.wave_type,{'CHIRP','CUSTOM','DOWNCHIRP','UPCHIRP'}))
    new_spectrum = 1; % chirps and custom are always new
elseif (conditions.T ~= spec.T)
    new_spectrum = 1; % we've changed the peak period
elseif (conditions.h ~= spec.h)
    new_spectrum = 1; % we've changed the water depth
elseif (conditions.duration ~= spec.duration)
    new_spectrum = 1; % we've changed the duration
elseif ~strcmp(conditions.wave_type,spec.wave_type) % changing wave type
    if any(strcmp(conditions.wave_type,{'FRF','JS','PM','TMA'})) && ...
            any(strcmp(spec.wave_type,{'FRF','JS','PM','TMA'}))
        new_spectrum = 0; % we've changed among FRF/JS/PM/TMA
    elseif any(strcmp(conditions.wave_type,{'PINK','WHITE'})) && ...
            any(strcmp(spec.wave_type,{'PINK','WHITE'}))
        new_spectrum = 0; % we've changed between WHITE and PINK
    else % we've changed but neither among FRF/JS/PM/TMA nor among WHITE/PINK
        new_spectrum = 1;
    end
else
    new_spectrum = 0; % we are reusing the spectrum and haven't hit an exception
end

% build or rebuild unidirectional spectrum
if new_spectrum 
    % generate new spectrum from scratch
    switch conditions.wave_type
        case {'CHIRP','UPCHIRP','DOWNCHIRP'}
            if strcmp(conditions.wave_type,'UPCHIRP')
                conditions.chirp_direction = 'UP'; % this forces a low to high chirp
                conditions.wave_type = 'CHIRP'; % reset to chirp
            elseif strcmp(conditions.wave_type,'DOWNCHIRP')
                conditions.chirp_direction = 'DOWN'; % this forces a high to low chirp
                conditions.wave_type = 'CHIRP'; % reset to chirp
            else
                if ~isfield(conditions,'chirp_direction')
                    conditions.chirp_direction = 'UP'; % default chirp from low to high
                else
                    conditions.chirp_direction = upper(conditions.chirp_direction); % force upper-case
                end
            end
            spec = generate_chirp_spectrum(conditions);
        case 'CUSTOM'
            spec = generate_custom_spectrum(conditions);
        case 'FRF'
            spec = generate_frf_spectrum(conditions);
        case 'JS'
            spec = generate_jonswap_spectrum(conditions);
        case 'PINK'
            spec = generate_pink_spectrum(conditions);
        case 'PM'
            spec = generate_pm_spectrum(conditions);
        case 'TMA'
            spec = generate_tma_spectrum(conditions);
        case 'WHITE'
            spec = generate_white_spectrum(conditions);
    end
    
    % generate new wave angles from scratch
    if isfield(conditions,'alpha_mean') || isfield(conditions,'spread_coef')
        if isfield(conditions,'alpha_mean')
            alpha_mean = conditions.alpha_mean; % mean wave angle
        else
            alpha_mean = 0;
        end
        alpha_max = pi/3; % maximum angle of ±60°
        if abs(alpha_mean) > alpha_max % check that alpha_mean is not too large
            disp(['generate_spectrum: input wave angle of ',num2str(alpha_mean*180/pi),'° exceeds max angle of 60°'])
            alpha_mean = sign(alpha_mean)*alpha_max;
            disp(['generate_spectrum: mean angle reduced to',num2str(alpha_mean*180/pi),'°'])
        end
        if isfield(conditions,'spread_coef')
            % randomly generated wave angle for each frequency
            spread_coef = conditions.spread_coef; % directional spread coefficient
            disp('generate_spectrum: generating spread angles')
            alpha = generate_wave_angles(spec.Nf,alpha_mean,spread_coef);
            y_board = dwb_wavemaker_drivepoints; % wavemaker paddle locations
        else % only mean angle for each frequency
            spread_coef = 0; % 0 spreading
            if alpha_mean == 0
                alpha = zeros(size(spec.f)); % zero wave angle for every frequency
                y_board = 0; % only 1 board position
            else
                alpha = alpha_mean*ones(size(spec.f)); % alpha_mean wave angle for every frequency
                y_board = dwb_wavemaker_drivepoints; % wavemaker paddle locations
            end
        end
    else
        alpha_mean = 0; % 0 mean angle
        spread_coef = 0; % 0 spreading
        alpha = zeros(size(spec.f)); % zero wave angle for every frequency
        y_board = 0; % only 1 board position
    end
    spec.alpha = alpha;
    spec.alpha_mean = alpha_mean;
    spec.spread_coef = spread_coef;
    spec.y_board = y_board;
else % reuse existing spectrum
    spec.new = 0; % spectrum is reused
    if options.shuffle % generate new phases or angles
        if options.shuffle_phases
            S = spec.S; % spectral density
            df = spec.df; % frequency resolution
            phi = 2*pi*rand(size(S)); % random phases
            A = sqrt(2*S*df).*exp(1i*phi); % complex A
            spec.phi = phi; % save new phases
            spec.A = A; % save new complex A
        end
        if options.shuffle_angles
            if isfield(conditions,'alpha_mean')
                alpha_mean = conditions.alpha_mean; % new mean wave angle
            else
                alpha_mean = 0; % zero mean wave angle
            end
            alpha_max = pi/3; % maximum angle of ±60°
            if abs(alpha_mean) > alpha_max % check that alpha_mean is not too large
                disp(['generate_spectrum: input wave angle of ',num2str(alpha_mean*180/pi),'° exceeds max angle of 60°'])
                alpha_mean = sign(alpha_mean)*alpha_max;
                disp(['generate_spectrum: mean angle reduced to',num2str(alpha_mean*180/pi),'°'])
            end
            if isfield(conditions,'spread_coef')
                % randomly generated wave angle for each frequency
                spread_coef = conditions.spread_coef; % directional spread coefficient
                disp('generate_spectrum: generating spread angles')
                alpha = generate_wave_angles(spec.Nf,alpha_mean,spread_coef);
            else
                % only mean angle for each frequency
                if alpha_mean == 0
                    alpha = zeros(size(spec.f)); % zero wave angle for every frequency
                    y_board = 0; % only 1 board position
                else
                    alpha = alpha_mean*ones(size(spec.f)); % alpha_mean wave angle for every frequency
                    y_board = dwb_wavemaker_drivepoints; % wavemaker paddle locations
                end
            end
            if all(alpha == spec.alpha) && all(y_board == spec.y_board)
                spec.new = 0; % directions unchanged
            else
                spec.new = 1; % spectrum is reused but direction changes requre rerun of FG_plus/minus in wm_core_nl
                spec.alpha = alpha;
                spec.alpha_mean = alpha_mean;
                spec.spread_coef = spread_coef;
                spec.y_board = y_board;
            end
        end
    end
    if ~strcmp(conditions.wave_type,spec.wave_type) % changing wave type
        h = spec.h; % water depth
        g = gravity; % gravity
        f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)
        f = spec.f; % frequency
        H = conditions.H; % wave height
        switch conditions.wave_type
            case 'FRF'
                fp = 1/spec.T;
                if ~isfield(conditions,'gm')
                    conditions.gm = 3.3;
                end
                gm = conditions.gm; % spectral shape factor
                S = frf_core(f,fp,H,h,gm); % FRF spectrum
            case 'JS'
                fp = 1/spec.T;
                if ~isfield(conditions,'gm')
                    conditions.gm = 3.3;
                end
                gm = conditions.gm; % spectral shape factor
                S = js_core(f,fp,H,gm); % JS spectrum
            case 'PM'
                fp = 1/spec.T; % peak frequency
                S = pm_core(f,fp,H); % Pierson-Moskowitz spectrum
            case 'TMA'
                fp = 1/spec.T;
                if ~isfield(conditions,'gm')
                    conditions.gm = 3.3;
                end
                gm = conditions.gm; % spectral shape factor
                S = tma_core(f,fp,H,h,gm); % TMA spectrum
            case {'PINK','WHITE'}
                S = zeros(size(f)); % preallocate spectrum with zeros
                S(nf) = 1; % nf bins are the band of white noise
                if strcmp(conditions.wave_type,'PINK')
                    S(2:end) = S(2:end)./f(2:end); % apply 1/f to white noise to get pink noise
                end
        end
        S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
        m0 = trapz(f,S); % Goda 2.29
        S = S*H^2/4.004^2/m0; % Goda 2.40
        spec.S = S; % save modified spectral density
        spec.H = H; % save wave height
        df = spec.df; % frequency resolution
        phi = spec.phi; % wave phases
        A = sqrt(2*S*df).*exp(1i*phi); % modified complex A
        spec.A = A; % save modified complex A
    elseif (conditions.H ~= spec.H) % keeping wave type but changing wave height
        sf = conditions.H^2/spec.H^2; % rescale factor
        S = spec.S; % spectral density
        df = spec.df; % frequency resolution
        phi = spec.phi; % wave phases
        S = S .* sf^2; % apply rescale to spectral density
        spec.S = S; % save modified spectral density
        spec.H = conditions.H; % save modified wave height
        A = sqrt(2*S*df).*exp(1i*phi); % modified complex A
        spec.A = A; % save modified complex A
    end
end

% done
disp('generate_spectrum: done')
end

function y_board = dwb_wavemaker_drivepoints
% Returns the y locations of the drive points for the DWB wavemaker
% paddles.  Observations collected in 2008 using a measuring tape.
y_board = [-13.2;-12.345;-11.426;-10.515;-9.6;-8.685;-7.77;-6.855; ...
    -5.945;-5.03;-4.11;-3.2;-2.285;-1.37;-0.455;0.46;1.375;2.29;3.2; ...
    4.12;5.03;5.95;6.86;7.78;8.69;9.605;10.52;11.44;12.35;13.195];
end

function spec = generate_blank_spectrum(conditions)
% generates a blank (baseline) spectrum with all fields filled in based on
% the input conditions
spec.new = 1;
spec.wave_type = conditions.wave_type;
if isfield(conditions,'H')
    spec.H = conditions.H;
end
if isfield(conditions,'T')
    spec.T = conditions.T;
end
if isfield(conditions,'h')
    spec.h = conditions.h;
end
if isfield(conditions,'duration')
    spec.duration = conditions.duration;
end
if isfield(conditions,'gm')
    spec.gm = conditions.gm;
end
if isfield(conditions,'chirp_direction')
    spec.chirp_direction = conditions.chirp_direction;
end
if isfield(conditions,'eta')
    spec.eta = conditions.eta;
end
if isfield(conditions,'t')
    spec.t = conditions.t;
end
if isfield(conditions,'dt')
    spec.dt = conditions.dt;
end
end

function spec = generate_chirp_spectrum(conditions)
% Chirp spectrum

% setup
H = conditions.H;
T = conditions.T;
h = conditions.h;
if isfield(conditions,'duration')
    duration = conditions.duration;
else
    duration = 600; % default
end
chirp_direction = conditions.chirp_direction;

% build frequency and time
f_max = 1/min(T); % highest frequency; lowest period
f_min = 1/max(T); % lowest frequency; highest period
if strcmp(chirp_direction,'UP')
    % UP is from low frequency to high frequency sweep
    f0 = f_min;
    f1 = f_max;
else
    % DOWN is from high frequency to low frequency sweep
    f0 = f_max;
    f1 = f_min;
end
fN_min = 2*f_max; % at least 2x the maximum frequency
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,5); % at least 5Hz
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
f = linspace(0,fN,Nf); % frequencies
n_min = find(f>=f_min,1,'first');
if isempty(n_min)
    n_min = 2; % throwing out the 0 frequency bin
end
n_max = find(f<=f_max,1,'last');
if isempty(n_max)
    n_max = Nf;
end
nf = n_min:n_max; % frequency bins used in second order analysis
N = 2*(Nf-1); % number of timesteps
t = linspace(0,duration,N+1); % time

% time domain chirp (progressive component of free surface)
r = 20; % 20 second hold duration
t0 = r; % hold for r seconds at f=f0
t1 = duration-r; % hold for r seconds at at f=f1;
eta_p = H/2*chirp_core(t,t0,t1,f0,f1); % chirp

% fft to get amplitudes
Z = fft(eta_p(1:end-1)); % fft
A = 2*Z(1:Nf)/N; A(Nf) = A(Nf)/2; % complex amplitude
phi = angle(A); % phases
S = abs(A).^2/2/df; % spectral density

% taper spectrum between f_ev and 2*f_ev
S = taper_core(S,f,f_ev);

% regenerate complex spectrum with original phases
A = sqrt(2*S*df).*exp(1i*phi); % tapered complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.H = H;
spec.T = T;
spec.h = h;
spec.duration = duration;
spec.chirp_direction = chirp_direction;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function spec = generate_custom_spectrum(conditions)
% custom spectrum from a user-specified timeseries

% setup
h = conditions.h; % water depth
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)
input_eta = conditions.eta;  % free surface elevation
if isfield(conditions,'t')
    input_t = conditions.t - conditions.t(1); % time
    duration = input_t(end); % duration of timeseries
    dt = input_t(2); % timestep
elseif isfield(conditions,'dt')
    dt = conditions.dt; % timestep
    N = length(input_eta) - 1; % number of timesteps
    duration = N*dt; % duration of timeseries
    input_t = linspace(0,duration,N+1); % time
elseif isfield(conditions,'duration')
    duration = conditions.duration; % duration of timeseries
    N = length(input_eta) - 1; % number of timesteps
    input_t = linspace(0,duration,N+1); % time
    dt = input_t(2); % timestep
end
input_fs = 1/dt; % sampling frequency

% build frequency and time
fN_min = min(input_fs,8); % resolve at least input_fs but no more than 8Hz
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,2); % at least 2Hz
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
fs = 2*fN; % sampling frequency
f = linspace(0,fN,Nf); % frequencies
N = 2*(Nf-1); % number of timesteps
t = linspace(0,duration,N+1); % time

% get eta as a function of t by resampling input_eta
[p,q] = rat(fs/input_fs/2); % nearly ensuring that fs*q/p > 2*input_fs
eta_r = resample(input_eta,input_t,fs,p,q,'spline');

% fft to get amplitudes
Z = fft(eta_r(1:end-1)); % fft
A = 2*Z(1:Nf)/N; A(Nf) = A(Nf)/2; % complex amplitude
phi = angle(A); % phases
S = abs(A).^2/2/df; % spectral density
m0 = trapz(f,S); % Goda 2.29

% frequency bins used in second order analysis
nf = second_order_bins(f,S); 

% taper and rescale
S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
m0_taper = trapz(f,S); % Goda 2.29
S = S*m0/m0_taper; % rescale S to match original m0
   
% regenerate complex spectrum
A = sqrt(2*S*df).*exp(1i*phi); % modified complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.eta = eta_r;
spec.t = t;
spec.h = h;
spec.duration = duration;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function spec = generate_frf_spectrum(conditions)
% FRF spectrum

% setup
H = conditions.H;
T = conditions.T;
h = conditions.h;
duration = conditions.duration;
if ~isfield(conditions,'gm')
    conditions.gm = 3.3; % default gm
end
gm = conditions.gm; disp(['generate_spectrum: gm=',num2str(gm)])
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)

% build ideal FRF spectrum
f = linspace(0,16,2^21+1); % 2^21+1 frequencies in 0-16 Hz range, df<1e-5
df = f(2); % frequency resolution
fp = 1/T; % desired peak frequency
fp_input = fp; % input peak frequency
S = frf_core(f,fp_input,H,h,gm); % FRF spectrum
[~,I] = max(S); fp_output = f(I); % output peak frequency
while abs(fp-fp_output)>df
    fp_input = fp_input*fp/fp_output; % new input peak frequency
    S = frf_core(f,fp_input,H,h,gm); % FRF spectrum
    [~,I] = max(S); fp_output = f(I); % new output peak frequency
end

% find highest frequency we want to resolve
Sp = max(S); % energy density at spectral peak
fN_min = 2*f(find(S>=Sp*0.01,1,'last')); % at least 2x highest frequency with energy at least 1% of the peak
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,2); % at least 2Hz

% build spectrum
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
f = linspace(0,fN,Nf); % frequencies
S = frf_core(f,fp_input,H,h,gm); % FRF spectrum

% frequency bins used in second order analysis
nf = second_order_bins(f,S); 

% taper and rescale
S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
m0 = trapz(f,S); % Goda 2.29
S = S*H^2/4.004^2/m0; % Goda 2.40

% generate complex spectrum with random phases
phi = 2*pi*rand(size(S)); % random phases
A = sqrt(2*S*df).*exp(1i*phi); % complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.H = H;
spec.T = T;
spec.h = h;
spec.duration = duration;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function spec = generate_jonswap_spectrum(conditions)
% JONSWAP spectrum

% setup
H = conditions.H;
T = conditions.T;
h = conditions.h;
duration = conditions.duration;
if ~isfield(conditions,'gm')
    conditions.gm = 3.3; % default gm
end
gm = conditions.gm; disp(['generate_spectrum: gm=',num2str(gm)])
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)

% build ideal JONSWAP spectrum
f = linspace(0,16,2^21+1); % 2^21+1 frequencies in 0-16 Hz range, df<1e-5
fp = 1/T; % peak frequency
S = js_core(f,fp,H,gm); % JONSWAP spectrum
Sp = max(S); % peak energy

% find highest frequency we want to resolve
fN_min = 2*f(find(S>=Sp*0.01,1,'last')); % at least 2x highest frequency with energy at least 1% of the peak
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,2); % at least 2Hz

% build spectrum
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
f = linspace(0,fN,Nf); % frequencies
fp = 1/T; % peak frequency
S = js_core(f,fp,H,gm); % JONSWAP spectrum

% frequency bins used in second order analysis
nf = second_order_bins(f,S); 

% taper and rescale
S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
m0 = trapz(f,S); % Goda 2.29
S = S*H^2/4.004^2/m0; % Goda 2.40

% generate complex spectrum with random phases
phi = 2*pi*rand(size(S)); % random phases
A = sqrt(2*S*df).*exp(1i*phi); % complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.H = H;
spec.T = T;
spec.h = h;
spec.duration = duration;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function spec = generate_pink_spectrum(conditions)
% Band-limited pink noise spectrum

% first generate white spectrum
spec = generate_white_spectrum(conditions);
S = spec.S;
f = spec.f;
H = spec.H;

% apply 1/f to white noise to get pink noise
S(2:end) = S(2:end)./f(2:end);

% rescale S to force m0=H^2/16 (H=4.004*sqrt(m0))
S = S .* H^2 / 4.004^2 / trapz(f,S);

% regenerate complex spectrum with random phases
phi = spec.phi; % original random phases
df = spec.df; % original spectral resolution
A = sqrt(2*S*df).*exp(1i*phi); % modified complex A

% rebuild spec structure
spec.A = A;
spec.S = S;
end

function spec = generate_pm_spectrum(conditions)
% Pierson-Moskowitz (aka Bretschnieder) spectrum

% setup
H = conditions.H;
T = conditions.T;
h = conditions.h;
duration = conditions.duration;
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)

% build ideal Pierson-Moskowitz spectrum
f = linspace(0,16,2^21+1); % 2^21+1 frequencies in 0-16 Hz range, df<1e-5
fp = 1/T; % peak frequency
S = pm_core(f,fp,H); % Pierson-Moskowitz spectrum
Sp = max(S); % peak energy

% find highest frequency we want to resolve
fN_min = 2*f(find(S>=Sp*0.01,1,'last')); % at least 2x highest frequency with energy at least 1% of the peak
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,2); % at least 2Hz

% build spectrum
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
f = linspace(0,fN,Nf); % frequencies
fp = 1/T; % peak frequency
S = pm_core(f,fp,H); % Pierson-Moskowitz spectrum

% frequency bins used in second order analysis
nf = second_order_bins(f,S); 

% taper and rescale
S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
m0 = trapz(f,S); % Goda 2.29
S = S*H^2/4.004^2/m0; % Goda 2.40

% generate complex spectrum with random phases
phi = 2*pi*rand(size(S)); % random phases
A = sqrt(2*S*df).*exp(1i*phi); % complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.H = H;
spec.T = T;
spec.h = h;
spec.duration = duration;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function spec = generate_tma_spectrum(conditions)
% TMA (TEXEL, MARSLEN, ARSOE) spectrum

% setup
H = conditions.H;
T = conditions.T;
h = conditions.h;
duration = conditions.duration;
if ~isfield(conditions,'gm')
    conditions.gm = 3.3; % default gm
end
gm = conditions.gm; disp(['generate_spectrum: gm=',num2str(gm)])
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)

% build ideal TMA spectrum
f = linspace(0,16,2^21+1); % 2^21+1 frequencies in 0-16 Hz range, df<1e-5
df = f(2); % frequency resolution
fp = 1/T; % desired peak frequency
fp_input = fp; % input peak frequency
S = tma_core(f,fp_input,H,h,gm); % TMA spectrum
[~,I] = max(S); fp_output = f(I); % output peak frequency
while abs(fp-fp_output)>df
    fp_input = fp_input*fp/fp_output; % new input peak frequency
    S = tma_core(f,fp_input,H,h,gm); % TMA spectrum
    [~,I] = max(S); fp_output = f(I); % new output peak frequency
end

% find highest frequency we want to resolve
Sp = max(S); % energy density at spectral peak
fN_min = 2*f(find(S>=Sp*0.01,1,'last')); % at least 2x highest frequency with energy at least 1% of the peak
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,2); % at least 2Hz

% build spectrum
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
f = linspace(0,fN,Nf); % frequencies
S = tma_core(f,fp_input,H,h,gm); % TMA spectrum

% frequency bins used in second order analysis
nf = second_order_bins(f,S); 

% taper and rescale
S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
m0 = trapz(f,S); % Goda 2.29
S = S*H^2/4.004^2/m0; % Goda 2.40

% generate complex spectrum with random phases
phi = 2*pi*rand(size(S)); % random phases
A = sqrt(2*S*df).*exp(1i*phi); % complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.H = H;
spec.T = T;
spec.h = h;
spec.duration = duration;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function spec = generate_white_spectrum(conditions)
% Band-limited white noise spectrum

% setup
H = conditions.H;
T = conditions.T;
h = conditions.h;
duration = conditions.duration;

% build frequencies
f_max = 1/min(T); % highest frequency; lowest period
f_min = 1/max(T); % lowest frequency; highest period
fN_min = 2*f_max; % at least 2x the maximum frequency
g = gravity; % gravity
f_ev = 3.35/(2*pi)*sqrt(g/h); % frequency above which evanescent modes exceed progressive modes for a piston (Schaffer, 1996, fig 3a)
fN_min = max(fN_min,2*f_ev); % at least 2*f_ev
fN_min = max(fN_min,2); % at least 2Hz

% build spectrum
df = 1/duration; % spectral resolution
Nf = 2^nextpow2(fN_min/df)+1; % number of frequencies
fN = df*(Nf-1); % Nyquist frequency
f = linspace(0,fN,Nf); % frequencies
n_min = find(f>=f_min,1,'first');
if isempty(n_min)
    n_min = 2; % throwing out the 0 frequency bin
end
n_max = find(f<=f_max,1,'last');
if isempty(n_max)
    n_max = Nf;
end
nf = n_min:n_max; % frequency bins used in second order analysis

% generate spectrum with zeros outside nf and ones inside nf
S = zeros(size(f)); % preallocate spectrum with zeros
S(nf) = 1; % nf bins are the band of white noise

% taper and rescale
S = taper_core(S,f,f_ev); % taper spectrum between f_ev and 2*f_ev
m0 = trapz(f,S); % Goda 2.29
S = S*H^2/4.004^2/m0; % Goda 2.40

% generate complex spectrum with random phases
phi = 2*pi*rand(size(S)); % random phases
A = sqrt(2*S*df).*exp(1i*phi); % complex A

% build spec structure
spec.new = 1;
spec.wave_type = conditions.wave_type;
spec.H = H;
spec.T = T;
spec.h = h;
spec.duration = duration;
spec.A = A;
spec.phi = phi;
spec.S = S;
spec.f = f;
spec.omega = 2*pi*f;
spec.df = df;
spec.Nf = Nf;
spec.nf = nf;
end

function y = chirp_core(t,t0,t1,f0,f1)
% core equations for generalized chirp
% f=f0 from t=0 to t=t0
% f linear sweep from f0 to f1 for t from t0 to t1
% f=f1 from t=t1 to end
beta = (f1-f0)/(t1-t0);
f = beta/2*(t-t0) + f0;
y = cos(2*pi*f.*(t-t0));
phi1 = -2*pi*f0*t0;
phi2 = 2*pi*(f0*(t1-t0)/2 - f1*(t1+t0)/2);
y(t<t0) = cos(2*pi*f0.*t(t<t0)+phi1);
y(t>t1) = cos(2*pi*f1.*t(t>t1)+phi2);
end

function S = frf_core(f,fp,H,h,gm)
% core equations for FRF
% Miller&Vincent (1990)
% Goda (2010) p35-36
Bj = 0.0624/(0.230+0.0336*gm-0.185/(1.9+gm))*(1.094-0.01915*log(gm)); % 2.13
sigma = ones(size(f))*0.115; sigma(f>fp) = 0.114; % FRF constants
omega = 2*pi*f; % angular frequencies
k = dispersion_p(h,omega); % dispersion
domegadk = 0.5*omega./k.*(1+2*k*h./sinh(2*k*h)); % group velocity
S = Bj*H^2.*k.^(-2.5)./domegadk.* ... % FRF leading term
    exp(-1.25*(f/fp).^(-4)) .* ...
    gm.^exp(-(f/fp - 1).^2./(2*sigma.^2)); % 2.12
S(1) = 0;
m0 = trapz(f,S); % 2.29
S = S*H^2/4.004^2/m0; % 2.40
end

function S = js_core(f,fp,H,gm)
% core equations for JONSWAP
% Goda (2010) p35
Bj = 0.0624/(0.230+0.0336*gm-0.185/(1.9+gm))*(1.094-0.01915*log(gm)); % 2.13
sigma = ones(size(f))*0.07; sigma(f>fp) = 0.09; % 2.15
S = Bj*H^2*fp^4./f.^5 .* ...
    exp(-1.25*(f/fp).^(-4)) .* ...
    gm.^exp(-(f/fp - 1).^2./(2*sigma.^2)); % 2.12
S(1)=0;
m0 = trapz(f,S); % 2.29
S = S*H^2/4.004^2/m0; % 2.40
end

function S = pm_core(f,fp,H)
% core equation for Pierson-Moskowitz
% Stansberg (2002) Table A.3
S = 5/16*H^2*fp^4./f.^5.*exp(-1.25*(f/fp).^(-4));
S(1)=0;
m0 = trapz(f,S); % Goda 2.29
S = S*H^2/4.004^2/m0; % Goda 2.40
end

function S = tma_core(f,fp,H,h,gm)
% core equations for TMA
% Goda (2010) p36
k = dispersion_p(h,2*pi*f); % dispersion
Bj = 0.0624/(0.230+0.0336*gm-0.185/(1.9+gm))*(1.094-0.01915*log(gm)); % 2.13
sigma = ones(size(f))*0.07; sigma(f>fp) = 0.09; % 2.15
S = Bj*H^2*fp^4./f.^5 .* ...
    exp(-1.25*(f/fp).^(-4)) .* ...
    gm.^exp(-(f/fp - 1).^2./(2*sigma.^2)) .* ...
    tanh(k*h).^2./(1 + 2*k*h./sinh(2*k*h)); % 2.12 and 2.19
S(1) = 0;
m0 = trapz(f,S); % 2.29
S = S*H^2/4.004^2/m0; % 2.40
end

function S = taper_core(S,f,f_ev)
% core equations for spectrum taper from f_ev to 2*f_ev
S_taper = ones(size(S));
S_taper(f>f_ev) = 0.5*cos(pi*(f(f>f_ev)/f_ev-1)) + 0.5;
S_taper(f>2*f_ev) = 0;
S = S .* S_taper;
end

function nf = second_order_bins(f,S)
% frequency bins nf are used in fully nonlinear second order wavemaker theory

% find bins with power spectral density >=1% of the peak
Nf = length(f); % number of frequencies
Sp = max(S); % peak power spectral density
n_min = find(S>=Sp*0.01,1,'first');
if isempty(n_min)
    n_min = 2; % throwing out the 0 frequency bin
end
n_max = find(S>=Sp*0.01,1,'last');
if isempty(n_max)
    n_max = (Nf-1)/2; % half the highest frequency
end

% initial second order frequency range
nf = n_min:n_max;

% now ensure that we capture 99% of the spectral density
done = 0;
while ~done
    if trapz(f(nf),S(nf))<0.99*trapz(f,S)
        nf = (max(2,min(nf)-1)):(min((Nf-1)/2,max(nf)+1));
        done = (min(nf)==2) && (max(nf)==(Nf-1)/2);
    else
        done = 1;
    end
end
end

function theta = generate_wave_angles(N_angles,theta_mean,s)
% Randomly generates a number 'N_angles' of wave angles within ±60°.  Wave
% angles are spread about a mean angle 'theta_mean' (rad) according to the
% cosine-squared relationship where spreading function 's' is a number 
% greater than 1.
%
% Reference:
% H. Mitsuyasu, 1975, Observations of the directional spectrum of ocean
% waves using a cloverleaf buoy, Journal of Physical Oceanography, v5, 750-760.

% setup
s = max(s,eps); % ensure that s is greater than 0
theta_spread = -pi:pi/3600:pi; % array of possible angles within ±180° at 0.1° resolution
theta_max = pi/3; % maximum angle of ±60°
r = rand(1,N_angles); % random #s between 0 and 1
theta = zeros(1,N_angles);

if s <= 170
    % spreading function (Mitsuyasu, 1975)
    % this version blows up less slowly than gamma(2*s+1)
    % see p.108 of notebook, 3-23-2018, for derivation
    g_spread = s/2/sqrt(pi)*gamma(s)/gamma(s+1/2)*cos(theta_spread/2).^(2*s); % spreading function
else
    % Approximate spreading function (Mitsuyasu, 1975)
    % this version forces the area = 1 rather than use gamma functions
    g_spread = cos(theta_spread/2).^(2*s)/trapz(theta_spread,cos(theta_spread/2).^(2*s)); % spreading function
end

% spread angles plus a mean angle
theta_initial = theta_spread + theta_mean; 

% crop spreading function and renormalize it
ind_cropped = find(theta_initial > -theta_max & theta_initial < theta_max);
theta_cropped = theta_initial(ind_cropped); % angles cropped to ±theta_max
g_norm = g_spread(ind_cropped) / trapz(theta_cropped,g_spread(ind_cropped)); % renormalized cropped spreading function

% pick an angle for every frequency
theta_pdf = cumtrapz(theta_cropped,g_norm); % cumulative pdf
for i=1:N_angles % use theta_pdf to pick each theta based on random number r
    theta(i) = theta_cropped(find(theta_pdf >= r(i),1,'first'));
end
end