function [t,s,eta] = linear_displacement(H,T,h,duration)

% LINEAR_DISPLACEMENT
%
% [t,s,eta] = linear_displacement(H,T,h,duration);
%
% Generates a time series 't' (s) of board stroke 's' (m) and free surface
% elevation 'eta' (m) for a linear (sinusoidal) wave train of input wave 
% height 'H' (m), wave period 'T' (s), and water depth 'h' (m), for the 
% specified 'duration' (s).
%
% References:
% Dean, R.G., Dalrymple, R.A., 1991, Water wave mechanics for engineers
% and scientists, V2 of Advanced Series on Ocean Engineering, World
% Scientific: Singapore.
%
% Second-order wavemaker theory for irregular waves, H.A. Schaffer,
% Ocean Engineering, v23, n1, pp. 47-88, 1996.

disp('linear_displacement: starting')

A = H/2; % amplitudes
f = 1./T; % frequencies
omega = 2*pi*f; % angular frequency
N_evanescent = 100; % number of evanescent modes
k = dispersion_complex(h,omega,N_evanescent);

% Biesel coefficients (height-to-stroke ratio)
cj = 2*(cosh(2*k*h) - 1)./(sinh(2*k*h) + 2*k*h);
cj(k*h>20) = 2; % asymptotic limit for deep water
c0 = cj(1,:); % progressive
ce = sum(cj(2:end,:)); % evanescent

% board stroke
Xa = A ./ c0; % Schaffer, 1996, equation 22

% generate free surface and board stroke time series
disp('linear_displacement: generating linear time series')
pause(0.1)
N = duration * 50; % number of timesteps (50 Hz)
t = linspace(0,duration,N+1); % time
eta1 = zeros(size(t));
X01 = zeros(size(t));
for nt=1:length(t)
    % free surface (Schaffer, 1996, equation 14b)
    arg = (1 + ce./c0) .* A .* exp(1i*omega*t(nt)); % progressive (1) and evanescent (ce./c0)
    eta1(nt) = 0.5*sum(arg + conj(arg));
    
    % board stroke (Schaffer, 1996, equation 13)
    arg = -1i*Xa .* exp(1i*omega*t(nt));
    X01(nt) = 0.5*sum(arg + conj(arg));
end
eta = eta1;
s = X01;

% done
disp('linear_displacement: done')