function [t,s,eta,sw] = solitary_displacement(H,h,varargin)

% SOLITARY_DISPLACEMENT
%
% [t,s,eta,sw] = solitary_displacement(H,h);
%
% Generates a time series 't' (s) of board stroke 's' (m) and free surface
% elevation 'eta' (m) for a solitary  wave of input height 'H' (m) and 
% water depth 'h' (m).
%
% References:
% 
% Dutykh D., Clamond D., 2014, Efficient computation of steady solitary
% gravity waves, Wave Motion, 51, pp. 86-99.
%
% Fenton J., 1972, A ninth-order solution for the solitary wave, J. Fluid
% Mech., v53, p2, pp. 257-271.
%
% Grimshaw R., 1971, The solitary wave in water of variable depth, part 2,
% J. Fluid Mech., v46, p3, pp. 611-612.
%
% Hughes S.A., 1993, Physical models and laboratory techniques in coastal
% engineering, Advanced Series on Ocean Engineering, Vol. 7, World
% Scientific Publishing, Singapore.
%
% Wu et.al., 2014, Generation of stable solitary waves by a piston-type
% wave maker, Wave Motion, 51, pp. 240-255.
%
% SEE ALSO: SOLITARY_WAVE

disp('solitary_displacement starting')

% handle inputs
if isempty(varargin)
    wave_type = 'F'; % default (Fenton 9th order)
else
    wave_type = varargin{1}; % specified wave type 
    switch wave_type % check wave_type
        case 'B'
            disp('solitary wave type Boussinesq (1st order)')
        case 'G'
            disp('solitary wave type Grimshaw (3rd order)')
        case 'F'
            disp('solitary wave type Fenton (9th order)')
        case 'R'
            disp('solitary wave type Rayleigh (1st order)')
        case 'DC'
            disp('solitary wave type Dutykh-Clamond (numerical)')
        otherwise
            disp(['wave type ',wave_type,' not recognized!'])
            t = NaN;s = NaN;eta = NaN;
            return
    end
end

% define c (phase speed), ode45 inputs 'tspan' & 's0' 
g = gravity; % gravity
dt = 1/100; % 100 Hz
if strcmp(wave_type,'DC')
    sw = solitary_wave_solution(H,h); % spatial numerical solution
    t = sw.x/sw.c; % x=ct
else
    t = -1000:dt:1000; % more than enough time
    sw = solitary_wave(t,0,H,h,wave_type); % solitary wave solution
end
c = sw.c; % wave speed
t0 = t(find(sw.eta>1e-4,1,'first')); % resolve eta down to at least 0.1mm (note that t0 is negative so we use the 'min' function)
t0 = min(t0,t(find(sw.eta>1e-4*H,1,'first'))); % resolve eta down to at least 0.1% of H
t0 = min(t0,-(3.80+H/h)/sqrt(3*H/4/h^3)/sqrt(g*(h+H))); % Hughes equation 7.1.68
t0 = ceil(abs(t0)/dt)*dt; % round up to nearest multiple of dt
tspan = -t0:dt:t0; % time
s0=0; % initial stroke is 0

% solitary wave free surface function handle
if strcmp(wave_type,'DC')
    f_eta = @sw_eta_interp; % this function interpolates existing 'sw' solution to input t,s
else
    f_eta = @sw_eta; % this function directly solves for the input t,s
end

% shallow water wavemaker equation, Hughes equation 7.145
% s' = (c*eta)/(h+eta)
if strcmp(wave_type,'DC')
    % we pass the solution from above and interpolate it to get 'eta' at
    % timesteps 't' and board positions 's' as the board moves
    f_dsdt = @(t,s) c.*f_eta(t,s,sw)./(h+f_eta(t,s,sw));
else
    % we solve at timesteps 't' and board positions 's' for 'eta' directly
    f_dsdt = @(t,s) c.*f_eta(t,s,H,h,wave_type)./(h+f_eta(t,s,H,h,wave_type));
end

% set options then solve for time t and stroke s
options = odeset('RelTol',1e-9,'AbsTol',1e-12,'InitialStep',dt,'MaxStep',dt); % require higher accuracy
[t,s] = ode45(f_dsdt,tspan,s0,options);

% now solve for eta at time 't' and board stroke 's'
if strcmp(wave_type,'DC')
    eta = sw_eta_interp(t,s,sw);
else
    sw = solitary_wave(t,s,H,h,wave_type);
    eta = sw.eta;
end

% done
disp('solitary_displacement done')
end

function eta = sw_eta(t,s,H,h,wave_type)
% calls solitary_wave and returns the resulting 'eta' field
sw = solitary_wave(t,s,H,h,wave_type);
eta = sw.eta;
end

function eta = sw_eta_interp(t,x,sw)
% interpolates 'sw' to find 'eta' for the given 't' and 'x'
z = x - sw.c*t; % desired spatial coordinate
eta = interp1(sw.x,sw.eta,z,'spline',0); % interpolate solution to desired z
end