function sw = solitary_wave(t,x,H,h,varargin)

% SOLITARY_WAVE
%
% sw = solitary_wave(t,x,H,h);
%
% Solitary wave free surface elevation timeseries, as a function of time
% 't' (s), board position 's' (m), wave height 'H' (m), phase speed 'c'
% (m/s) and water depth 'h' (m).
%
% References:
% Dutykh D., Clamond D., 2014, Efficient computation of steady solitary
% gravity waves, Wave Motion, 51, pp. 86-99.
%
% Fenton J., 1972, A ninth-order solution for the solitary wave, J. Fluid
% Mech., v53, p2, pp. 257-271.
%
% Grimshaw R., 1971, The solitary wave in water of variable depth, part 2,
% J. Fluid Mech., v46, p3, pp. 611-612.
%
% Hughes S.A., 1993, Physical models and laboratory techniques in coastal
% engineering, Advanced Series on Ocean Engineering, Vol. 7, World
% Scientific Publishing, Singapore.
%
% Wu et.al., 2014, Generation of stable solitary waves by a piston-type
% wave maker, Wave Motion, 51, pp. 240-255.

% handle inputs
if isempty(varargin)
    wave_type = 'F'; % default (Fenton 9th order)
else
    wave_type = varargin{1}; % specified wave type
end

% check wave_type
switch wave_type
    case 'B'
        sw = solitary_wave_b(t,x,H,h);
    case 'R'
        sw = solitary_wave_r(t,x,H,h);
    case 'G'
        sw = solitary_wave_g(t,x,H,h);
    case 'F'
        sw = solitary_wave_f(t,x,H,h);
    case 'DC'
        sw = solitary_wave_solution(H,h); % numerical solver
        sw.eta = sw_eta_interp(t,x,sw); % interpolate to desired t,x
    otherwise
        sw = NaN;
        return
end
end

function sw = solitary_wave_b(t,x,H,h)
% Boussinesq 1st order solution

% expansion parameter
e = H/h; % Fenton, equation 14

% alpha
alpha = sqrt(3/4*e); % Fenton 20c, or Grimshaw 2.30, to first order

% c and F squared
g = gravity; % gravity
c = sqrt(g*(h+H)); % Hughes, equation 7.158
F = c^2/g/h; % Froude number squared

% eta
xp = x/h; % nondimensional x
tp = c*t/h; % nondimensional t
S = sech(alpha*(xp - tp));
eta = 1 + e*S.^2; % Fenton 20a, or Grimshaw 2.29
eta = eta*h - h; % physical units, relative to still water

% return
sw.H = H; % wave height
sw.h = h; % water depth
sw.c = c; % phase speed
sw.F = F; % Froude number ^2
sw.alpha = alpha; % straining parameter
sw.eta = eta; % free surface
end

function sw = solitary_wave_r(t,x,H,h)
% Rayleigh 1st order solution

% expansion parameter
e = H/h; % Fenton, equation 14

% alpha
alpha = sqrt(3/4*e*h/(h+H)); % Wu (37), to first order

% c and F squared
g = gravity; % gravity
c = sqrt(g*(h+H)); % Hughes, equation 7.158
F = c^2/g/h; % Froude number squared

% eta
xp = x/h; % nondimensional x
tp = c*t/h; % nondimensional t
S = sech(alpha*(xp - tp));
eta = 1 + e*S.^2; % Fenton 20a, or Grimshaw 2.29
eta = eta*h - h; % physical units, relative to still water

% return
sw.H = H; % wave height
sw.h = h; % water depth
sw.c = c; % phase speed
sw.F = F; % Froude number ^2
sw.alpha = alpha; % straining parameter
sw.eta = eta; % free surface
end

function sw = solitary_wave_g(t,x,H,h)
% Grimshaw 3rd order solution

% expansion parameter
e = H/h; % Fenton, equation 14

% alpha
alpha = sqrt(3/4*e)*(1 - 5/8*e + 71/128*e^2); % Fenton 20c, or Grimshaw 2.30

% c
g = gravity; % gravity
c = sqrt(g*h)*(1 + 1/2*e - 3/20*e^2 + 3/56*e^3); % Grimshaw, equation 2.30
F = c^2/g/h; % Froude number squared

% eta
xp = x/h; % nondimensional x
tp = c*t/h; % nondimensional t
S = sech(alpha*(xp - tp));
T = tanh(alpha*(xp - tp));
eta = 1 + e*S.^2 - 3/4*e^2*S.^2.*T.^2 + ...
    e^3*(5/8*S.^2.*T.^2 - 101/80*S.^4.*T.^2); % Fenton 20a, or Grimshaw 2.29
eta = eta*h - h; % physical units, relative to still water

% return
sw.H = H; % wave height
sw.h = h; % water depth
sw.c = c; % phase speed
sw.F = F; % Froude number ^2
sw.alpha = alpha; % straining parameter
sw.eta = eta; % free surface
end

function sw = solitary_wave_f(t,x,H,h)
% Fenton 9th order solution

% setup
N = 9; % 9th order

% expansion parameter
e = H/h; % Fenton, equation 14

% Froude number squared (F) and wave speed (c)
fc = [1, 1, -0.05, -0.0428571, -0.0342857, -0.0315195, -0.0292784, ...
    -0.0268451, -0.0302634, -0.0219347]; % coefficients from Fenton, Table 1
L = 4;
M = 4;
[p,q] = pade_lm(fc,L,M);
P = sum(p.*e.^(0:L));
Q = sum(q.*e.^(0:M));
F = P/Q;
g = gravity; % gravity
c = sqrt(F*g*h); % wave speed

% alpha straining parameter
ac = [1, -0.625, 0.554688, -0.561535, 0.567095, -0.602969, 0.624914, ...
    -0.670850, 0.700371]; % coefficients from Fenton, Table 1
L = 4;
M = 4;
[p,q]=pade_lm(ac,L,M);
P = sum(p.*e.^(0:L));
Q = sum(q.*e.^(0:M));
alpha0 = sqrt(3/4*e)*P/Q;
alpha = sw_alpha_from_f(F,alpha0); % solve F=tan(2alpha)/2alpha (Fenton equation 31)

% eta
xp = x/h; % nondimensional x
tp = c*t/h; % nondimensional t
if size(xp,2) > 1
    xp = xp.'; % force xp to be K-by-1
end
if size(tp,2) > 1
    tp = tp.'; % force tp to be K-by-1
end
S = sech(alpha*(xp - tp));
K = length(S);
eta_i = zeros(K,N);
eta_i(:,1) = 1 + e*S.^2;
i = 2;
while i<=N
    switch i
        case 2
            eta = e^2*(-0.75*S.^2 + 0.75*S.^4);
        case 3
            eta = e^3*(0.625*S.^2 - 1.8875*S.^4 + 1.2625*S.^6);
        case 4
            eta = e^4*(-1.36817*S.^2 + 3.88033*S.^4 - 4.68304*S.^6 + ...
                2.17088*S.^8);
        case 5
            eta = e^5*(1.86057*S.^2 - 7.45136*S.^4 + 12.7637*S.^6 - ...
                11.4199*S.^8 + 4.24699*S.^10); % last term modified to force coefficient sum = 0
        case 6
            eta = e^6*(-2.57413*S.^2 + 13.2856*S.^4 - 31.1191*S.^6 + ...
                40.1068*S.^8 - 28.4272*S.^10 + 8.72803*S.^12); % last term modified to force coefficient sum = 0
        case 7
            eta = e^7*(3.4572*S.^2 - 22.782*S.^4 + 68.258*S.^6 - ...
                116.974*S.^8 + 120.490*S.^10 - 71.057*S.^12 + 18.6078*S.^14); % last term modified to force coefficient sum = 0
        case 8
            eta = e^8*(-4.6849*S.^2 + 37.670*S.^4 - 139.28*S.^6 + ...
                301.442*S.^8 - 411.416*S.^10 + 355.069*S.^12 - ...
                180.212*S.^14 + 41.4119*S.^16); % last term modified to force coefficient sum = 0
        case 9
            eta = e^9*(6.191*S.^2 - 60.57*S.^4 + 269.84*S.^6 - ...
                712.125*S.^8 + 1217.98*S.^10 - 1384.37*S.^12 + ...
                1023.07*S.^14 - 450.29*S.^16 + 90.274*S.^18); % last term modified to force coefficient sum = 0
    end
    eta_i(:,i) = eta_i(:,i-1) + eta;
    i = i+1;
end
eta_t = zeros(K,1);
for k=1:K
    et = shanks_transformation(eta_i(k,:)); % single Shanks transformation, per Fenton p.268
    if isfinite(et(end))
        eta_t(k) = et(end);
    else
        eta_t(k) = eta_i(k,N);
    end
end
eta = eta_t;
eta = eta*h - h; % physical units, relative to still water

% return
sw.H = H; % wave height
sw.h = h; % water depth
sw.N = N; % order
sw.F = F; % froude number ^2
sw.c = c; % phase speed
sw.alpha = alpha; % straining parameter
sw.eta = eta; % free surface
end

function [p,q] = pade_lm(a,L,M)
% generate pade coefficients from series of coefficients a
% a is assumed to be length L+M
% returns p 0 to L and q 1 to M (q0 assumed 1)
A = zeros(M,M);
for m=1:M
    n = (M:-1:1)+m;
    A(m,:) = a(n);
end
B = zeros(M,1);
for m=1:M
    B(m) = -a(m+L+1);
end
q = A\B; % first solve for q using an 4x4 matrix
q = [1;q].'; % first element of q is 1 by convention
p = zeros(1,L+1);
for l = 1:L+1
    p(l) = sum(a(l:-1:1).*q(1:l)); % now solve directly for p
end
end

function alpha = sw_alpha_from_f(F,alpha0)
% solves for solitary wave alpha given Froude number squared (F) and
% initial guess (alpha0), using equation 31 and Halley's method.
x = F;
y = 2*alpha0;
t = tan(y);
f = t-x.*y;
dfdy = 1+t.^2-x;
d2fdy2 = 2*t.*(dfdy+x);
y1 = y - 2*f.*dfdy./(2*dfdy.^2 - f.*d2fdy2); y1(isnan(y1)) = 0;
dy = y1-y;
y = y1;
count = 1;
while any(abs(dy)>eps(y)) && any(abs(f)>eps(x.*y)) && count < 10
    t = tan(y);
    f = t-x.*y;
    dfdy = 1+t.^2-x;
    d2fdy2 = 2*t.*(dfdy+x);
    y1 = y - 2*f.*dfdy./(2*dfdy.^2 - f.*d2fdy2); y1(isnan(y1)) = 0;
    dy = y1-y;
    y = y1;
    count = count+1;
end
alpha = y/2;
end

function eta = sw_eta_interp(t,x,sw)
% interpolates 'sw' to find 'eta' for the given 't' and 'x'
z = x - sw.c*t; % desired spatial coordinate
eta = interp1(sw.x,sw.eta,z,'spline',0); % interpolate solution to desired z
end