function sw = solitary_wave_solution(H,h)

%
% Solves for a solitary wave of the specified height 'H' (m) in still water
% depth 'h' (m).
%
% References:
% Dutykh D., Clamond D., 2014, Efficient computation of steady solitary
% gravity waves, Wave Motion, 51, pp. 86-99.
%
% Fenton J., 1972, A ninth-order solution for the solitary wave, J. Fluid
% Mech., v53, p2, pp. 257-271.
%
% This is an edited port of SolitaryGravityAmplitude.m, by original authors:
% Denys Dutykh, CNRS -- LAMA, University of Savoie, France
% Didier Clamond, University of Nice - Sophia Antipolis
% Denys.Dutykh@univ-savoie.fr
% http://www.denys-dutykh.com/

% expansion parameter
e = H/h; % Fenton, equation 14
if e>0.79 || e<=0
    error('e = H/h must be positive and less than 0.79');
end

% Froude number squared (F)
fc = [1, 1, -0.05, -0.0428571, -0.0342857, -0.0315195, -0.0292784, ...
    -0.0268451, -0.0302634, -0.0219347]; % coefficients from Fenton, Table 1
L = 4;
M = 4;
[p,q] = pade_lm(fc,L,M);
P = sum(p.*e.^(0:L));
Q = sum(q.*e.^(0:M));
F = P/Q;

% alpha straining parameter
ac = [1, -0.625, 0.554688, -0.561535, 0.567095, -0.602969, 0.624914, ...
    -0.670850, 0.700371]; % coefficients from Fenton, Table 1
L = 4;
M = 4;
[p,q]=pade_lm(ac,L,M);
P = sum(p.*e.^(0:L));
Q = sum(q.*e.^(0:M));
alpha0 = sqrt(3/4*e)*P/Q;
alpha = sw_alpha_from_f(F,alpha0); % solve F=tan(2alpha)/2alpha (Fenton equation 31)

% half-length of the computational domain
L  = 8.5*log(10)/alpha;

% Numerical parameters (can be changed if you know what you are doing).
N   = 65536;                       % number of Fourier modes (must be even)
tol = 1e-15;                       % iterations stopping criterium
gam = 2.0;                         % the exponent parameter in the Petviashvili method  % (needed to compute the stabilizing factor)

% Discrete variables
dxi = 2*L/N;                       % distance between two consecutive points in the mapped space
xi  = (1-N/2:N/2)'*dxi;            % mapped space discretization
k   = [0:N/2-1,-N/2:-1]'*pi/L;     % wavenumbers
COP = k.*coth(k); COP(1) = 1;      % nonlocal C-operator in the Babenko equation
IOP = -1i*coth(k); IOP(1) = 0;     % nonlocal int-C-operator

% initial guess from 9th order (Fenton)
S = sech(alpha*xi).^2;
eta = e*S + e^2*(-0.75*S + 0.75*S.^2) + e^3*(0.625*S - 1.8875*S.^2 + 1.2625*S.^3) + ...
    e^4*(-1.36817*S + 3.88033*S.^2 - 4.68304*S.^3 + 2.17088*S.^4) + ...
    e^5*(1.86057*S - 7.45136*S.^2 + 12.7637*S.^3 - 11.4199*S.^4 + 4.24699*S.^5) + ...
    e^6*(-2.57413*S + 13.2856*S.^2 - 31.1191*S.^3 + 40.1068*S.^4 - 28.4272*S.^5 + 8.72803*S.^6) + ...
    e^7*(3.4572*S - 22.782*S.^2 + 68.258*S.^3 - 116.974*S.^4 + 120.490*S.^5 - 71.057*S.^6 + 18.6078*S.^7) + ...
    e^8*(-4.6849*S + 37.670*S.^2 - 139.28*S.^3 + 301.442*S.^4 - 411.416*S.^5 + 355.069*S.^6 - 180.212*S.^7 + 41.4119*S.^8) + ...
    e^9*(6.191*S - 60.57*S.^2 + 269.84*S.^3 - 712.125*S.^4 + 1217.98*S.^5 - 1384.37*S.^6 + 1023.07*S.^7 - 450.29*S.^8 + 90.274*S.^9);

% Petviashvili's iterations
err = inf;
while err>tol
    % Compute intermediate quantities
    eta_hat  = fft(eta);
    eta2_hat = fft(eta.^2);
    Ceta     = real(ifft(COP.*eta_hat));
    
    % Estimation of the squared Froude number:
    M0 = dxi*sum(eta.*(1 + Ceta));            % Mass
    P0 = 0.5*dxi*sum((1 + Ceta).*(eta.^2));   % Potential energy
    F = 1 + 3*P0/M0;                         % squared Froude
    
    LOP = F*COP - 1;         % linear operator of Babenko eq.
    LOI = 1./LOP;             % its inverse
    
    Leta_hat = LOP.*eta_hat;
    Nl_hat   = 0.5*COP.*eta2_hat + fft(eta.*Ceta);
    
    % Compute weight.
    S1 = eta_hat'*Leta_hat;   % scalar product
    S2 = eta_hat'*Nl_hat;     % scalar product
    S  = (S1/S2)^gam;
    
    % new value
    eta1 = real(ifft(S*LOI.*Nl_hat));
    
    % Renormalize the free surface elevation:
    a = max(eta1);
    eta1 = e*eta1/a;
    
    err  = norm(eta1-eta,inf);        % errors are measured in l_inf norm
    eta  = eta1;
end
% End of numerical resolution.

% save output to sw structure
% nondimensional terms
sw.F = F; % Froude number (Fr) squared
Ceta = real(ifft(COP.*fft(eta))); % C(eta)
sw.M = dxi*eta'*(1+Ceta); % excess mass
sw.C = sqrt(sw.F)*dxi*sum(Ceta); % circulation
sw.I = sqrt(sw.F)*sw.M; % impulse (calculated directly as I=sqrt(F)*M where F=Fr^2)
sw.T = 0.5*sw.F*dxi*eta'*Ceta; % kinetic energy
sw.V = 0.5*dxi*(eta.^2)'*(1+Ceta); % potential energy

% terms in physical units
g = gravity; % gravity
sw.c = sqrt(F*g*h); % wave speed
sw.H = H; % wave height
sw.h = h; % water depth
sw.x = (1 + mean(eta))*xi + real(ifft(IOP.*fft(eta-mean(eta)))); % cross-shore coordinate
sw.x = sw.x*h; % physical units
sw.eta = eta; % free surface elevation
sw.eta = sw.eta*h; % physical units

end

function [p,q] = pade_lm(a,L,M)
% generate pade coefficients from series of coefficients a
% a is assumed to be length L+M
% returns p 0 to L and q 1 to M (q0 assumed 1)
A = zeros(M,M);
for m=1:M
    n = (M:-1:1)+m;
    A(m,:) = a(n);
end
B = zeros(M,1);
for m=1:M
    B(m) = -a(m+L+1);
end
q = A\B; % first solve for q using an 4x4 matrix
q = [1;q].'; % first element of q is 1 by convention
p = zeros(1,L+1);
for l = 1:L+1
    p(l) = sum(a(l:-1:1).*q(1:l)); % now solve directly for p
end
end

function alpha = sw_alpha_from_f(F,alpha0)
% solves for solitary wave alpha given Froude number squared (F) and
% initial guess (alpha0), using equation 31 and Halley's method.
x = F;
y = 2*alpha0;
t = tan(y);
f = t-x.*y;
dfdy = 1+t.^2-x;
d2fdy2 = 2*t.*(dfdy+x);
y1 = y - 2*f.*dfdy./(2*dfdy.^2 - f.*d2fdy2); y1(isnan(y1)) = 0;
dy = y1-y;
y = y1;
count = 1;
while any(abs(dy)>eps(y)) && any(abs(f)>eps(x.*y)) && count < 10
    t = tan(y);
    f = t-x.*y;
    dfdy = 1+t.^2-x;
    d2fdy2 = 2*t.*(dfdy+x);
    y1 = y - 2*f.*dfdy./(2*dfdy.^2 - f.*d2fdy2); y1(isnan(y1)) = 0;
    dy = y1-y;
    y = y1;
    count = count+1;
end
alpha = y/2;
end