function [t,s,eta] = stokes_displacement(H,T,h,duration)

% STOKES_DISPLACEMENT
%
% [t,s,eta] = stokes_displacement(H,T,h,duration);
%
% Generates a time series 't' (s) of board stroke 's' (m) and free surface
% elevation 'eta' (m) for a fully second-order nonlinear (Stokes) wave train
% of input wave height 'H' (m), wave period 'T' (s), and water depth 'h' (m),
% for the specified 'duration' (s).
%
% References:
% Dean, R.G., Dalrymple, R.A., 1991, Water wave mechanics for engineers
% and scientists, V2 of Advanced Series on Ocean Engineering, World
% Scientific: Singapore.
%
% Schaffer H.A., 1996, Second-order wavemaker theory for irregular waves,
% Ocean Engineering, v23, n1, pp. 47-88.
%
% Schaffer H.A., Steenberg C.M., 2003, Second-order wavemaker theory for
% multidirectional waves, Ocean Engineering, v30, pp. 1203-1231.
%
% SEE ALSO: DISPERSION_SUMDIF, FG_MINUS, FG_PLUS

disp('stokes_displacement: starting')

% setup
spec.new = 1;
spec.wave_type = 'REG';
spec.H = H;
spec.T = T;
spec.h = h;
A = H/2; % amplitudes
phi = zeros(size(A)); % 0 wave phases
f = 1./T; % frequencies
omega = 2*pi*f; % angular frequencies
Nf = length(f); % number of spectral frequencies
nf = 1:Nf; % frequencies to use in timeseries generation
alpha = zeros(size(f)); % no wave angles
alpha_mean = 0; % 0 mean angle
spread_coef = 0; % 0 spreading
y_board = 0; % only 1 board position
spec.duration = T; % only a single wave period
spec.A = A;
spec.phi = phi;
spec.f = f;
spec.omega = omega;
spec.Nf = Nf;
spec.nf = nf;
spec.alpha = alpha;
spec.alpha_mean = alpha_mean;
spec.spread_coef = spread_coef;
spec.y_board = y_board;

% linear wavemaker theory
[t,X01,eta1] = wm_core_l(spec);

% second order nonlinear wavemaker theory
[X02_plus,X02_minus,eta2_plus,eta2_minus,~] = wm_core_nl(spec);

% build timeseries for output
eta = eta1 + eta2_plus + eta2_minus; % total second order free surface
s = X01 + X02_plus + X02_minus; % total second order displacement

% pad out to full duration
dt = t(2); % timestep
t = 0:dt:duration;
s = padarray(s(1:end-1),[0 length(t) - length(s) + 1],'circular','post');
eta = padarray(eta(1:end-1),[0 length(t) - length(eta) + 1],'circular','post');

% done
disp('stokes_displacement: done')
end