function [t,s,eta] = stream_function_displacement(H,T,h,duration,N)

% STREAM_FUNCTION_DISPLACEMENT
%
% [t,s,eta] = stream_function_displacement(H,T,h,duration);
% [t,s,eta] = stream_function_displacement(H,T,h,duration,N);
%
% Generates a time series 't' (s) of board stroke 's' (m) and free surface
% elevation 'eta' (m) for a stream function wave train of input wave height
% 'H' (m), wave period 'T' (s), and water depth 'h' (m), for the specified 
% 'duration' (s) and order N (-, default 30).
%
% References: 
% Fenton J.D., 1999. Numerical methods for nonlinear waves, in 
% Advances in Coastal and Ocean Engineering, V5, ed. P.L.-F. Liu, p241-324,
% World Scientific: Singapore.
%
% Hughes S.A., 1993. Physical models and laboratory techniques 
% in coastal engineering. Advanced Series on Ocean Engineering, Vol. 7, 
% World Scientific Publishing, Singapore.
%
% Zhang H., Schaffer H.A., 2007. Approximate stream function wavemaker
% theory for highly non-linear waves in wave flumes, Ocean Engineering, 34,
% p1290-1302.
%
% SEE ALSO: STREAM_FUNCTION_SOLUTION, STREAM_FUNCTION_WAVE

disp('stream_function_displacement: starting')

% setup
g = gravity; % gravity
disp('stream_function_solution: setting up')
if nargin < 5
    N = 30; % default
end

% stream function solution
sf = stream_function_solution(H,T,h,N);
if ~isstruct(sf)
    t=NaN;s=NaN;eta=NaN;
    return
end

% phase speed
c = sf.c;

% cnoidal wave free surface function handle
f_eta = @stream_function_wave;

% shallow water wavemaker equation, Hughes equation 7.145
% s' = c*eta/(h+eta)
pause(0.1)
disp('stream_function_displacement: solving shallow water wavemaker equation')
f_dsdt = @(t,s) c*f_eta(t,s,sf)./(h+f_eta(t,s,sf)); 

% solving 1/2 period in two directions
f_min = 2*N/T; % N points in T/2 seconds
f_min = max(f_min,50); % or 50 Hz, whichever is larger
n = 2^nextpow2(T/2*f_min); % number of timesteps for 1/2 period
dt = T/2/n; % timestep

% forward from 0 to T/2
tspan = linspace(0,T/2,n+1);
s0 = 0; % initial stroke is 0
options = odeset('RelTol',1e-9,'AbsTol',1e-12); % require higher accuracy
[t_f,s_f] = ode45(f_dsdt,tspan,s0,options);

% solve backward from T/2 to 0
tspan = linspace(T/2,0,n+1); % running time backwards
s0 = 0; % initial stroke is 0
options = odeset('RelTol',1e-9,'AbsTol',1e-12); % require higher accuracy
[~,s_b] = ode45(f_dsdt,tspan,s0,options);

% merge forward and backward solutions to eliminate drift in board stroke
% this is a larger issue with cnoidal waves (see cnoidal_displacement.m)
% I am using it here for consistency
% result is a solution from 0 to T/2
coeff = 0.5*(cos(2*pi*t_f/T)+1);
s_m = s_f.*coeff + s_b(end:-1:1).*(1-coeff); % ensure start and end at 0

% flip, negate, concatenate to run solution from 0 to T-dt
t = [t_f;t_f(end)+t_f(2:end-1)];
s = [s_m;-s_m(end-1:-1:2)];

% build gamma, dispersion correction transfer function for linear wave generation
disp('stream_function_displacement: applying dispersion correction for ad-hoc unified wave generation')
fs = 1/dt; % sample rate
fN = fs/2; % Nyquist frequency
f = linspace(0,fN,2*n+1); % frequencies
omega = 2*pi*f;
k = dispersion_p(h,omega);
gamma = k*h.*(2*k*h + sinh(2*k*h))./4./sinh(k*h).^2; % Zhang & Schaffer eq'n 8
gamma(1) = 1; % f=0 (DC) term

% limit to high-frequency amplification at omega*sqrt(h/g) 50% that of Zhang & Schaffer
% gamma(omega*sqrt(h/g) > 7) = gamma(omega*sqrt(h/g) == 7); % Zhang & Schaffer eq'n 17
omega_limit = 3.5*sqrt(g/h); % value of omega corresponding to omega*sqrt(h/g) = 3.5
k_limit = dispersion_p(h,omega_limit);
gamma_limit = k_limit*h*(2*k_limit*h + sinh(2*k_limit*h))/4/sinh(k_limit*h)^2; % Zhang & Schaffer eq'n 8
gamma(omega > omega_limit) = gamma_limit; % maximum dispersion correction

% apply dispersion correction
s_padded = padarray(s,n,'circular'); % pad
F_s = fft(s_padded); % fft
F_s_corrected = F_s .* [gamma, gamma(end-1:-1:2)]'; % apply correction
s = ifft(F_s_corrected); % ifft
s = s(n+1:3*n); % unpad

% now solve for eta
disp('stream_function_displacement: solving for eta at paddle')
eta = stream_function_wave(t,s,sf);

% pad out to full duration
t = (0:dt:duration);
s = padarray(s,length(t) - length(s),'circular','post')';
eta = padarray(eta,length(t) - length(eta),'circular','post')';

% done
disp('stream_function_displacement: done')