function sf = stream_function_solution(H,T,h,N)

% STREAM_FUNCTION_SOLUTION
%
% sf = stream_function_solution(H,T,h);
% sf = stream_function_solution(H,T,h,N);
%
% Solves for a stream function wave of specified height 'H' (m), period 
% 'T' (s), water depth 'h' (m) and order N (-, default 30).
%
% Returns the structure 'sf' with the following fields:
% order sf.N
% frequency sf.omega 
% wavenumber sf.k
% phase speed sf.c
% N+1 free surface coefficients sf.eta over 1/2 wavelength
% N stream function constants sf.B
% mean fluid speed sf.U
% volume flow rate sf.Q
% Bernoulli constant sf.R
% water depth sf.h
%
% Reference: 
% Fenton J.D., 1999. Numerical methods for nonlinear waves, in 
% Advances in Coastal and Ocean Engineering, V5, ed. P.L.-F. Liu, p241-324,
% World Scientific: Singapore.
%
% SEE ALSO: STREAM_FUNCTION_DISPLACEMENT, STREAM_FUNCTION_WAVE

% setup
disp('stream_function_solution: setting up')
H_steps = sf_steps(H,T,h); % determine wave height steps
if isnan(H_steps)
    sf = NaN;
    return
end
if nargin < 4
    N = sf_order(H,T,h); % determine order
    if isnan(N)
        sf = NaN;
        return
    end
end
I_steps = length(H_steps); % number of steps
omega = 2*pi/T; % frequency
g = gravity; % gravity 

% solution routine
disp('stream_function_solution: solving')
success = 0;
give_up = 0;
while ~success && ~give_up
    % set up
    H0 = H_steps(1); % first wave height
    x0 = sf_initialize(T,H0,h,N); % initial solution values
    H_m1 = 0;
    x_m1 = 0;
    options = sf_solution_options(x0);
    i_step = 1;
    failed_step = 0;
    
    % increment through wave height steps
    while i_step <= I_steps && ~failed_step
        disp(['stream_function_solution: step ',num2str(i_step),' of ',num2str(I_steps)])
        [solved_this_step,x] = sf_step_solution_attempt(x0,H0,T,h,N,options); % solve
        if solved_this_step
            if i_step < I_steps
                % set up for next solution step
                x_m2 = x_m1; H_m2 = H_m1; % previous step
                x_m1 = x; H_m1 = H0; % this step
                if i_step == 1
                    H0 = H_steps(i_step+1); % increment height for next step
                    x0 = x_m1; % initial condition for next step is this step's solution
                else
                    % initial condition for next step is a truncated Taylor series
                    % approximation, per Sobey (2012)
                    H0 = H_steps(i_step+1); % increment height for next step
                    x0 = x_m1 + (x_m1 - x_m2)/(H_m1 - H_m2)*(H0 - H_m1); % Sobey equation 52
                end
            end
            i_step = i_step + 1; % increment the step
        else
            disp('stream_function_solution: step failed')
            failed_step = 1;
        end
    end
    
    % either we got through all steps successfully or we had a bad step
    if failed_step
        % failure, give up
        give_up = 1;
    else
        % success; rerun final solution
        x0 = x;
        x = fsolve(@(x) sf_f(x,H,T,h,N),x0,options);
        success = 1;
    end
end

if success
    % return sf structure
    sf.N = N; % order
    sf.omega = omega; % frequency
    sf.k = x(2*N+5)/h; % wavenumber, x(2*N+5) = kh
    sf.c = sf.omega/sf.k; % phase speed
    sf.eta = x(1:N+1)/sf.k; % eta coefficients, x(1:N+1) = k*eta(1:N+1)
    sf.B = x(N+2:2*N+1); % stream function constants B, x(N+2:2*N+1) = B(1:N)
    sf.U = x(2*N+2)/sqrt(sf.k/g); % mean fluid speed U, x(2*N+2) = U*sqrt(k/g)
    sf.Q = x(2*N+3)/sqrt(sf.k^3/g); % volume flow rate Q, x(2*N+3) = Q*sqrt(k^3/g)
    sf.R = x(2*N+4)*g/sf.k; % Bernoulli constant R, x(2*N+4) = R*k/g
    sf.h = x(2*N+5)/sf.k; % water depth h, x(2*N+5) = kh
    sf.eta = sf.eta - sf.h; % subtract depth from eta coefficients
    
    disp('stream_function_solution: done')
else
    % failure
    sf = NaN;
    disp('stream_function_solution: failed')
end

end

function [solved,x] = sf_step_solution_attempt(x0,H0,T,h,N,options)
% try to solve a step
[x,~,exitflag,~] = fsolve(@(x) sf_f(x,H0,T,h,N),x0,options);

if exitflag == 1
    % success!
    solved = 1;
elseif exitflag < 0
    % failure!
    solved = 0;
else
    % 2nd attempt 
    x0 = x; % use previous step solution
    [x,~,exitflag,~] = fsolve(@(x) sf_f(x,H0,T,h,N),x0,options);
    if exitflag > 0 % accept any positive exitflag
        % success!
        solved = 1;
    elseif exitflag < 0
        % failure!
        solved = 0;
    else
        % third attempt, try for more iterations and evaluations 
        x0 = x; % use previous step solution
        options = optimoptions(options,'MaxIterations',800); % 2x default maximum iterations
        options = optimoptions(options,'MaxFunctionEvaluations',400*length(x0)); % 4x default maximum function evals
        [x,~,exitflag,~] = fsolve(@(x) sf_f(x,H0,T,h,N),x0,options);
        if exitflag > 0 % accept any positive exitflag
            % success!
            solved = 1;
        elseif exitflag < 0
            % failure!
            solved = 0;
        else
            % fourth attempt, try for more iterations and evaluations
            x0 = x; % use previous step solution
            options = optimoptions(options,'MaxIterations',1600); % 4x default maximum iterations
            options = optimoptions(options,'MaxFunctionEvaluations',800*length(x0)); % 8x default maximum function evals
            [x,~,exitflag,~] = fsolve(@(x) sf_f(x,H0,T,h,N),x0,options);
            if exitflag > 0 % accept any positive exitflag
                % success!
                solved = 1;
            elseif exitflag < 0
                % failure!
                solved = 0;
            else
                % fifth attempt, try for more iterations and evaluations
                x0 = x; % use previous step solution
                options = optimoptions(options,'MaxIterations',3200); % 8x default maximum iterations
                options = optimoptions(options,'MaxFunctionEvaluations',1600*length(x0)); % 16x default maximum function evals
                [x,~,exitflag,~] = fsolve(@(x) sf_f(x,H0,T,h,N),x0,options);
                if exitflag > 0 % accept any positive exitflag
                    % success!
                    solved = 1;
                elseif exitflag < 0
                    % failure!
                    solved = 0;
                else
                    % final attempt, even more iterations and evaluations
                    x0 = x; % use previous step solution
                    options = optimoptions(options,'MaxIterations',6400); % 16x default maximum iterations
                    options = optimoptions(options,'MaxFunctionEvaluations',3200*length(x0)); % 32x default maximum function evals
                    [x,~,exitflag,~] = fsolve(@(x) sf_f(x,H0,T,h,N),x0,options);
                    if exitflag >= 0 % accept any nonnegative exitflag
                        % success!
                        solved = 1;
                    else
                        % failure!
                        solved = 0;
                    end
                end
            end
        end
    end
end
end

function x0 = sf_initialize(T,H0,h,N)
% initial values for x in stream function solution
x0 = zeros(1,2*N+5);

% dispersion
omega = 2*pi/T;
k0 = dispersion_p(h,omega);

% N+1 initial values for x(1:N+1) = k*eta(1:N+1);
n = 0:N;
x0(1:N+1) = k0*h + 0.5*k0*H0*cos(n*pi/N);

% N initial values for x(N+2:2*N+1) = B(1:N)
B = zeros(1,N);
B(1) = 0.5*k0*H0/sqrt(tanh(k0*h));
x0(N+2:2*N+1) = B;

% initial value for x(2*N+2) = U*sqrt(k/g)
x0(2*N+2) = sqrt(tanh(k0*h));

% initial value for x(2*N+3) = Q*sqrt(k^3/g)
x0(2*N+3) = k0*h*sqrt(tanh(k0*h));

% initial value for x(2*N+4) = R*k/g
x0(2*N+4) = k0*h + 0.5*tanh(k0*h);

% initial value for x(2*N+5) = kh
x0(2*N+5) = k0*h;
end

function N = sf_order(H,T,h)
% determine default order
% used if solution order is unspecifed

% choose order based on Ursell number
omega = 2*pi/T; % wave frequency
k = dispersion_p(h,omega); % wavenumber from dispersion
L = 2*pi/k; % wavelength
U = H*L^2/h^3; % Ursell number
if U < 5
    N_u = 16;
elseif U < 10
    N_u = 32;
elseif U < 25
    N_u = 40;
elseif U < 50
    N_u = 48;
elseif U < 100
    N_u = 64;
elseif U < 250
    N_u = 72;
elseif U < 500
    N_u = 80;
elseif U < 1000
    N_u = 88;
else
    N_u = 96;
end

% possibly increase order based on H/H_max
p = L/h;
H_max = h*(0.141063*p + 0.0095721*p^2 + 0.0077829*p^3)...
    /(1 + 0.0788340*p + 0.0317567*p^2 + 0.0093407*p^3); % Fenton (1990), equation 6.1
if H > H_max
    N = NaN;
    return
elseif H < 0.2*H_max
    N = max(N_u,16);
elseif H < 0.4*H_max
    N = max(N_u,32);
elseif H < 0.8*H_max
    N = max(N_u,40);
elseif H < 0.9*H_max
    N = max(N_u,48);
elseif H < 0.95*H_max
    N = max(N_u,64);
elseif H < 0.99*H_max
    N = max(N_u,80);
else
    N = max(N_u,96);
end

% finally limit order based on k
if k*h > 1
    N = min(N,32);
elseif k*h > 0.8
    N = min(N,48);
elseif k*h > 0.2
    N = min(N,64);
end
end

function H_steps = sf_steps(H,T,h)
% determine wave height steps
omega = 2*pi/T; % wave frequency
k = dispersion_p(h,omega); % wavenumber from dispersion
L = 2*pi/k; % wavelength
U = H*L^2/h^3; % Ursell number
p = L/h;
H_max = h*(0.141063*p + 0.0095721*p^2 + 0.0077829*p^3)...
    /(1 + 0.0788340*p + 0.0317567*p^2 + 0.0093407*p^3); % Fenton (1990), equation 6.1
disp(['stream_function_solution: H/Hmax = ',num2str(H/H_max)])
if H > H_max
    disp('stream_function_solution: error. H out of bounds')
    H_steps = NaN;
    return
elseif (H < 0.2*H_max) && (U < 10)
    H_steps = [1/2,1]*H;
elseif (H < 0.4*H_max) && (U < 25)
    H_steps = [1/4,1/2,3/4,1]*H;
elseif (H < 0.8*H_max) && (U < 50)
    H_steps = [1/8,1/4,3/8,1/2,5/8,3/4,7/8,1.0]*H;
elseif (H < 0.9*H_max) && (U < 100)
    H_steps = [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0]*H;
elseif H < 0.95*H_max && (U < 250)
    H_steps = [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9, ...
        0.92,0.94,0.96,0.98,0.99,1.0]*H;
elseif H < 0.99*H_max && (U < 500)
    H_steps = [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8, ...
        0.82,0.84,0.86,0.88,0.9, ...
        0.91,0.92,0.93,0.94,0.95,0.96,0.97,0.98,0.99, ...
        0.995,0.998,1.0]*H;
else
    H_steps = [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8, ...
        0.82,0.84,0.86,0.88,0.9, ...
        0.91,0.92,0.93,0.94,0.95,0.96,0.97,0.98,0.99, ...
        0.992,0.994,0.995,0.996,0.997,0.998,0.999,1.0]*H;
end
end

function options = sf_solution_options(x0)
options = optimoptions('fsolve'); % default options
options = optimoptions(options,'Display','off'); % disable spam
options = optimoptions(options,'FinDiffType','central'); % central differencing for higher accuracy
options = optimoptions(options,'MaxFunctionEvaluations',200*length(x0)); % 2x default maximum function evals
end

function F = sf_f(x,H,T,h,N)

% SF_F
%
% F = sf_f(x,H,T,h,N);
%
% Function (F=0) is to be solved for stream function waves of specified
% height 'H' (m), period 'T' (s), water depth 'h' (m) and order N.
%
% Solution x consists of 2*N+5 points as follows:
% x(1:N+1) = k*eta(1:N+1);
% x(N+2:2*N+1) = B(1:N)
% x(2*N+2) = U*sqrt(k/g)
% x(2*N+3) = Q*sqrt(k^3/g)
% x(2*N+4) = R*k/g
% x(2*N+5) = kh
%
% Reference:
% Fenton, J.D., 1999, Numerical methods for nonlinear waves, in Advances in
% Coastal and Ocean Engineering, V5, ed. P.L.-F. Liu, pp241-324, World
% Scientific: Singapore.

% setup
F = zeros(1,2*N+5);
g = gravity; % gravity
j = 1:N;
omega = 2*pi/T;

for m = 1:N+1
    % bottom kinematic boundary condition, equation 3.7
    F(m) = -x(2*N+2)*x(m) + sum(x(N+2:2*N+1).*sinh(j*x(m))./cosh(j*x(2*N+5)).*cos(j*pi*(m-1)/N)) + x(2*N+3);
    
    % free surface kinematic boundary condition, equation 3.8
    F(N+1+m) = 0.5*(-x(2*N+2) + sum(j.*x(N+2:2*N+1).*cosh(j*x(m))./cosh(j*x(2*N+5)).*cos(j*pi*(m-1)/N)))^2 + ...
        0.5*(sum(j.*x(N+2:2*N+1).*sinh(j*x(m))./cosh(j*x(2*N+5)).*sin(j*pi*(m-1)/N)))^2 + ...
        x(m) - x(2*N+4);
end

% mean depth, equation 3.9
F(2*N+3) = (0.5*(x(1) + x(N+1)) + sum(x(2:N)))/N - x(2*N+5);

% wave height, equation 3.10
F(2*N+4) = x(1) - x(N+1) - x(2*N+5)*H/h;

% wave period, no mass transport, equation 3.16
F(2*N+5) = x(2*N+3)/sqrt(x(2*N+5)) - omega/sqrt(g/h);
end
