function eta = stream_function_wave(t,x,sf)

% STREAM_FUNCTION_WAVE
%
% eta = stream_function_wave(t,x,sf);
%
% Stream function wave free surface elevation timeseries, as a function of
% time 't' (s), cross-shore position 'x' (m), and stream function solution
% structure 'sf'.
%
% Reference: 
% Fenton J.D., 1999. Numerical methods for nonlinear waves, in 
% Advances in Coastal and Ocean Engineering, V5, ed. P.L.-F. Liu, p241-324,
% World Scientific: Singapore.
%
% SEE ALSO: STREAM_FUNCTION_DISPLACEMENT, STREAM_FUNCTION_SOLUTION

% setup
N = sf.N;

% Fenton, J.D., 1999, equation 3.26, p.15
E = zeros(1,N+1); % cosine transform term
S = ones(1,N+1); S(1) = 0.5; S(N+1) = 0.5; % trapezoidal summation term
m = 0:N; % indexes of trapezoidal summation
for j=0:N
    E(j+1) = sum(S.*sf.eta.*cos(j*m*pi/N)); % equation 3.26
end

% Fenton, J.D., 1999, equation 3.28, p.15
eta = zeros(size(t));
S = 2*S; % trapezoidal summation term
j = 0:N;
omega = sf.omega.*j;
k = sf.k*j;
for j = 1:N+1
    eta = eta + S(j)*E(j)*cos(k(j)*x - omega(j)*t)/N; % equation 3.28
end

% done
end