function s = theta_to_s(theta)

% THETA_TO_SPREAD_COEF
%
% s = theta_to_s(theta);
%
% Converts Gaussian spreading angle 'theta' (radians) to Longuet-Higgins
% spreading parameter 's' (-).
%
% Works analytically to about 6.21 deg (0.1084 radians) of spreading. At
% lower spreading the gamma function blows up and so a numerical
% approximation is used.
%
% References:
%
% H. Mitsuyasu, 1975, Observations of the directional spectrum of ocean
% waves using a cloverleaf buoy, Journal of Physical Oceanography, v5, 750-760.
%
% U.S. Army Engineer Waterways Experiment Station, 1985, Directional wave
% spectra using normal spreading function, CETN-I-6.

% setup
x = (-pi:pi/3600:pi).'; % array of possible angles within 180 at 0.1 resolution
y = 1/sqrt(2*pi)/theta*exp(-x.^2/2/theta^2); % gaussian spreading function per USACE WES CETN-I-6.
theta_min = 6.21003196787545*pi/180; % minimum theta to use gamma function version

if theta == 0
    s = Inf;
elseif theta > theta_min  
    s1 = eps; % lower bound on s
    s2 = 170; % upper bound on s; any higher and s*gamma(s) blows up
    s_guess = fzero(@(s) theta_s(theta,s),[s1 s2]); % solve for the spreading coefficient

    % Complete Longuet-Higgins spreading function (Mitsuyasu, 1975)
    % this version blows up less slowly than gamma(2*s+1)
    % see p.103 of notebook, 3-23-2018, for derivation
    F_lhc = @(s,x) s/2/sqrt(pi)*gamma(s)/gamma(s+1/2)*cos(x/2).^(2*s);
    
    % fit
    f = fit(x,y,F_lhc,'Lower',s1,'Upper',s2,'StartPoint',s_guess);
    s = coeffvalues(f);
else
    s1 = 170; % lower bound on s
    s2 = realmax; % upper bound on s
    s_guess = -1/4/log(cos(theta/2)); % see p.107 of notebook, 4/2/2018
    
    % Approximate Longuet-Higgins spreading function (Mitsuyasu, 1975)
    % this version forces the area = 1 rather than use gamma functions
    F_lha = @(s,x) cos(x/2).^(2*s)/trapz(x,cos(x/2).^(2*s));

    % fit
    f = fit(x,y,F_lha,'Lower',s1,'Upper',s2,'StartPoint',s_guess);
    s = coeffvalues(f);
end
end

function z = theta_s(theta,s)
% derived from 2^(2*s-1)/pi*gamma(s+1)^2/gamma(2*s+1)
% see p.103 of notebook, 3-23-2018, for derivation
z = 1/sqrt(2*pi)/theta - s/2/sqrt(pi)*gamma(s)/gamma(s+1/2);
end