function [t,s,eta,spec] = wavemaker_nl(conditions,facility,varargin)

% WAVEMAKER_NL
%
% [t,s,eta,spec_out] = wavemaker_nl(conditions,facility);
% [t,s,eta,spec_out] = wavemaker_nl(conditions,facility,options);
% [t,s,eta,spec_out] = wavemaker_nl(conditions,facility,options,spec_in);
%
% Generates waves for the specified HWRL facility ('LWF','DWB') for the provided 
% conditions defined by the input structures 'conditions', 'spec_in', and 'options'.
%
% Uses second-order nonlinear wavemaker theory.
%
% Outputs free surface at the wave maker 'eta' (m) and board stroke 's' (m) 
% as a function of time 't' (s). Checks limits of board stroke and applies
% 20 s ramp at start and finish. Also outputs spectral parameters in a 
% structure 'spec' with fields for frequency, angle, and complex amplitude.
%
% The wave conditions structure has fields as follows to define the desired
% waves and board stroke and spectral parameters (not all are required):
%
% wave_type (-). 'REG' is for regular waves, and 'CHIRP' is for a frequency-
% swept cosine regular wave train, requiring two wave periods. 'UPCHIRP' or 
% 'DOWNCHIRP' can also be used to define chirp_direction. 'CUSTOM' is a 
% custom-input timeseries of free surface at the wavemaker piston face. All
% other types are random. 'WHITE' is band-limited Gaussian white noise, 
% requiring two wave periods. 'PINK' is band-limited Gaussian pink noise 
% (1/f multiplied to white noise), also requiring two wave periods. 'PM' is
% Pierson-Moskowitz (Bretschneider) spectral shape. 'JS' is JONSWAP spectral
% shape and 'TMA' is the TMA modified JONSWAP shape.
%
% Wave height H (m). For regular waves this is the crest-to-trough height 
% of the waves and can be more than one height (e.g. 2 values for bichromatic 
% waves). For chirp waves the height is single-valued. For random waves this
% is the spectral significant wave height (Hm0) and is single-valued. For 
% custom timeseries waves this field is ignored.
%
% Wave period T (s). For regular waves this is the crest-to-crest period of 
% the waves and can be more than one period (e.g. 2 values for bichromatic 
% waves). For a chirp, two values are required defining the two periods that
% the chirp sweeps beween. For real sea random waves (types 'PM','JS','TMA')
% this is the spectral peak wave period (Tp) and is single-valued. For band-
% limited noise ('PINK', 'WHITE') two values are required. For custom
% timeseries waves this field is ignored.
%
% Water depth h (m). Still water depth at the wavemaker paddle(s).
%
% Timeseries duration (s). Duration of the waves. Includes a 20s ramp-up 
% and ramp-down time.
%
% Spectral shape factor gm (-). For JONSWAP ('JS') and TMA ('TMA')
% wave types. Defines the relative peakiness/broadness of the spectral
% peak.
%
% Mean wave angle alpha_mean (rad). For regular waves this can be more
% than one angle (e.g. 2 values for 2 regular wave trains at different
% angles). Default value (0) is normal incidence. Cannot exceed pi/3 (60).
% 
% Spreading coefficient 'spread_coef' (-). Ignored for regular waves.
% Input values less than 1 are forced to 1. All inputs are rounded to
% the nearest integer value. Default is no spreading.
%
% Wave phase phi (rad). Starting wave phase. Regular waves only.
%
% Chirp direction chirp_direction (-). Chirp wave sweep direction in frequency,
% from start to end, either 'UP' or 'DOWN'. Defaults to 'UP'. Specifiable 
% in wave_type above ('UPCHIRP' or 'DOWNCHIRP').
%
% eta (m). Free surface elevation. Required for 'CUSTOM'. Ignored otherwise.
%
% t (s). Time in seconds. Optional for 'CUSTOM'. Ignored otherwise.
%
% dt (s). Timestep in seconds. Optional for 'CUSTOM'. Ignored otherwise. 
%
% The 'spec' structure has the format of the output from GENERATE_SPECTRUM
% and is designed to provide input pre-existing spectral data for reuse 
% that can be enabled with the 'options' structure.
%
% The 'options' structure should be defined using WAVEMAKER_OPTIONS with 
% fields designed to treat the input structure. Options default to a newly-
% generated wave condition with random phases and angles. Options are
% ignored for 'CHIRP' and 'CUSTOM' wave types.
%
% References:
% Dean R.G., Dalrymple R.A., 1991, Water wave mechanics for engineers
% and scientists, V2 of Advanced Series on Ocean Engineering, World
% Scientific: Singapore.
%
% Schaffer H.A., 1996, Second-order wavemaker theory for irregular waves,
% Ocean Engineering, v23, n1, pp. 47-88.
%
% Schaffer H.A., Steenberg C.M., 2003, Second-order wavemaker theory for
% multidirectional waves, Ocean Engineering, v30, pp. 1203-1231.
%
% SEE ALSO: WAVEMAKER_REG_2D

disp('wavemaker_nl: starting')

% handle inputs
switch nargin
    case 2
        % no options, no input spectrum
        options = wavemaker_options;
        spec = generate_blank_spectrum(conditions);
    case 3
        % no input spectrum
        options = varargin{1};
        spec = generate_blank_spectrum(conditions);
    otherwise
        options = varargin{1};
        spec = varargin{2};
end

% display wave height, period, depth, duration, if applicable
conditions.wave_type(~isstrprop(conditions.wave_type,'alpha')) = ''; % strip non-alphabet
conditions.wave_type = upper(conditions.wave_type); % force upper-case
disp(['wavemaker_nl: wave type ',conditions.wave_type])
facility(~isstrprop(facility,'alpha')) = ''; % strip non-alphabet
facility = upper(facility); % force upper-
disp(['wavemaker_nl: facility ',facility])
if isfield(conditions,'H')
    Hstring = '';
    for n=1:length(conditions.H)
        if n>1
            Hstring = [Hstring,',']; %#ok<AGROW>
        end
        Hstring = [Hstring,num2str(conditions.H(n))]; %#ok<AGROW>
    end
    disp(['wavemaker_nl: H=',Hstring,' m'])
end
if isfield(conditions,'T')
    Tstring = '';
    for n=1:length(conditions.T)
        if n>1
            Tstring = [Tstring,',']; %#ok<AGROW>
        end
        Tstring = [Tstring,num2str(conditions.T(n))]; %#ok<AGROW>
    end
    disp(['wavemaker_nl: T=',Tstring,' s'])
end
disp(['wavemaker_nl: h=',num2str(conditions.h),' m'])
if isfield(conditions,'duration')
    disp(['wavemaker_nl: duration=',num2str(conditions.duration),' s'])
end

% generate spectra
if strcmp(conditions.wave_type,'REG') % regular waves
    A = conditions.H/2; % amplitudes
    if isfield(conditions,'phi')
        phi = conditions.phi;
        A = A .* exp(1i*phi); % (optional) phases
    else
        phi = zeros(size(A)); % 0 wave phases
    end
    f = 1./conditions.T; % frequencies
    omega = 2*pi*f; % angular frequencies
    Nf = length(f); % number of spectral frequencies
    nf = 1:Nf; % frequencies to use in timeseries generation
    if isfield(conditions,'alpha_mean') && strcmp(facility,'DWB') 
        alpha_mean = conditions.alpha_mean; % mean wave angle(s)
        alpha_max = pi/3; % maximum angle of 60
        if isscalar(alpha_mean) % handle a single-valued alpha_mean (same for all frequencies)
            alpha_mean = alpha_mean * ones(size(f));
        end
        for n=1:length(alpha_mean)
            if abs(alpha_mean(n)) > alpha_max % check that alpha_mean is not too large
                disp(['wavemaker_nl: input wave angle of ',num2str(alpha_mean(n)*180/pi),' exceeds max angle of 60'])
                alpha_mean(n) = sign(alpha_mean(n))*alpha_max;
                disp(['wavemaker_nl: mean angle reduced to',num2str(alpha_mean*180/pi),''])
            end
        end
        alpha = alpha_mean;
        spread_coef = 0; % 0 spreading
        y_board = dwb_wavemaker_drivepoints; % wavemaker paddle locations    
    else % LWF
        alpha_mean = 0; % 0 mean angle
        spread_coef = 0; % 0 spreading
        alpha = zeros(size(f)); % zero wave angle for every frequency
        y_board = 0; % only 1 board position
    end
    spec.new = 1;
    spec.wave_type = conditions.wave_type;
    spec.H = conditions.H;
    spec.T = conditions.T;
    spec.h = conditions.h;
    spec.duration = conditions.duration;
    spec.A = A;
    spec.phi = phi;
    spec.f = f;
    spec.omega = omega;
    spec.Nf = Nf;
    spec.nf = nf;
    spec.alpha = alpha;
    spec.alpha_mean = alpha_mean;
    spec.spread_coef = spread_coef;
    spec.y_board = y_board;
else % irregular waves
    if strcmp(conditions.wave_type,'UPCHIRP')
        conditions.chirp_direction = 'UP'; % this forces a low to high chirp
        conditions.wave_type = 'CHIRP'; % reset to chirp
    elseif strcmp(conditions.wave_type,'DOWNCHIRP')
        conditions.chirp_direction = 'DOWN'; % this forces a high to low chirp
        conditions.wave_type = 'CHIRP'; % reset to chirp
    end
    
    % generate spectrum and wave angles
    spec = generate_spectrum(conditions,options,spec);
end

% linear wavemaker theory
[t,X01,eta1] = wm_core_l(spec);

% check for repeating (extended duration)
repeating = options.repeating && (options.duration_extended > spec.duration);
if repeating
    [t_lt,X01_lt] = wm_pad(t,X01,options.duration_extended); % being careful NOT to overwrite t and X01 at this point
else
    t_lt = t;
    X01_lt = X01;
end

% check limits of linear board stroke time series
switch facility
    case 'LWF'
        limithit = check_lwf_displacement_limits(t_lt,wm_tk_win(t_lt,spec.y_board,X01_lt),0);
    case 'DWB'
        limithit = check_dwb_displacement_limits(t_lt,wm_tk_win(t_lt,spec.y_board,X01_lt),0);
    otherwise % we only have an LWF or a DWB
        disp(['wavemaker_nl: facility ',facility,' not recognized -- aborting'])
        limithit = 1;
end
if limithit
    disp('wavemaker_nl: limit hit on linear time series, aborting')
    t = NaN;
    s = NaN;
    eta = NaN;
    return
end

% second order nonlinear wavemaker theory
[X02_plus,X02_minus,eta2_plus,eta2_minus,spec] = wm_core_nl(spec);

% interpolate random wave timeseries to higher frequency
% regular waves are already at least 50Hz
if ~strcmp(conditions.wave_type,'REG') 
    % setup
    dt = t(2); % timestep
    fs = 1/dt; % sampling rate
    f_min = 50; % at least 50 Hz
    L = ceil(f_min/fs); % resampling factor
    if L > 1 % if we need to resample to get above 50Hz
        N = size(X01,2); % number of timesteps
        M = (N-1)*L+1; % interpolated points
        
        % pad
        eta1_pad = [eta1,eta1(:,2:end-1),eta1];
        eta2_plus_pad = [eta2_plus,eta2_plus(:,2:end-1),eta2_plus];
        eta2_minus_pad = [eta2_minus,eta2_minus(:,2:end-1),eta2_minus];
        X01_pad = [X01,X01(:,2:end-1),X01];
        X02_plus_pad = [X02_plus,X02_plus(:,2:end-1),X02_plus];
        X02_minus_pad = [X02_minus,X02_minus(:,2:end-1),X02_minus];
        N_pad = size(X01_pad,2); % number of padded timesteps
        M_pad = N_pad*L; % padded interpolated points
        
        % interpolate the padded timeseries
        eta1_pad_interp = interpft(eta1_pad,M_pad,2);
        eta2_plus_pad_interp = interpft(eta2_plus_pad,M_pad,2);
        eta2_minus_pad_interp = interpft(eta2_minus_pad,M_pad,2);
        X01_pad_interp = interpft(X01_pad,M_pad,2);
        X02_plus_pad_interp = interpft(X02_plus_pad,M_pad,2);
        X02_minus_pad_interp = interpft(X02_minus_pad,M_pad,2);
        
        % unpad the interpolated timeseries
        eta1_interp = eta1_pad_interp(:,M:2*M-1);
        eta2_plus_interp = eta2_plus_pad_interp(:,M:2*M-1);
        eta2_minus_interp = eta2_minus_pad_interp(:,M:2*M-1);
        X01_interp = X01_pad_interp(:,M:2*M-1);
        X02_plus_interp = X02_plus_pad_interp(:,M:2*M-1);
        X02_minus_interp = X02_minus_pad_interp(:,M:2*M-1);
        
        % reassign interpolated to original
        eta1 = eta1_interp;
        eta2_plus = eta2_plus_interp;
        eta2_minus = eta2_minus_interp;
        X01 = X01_interp;
        X02_plus = X02_plus_interp;
        X02_minus = X02_minus_interp;
        
        % adjust time
        t = linspace(0,spec.duration,M); % new time field
    end
end

% check for repeating (extended duration)
repeating = options.repeating && (options.duration_extended > spec.duration);
if repeating
    disp(['wavemaker_nl: padding timeseries for extended duration of ',num2str(options.duration_extended),'s'])
    [~,X01] = wm_pad(t,X01,options.duration_extended);
    [~,X02_plus] = wm_pad(t,X02_plus,options.duration_extended);
    [~,X02_minus] = wm_pad(t,X02_minus,options.duration_extended);
    [~,eta1] = wm_pad(t,eta1,options.duration_extended);
    [~,eta2_plus] = wm_pad(t,eta2_plus,options.duration_extended);
    [t,eta2_minus] = wm_pad(t,eta2_minus,options.duration_extended);
end

% check limits of board stroke
disp('wavemaker_nl: checking displacement limits of second order fully nonlinear board stroke')
switch facility
    case 'LWF'
        limithit = check_lwf_displacement_limits(t,wm_tk_win(t,spec.y_board,X01+X02_plus+X02_minus),0);
    case 'DWB'
        limithit = check_dwb_displacement_limits(t,wm_tk_win(t,spec.y_board,X01+X02_plus+X02_minus),0);
    otherwise % we only have an LWF or a DWB
        disp(['wavemaker_nl: facility ',facility,' not recognized -- aborting'])
        limithit = 1;
end
if limithit
    disp('wavemaker_nl: limit hit on second order fully nonlinear board stroke')
    disp('wavemaker_nl: checking displacement limits of second order board stroke w/o bound long waves')
    % try again without bound long waves
    switch facility
        case 'LWF'
            limithit = check_lwf_displacement_limits(t,wm_tk_win(t,spec.y_board,X01+X02_plus),0);
        case 'DWB'
            limithit = check_dwb_displacement_limits(t,wm_tk_win(t,spec.y_board,X01+X02_plus),0);
    end
    if limithit
        disp('wavemaker_nl: limit hit on second order nonlinear time series')
        disp('wavemaker_nl: returning linear time series')
        s = wm_tk_win(t,spec.y_board,X01); % linear stroke
        eta = wm_tk_win(t,spec.y_board,eta1); % linear free surface
    else
        disp('wavemaker_nl: returning second order time series w/o bound long waves')
        s = wm_tk_win(t,spec.y_board,X01+X02_plus); % second order board stroke w/o bound long waves
        eta = wm_tk_win(t,spec.y_board,eta1+eta2_plus); % free surface w/o bound long waves
    end
else
    disp('wavemaker_nl: second order fully nonlinear time series passed')
    s = wm_tk_win(t,spec.y_board,X01+X02_plus+X02_minus); % second order fully nonlinear board stroke
    eta = wm_tk_win(t,spec.y_board,eta1+eta2_plus+eta2_minus); % second order fully nonlinear free surface
end

% done
disp('wavemaker_nl: done')
end

function spec = generate_blank_spectrum(conditions)
% generates a blank (baseline) spectrum with all fields filled in based on
% the input conditions
spec.new = 1;
spec.wave_type = conditions.wave_type;
if isfield(conditions,'H')
    spec.H = conditions.H;
end
if isfield(conditions,'T')
    spec.T = conditions.T;
end
if isfield(conditions,'h')
    spec.h = conditions.h;
end
if isfield(conditions,'duration')
    spec.duration = conditions.duration;
end
if isfield(conditions,'gm')
    spec.gm = conditions.gm;
end
if isfield(conditions,'chirp_direction')
    spec.chirp_direction = conditions.chirp_direction;
end
if isfield(conditions,'eta')
    spec.eta = conditions.eta;
end
if isfield(conditions,'t')
    spec.t = conditions.t;
end
if isfield(conditions,'dt')
    spec.dt = conditions.dt;
end
end

function f_win = wm_tk_win(t,y_b,f)
% tukey window on first and last 20 seconds of 'f' as a function of 't' and 'y_b'
r = 40/t(end); % tukeywin tapers r/2, so input 40 seconds
r_win = repmat(tukeywin(length(t),r).',length(y_b),1);
f_win = f.*r_win;
end

function [t_long,f_long] = wm_pad(t,f,duration_extended)
% Takes input time 't' (s), and 'f' and repeats them over and over 
% (circular padding) out to the input 'duration_extended' (s).
N = length(t)-1; % assume t and f are the same length
duration_short = t(N+1);
L = duration_extended/duration_short; % multiplication factor
M = L*N; % number of timesteps in new records
t_long = linspace(0,duration_extended,M+1); % lengthened time
f_long = padarray(f(1:N),[0,M-N+1],'circular','post'); % lengthened f
end

function y_board = dwb_wavemaker_drivepoints
% Returns the y locations of the drive points for the DWB wavemaker
% paddles.  Observations collected in 2008 using a measuring tape.
y_board = [-13.2;-12.345;-11.426;-10.515;-9.6;-8.685;-7.77;-6.855; ...
    -5.945;-5.03;-4.11;-3.2;-2.285;-1.37;-0.455;0.46;1.375;2.29;3.2; ...
    4.12;5.03;5.95;6.86;7.78;8.69;9.605;10.52;11.44;12.35;13.195];
end
