function [t,s,eta] = wavemaker_reg_2d(facility,H,T,h,varargin)

% WAVEMAKER_REG_2D
%
% [t,s,eta] = wavemaker_reg_2d({'LWF'|'DWB'},H,T,h);
% [t,s,eta] = wavemaker_reg_2d({'LWF'|'DWB'},H,T,h,duration);
% [t,s,eta] = wavemaker_reg_2d({'LWF'|'DWB'},H,T,h,duration,{'LIN'|'STK'|'CN'|'SF'});
%
% Generates regular waves for the specified HWRL facility ('LWF','DWB') 
% for the provided conditions. 
%
% Outputs free surface at the wave maker 'eta' (m) and board stroke 's' (m)
% as a function of time 't' (s).  Checks limits of board stroke and applies
% 20 s ramp at start and finish.  Wave conditions are specified as follows:
%
% Wave height H (m).  This is the crest-to-trough height of the waves.
%
% Wave period T (s).  This is the crest-to-crest period of the waves.
%
% Water depth h (s).  Still water depth at the wavemaker paddle(s).
%
% Wave duration (s).  Duration of the waves.  Includes a total of 40s of
% ramp-up and ramp-down time (20s up, 20s down).  If not specified, duration 
% is 20 wave periods plus 40s.
%
% Wave type.  Accepted inputs are 'LIN' for classic linear wavemaker
% theory, 'STK' for fully nonlinear second-order Stokes wavemaker theory,
% 'CN' for cnoidal wavemaker theory, and 'SF' for approximate stream function
% wavemaker theory. If not specified, wave type is automatically chosen.
%
% References:
% Dean R.G., Dalrymple R.A., 1991. Water wave mechanics for engineers
% and scientists, V2 of Advanced Series on Ocean Engineering, World
% Scientific: Singapore.
%
% Fenton J.D., 1999. The cnoidal theory of water waves, Developements
% in offshore engineering, Chapter 2, Ed. J.B. Herbich, Gulf: Houston.
%
% Fenton J.D., 1999. Numerical methods for nonlinear waves, in 
% Advances in Coastal and Ocean Engineering, V5, ed. P.L.-F. Liu, p241-324,
% World Scientific: Singapore.
%
% Hughes S.A., 1993. Physical models and laboratory techniques 
% in coastal engineering. Advanced Series on Ocean Engineering, Vol. 7, 
% World Scientific Publishing, Singapore.
%
% Schaffer H.A., 1993. Second order irregular wave generation
% in flumes -- Computation of transfer functions by an asymptotic summation
% method.  Proc. Waves '93: 2nd Int. Symp. Ocean Wave Measurement and
% Analysis, New Orleans, LA.
%
% Schaffer H.A., 1996. Second-order wavemaker theory for irregular waves,
% Ocean Engineering, v23, n1, pp. 47-88.
%
% Svendsen I.A., 2006. Introduction to nearshore hydrodynamics.
% Advanced Series on Ocean Engineering, Vol. 24, World Scientific Publishing,
% Singapore.
%
% Zhang H., Schaffer H.A., 2007. Approximate stream function wavemaker
% theory for highly non-linear waves in wave flumes, Ocean Engineering, 34,
% p1290-1302.
%
% SEE ALSO: CNOIDAL_DISPLACEMENT, LINEAR_DISPLACEMENT, STOKES_DISPLACEMENT, STREAM_FUNCTION_DISPLACEMENT

% handle inputs
facility(~isstrprop(facility,'alpha')) = ''; % strip non-alphabet
facility = upper(facility); % force upper-
disp(['wavemaker_reg_2d: facility ',facility])
disp(['wavemaker_reg_2d: H=',num2str(H),' m'])
disp(['wavemaker_reg_2d: T=',num2str(T),' s'])
disp(['wavemaker_reg_2d: h=',num2str(h),' m'])
if ~isempty(varargin) % handle 1st extra input
    duration = varargin{1}; % duration
    disp(['wavemaker_reg_2d: duration=',num2str(duration),' s'])
    if length(varargin)>1 % handle 2nd extra input
        wave_type = varargin{2}; % wave_type
        wave_type(~isstrprop(wave_type,'alpha')) = ''; % strip non-alphabet
        wave_type = upper(wave_type); % force upper-
        auto_wave_type = 0;
        disp(['wavemaker_reg_2d: wave type ',wave_type])
    else % no 2nd input, automatically determine wave_type
        auto_wave_type = 1;
        disp('wavemaker_reg_2d: wave type to be chosen automatically')
    end
else % no extra inputs
    duration = 20*T + 40; % duration 20 waves plus 40s ramp time
    disp(['wavemaker_reg_2d: duration=',num2str(duration),' s'])
    auto_wave_type = 1;
    disp('wavemaker_reg_2d: wave type to be chosen automatically')
end % handle first input

if auto_wave_type % choose a wave type
    disp('wavemaker_reg_2d: choosing wave type')
    omega = 2*pi/T; % wave frequency
    k = dispersion_p(h,omega); % wavenumber from dispersion
    A = H/2;
    
    % check for validity of 2nd order Stokes with nonlinearity parameter S
    % valid wavenumbers are those with no trough bumps
    S = abs(k*A*cosh(k*h)*(2+cosh(2*k*h))/sinh(k*h)^3); % nonlinearity parameter S, Dean&Dalrymple, 1991, equation 11.41
    
    if S<=1 % Schaffer, 1996, equation 52
        wave_type = 'STK'; % Stokes is valid
    else % Stokes is invalid
        g = gravity; % gravity
        L = 2*pi/k; % wavenumber
        U = H*L^2/h^3; % Ursell number, Svendsen equation 7.5.13, and Fenton p.14, among others
        p = L/h;
        H_max = h*(0.141063*p + 0.0095721*p^2 + 0.0077829*p^3)...
            /(1 + 0.0788340*p + 0.0317567*p^2 + 0.0093407*p^3); % Fenton (1990), equation 6.1
        if T < 7.256*sqrt(h/g) % cnoidal invalid, h/L0 > 3/8/pi, Svendsen equation 9.6.56
            % cnoidal and Stokes both invalid
            if U > 100
                % testing shows that stream function fails for higher Ursell numbers
                % this should be impossible
                disp('wavemaker_reg_2d: WARNING should be impossible to get U>100, h/L0 > 3/8/pi, S>1')
                wave_type = 'LIN'; % linear; stream function invalid
            elseif H > H_max
                disp('wavemaker_reg_2d: WARNING H larger than maximum theoretical height')
                wave_type = 'LIN'; % linear; stream function invalid
            else
                wave_type = 'SF'; % stream function is valid
            end
        else
            % cnoidal valid; Stokes invalid
            if U > 100 % testing shows that stream function fails for higher Ursell numbers
                wave_type = 'CN'; % cnoidal; stream function invalid
            else
                wave_type = 'SF'; % stream function is valid
            end
        end
    end
    disp(['wavemaker_reg_2d: wave type ',wave_type])
end % auto_wave_type

% generate waves
switch wave_type
    case 'LIN' % linear
        [t,s,eta] = linear_displacement(H,T,h,duration);
    case 'STK' % Stokes
        [t,s,eta] = stokes_displacement(H,T,h,duration);  
    case 'CN' % cnoidal
        [t,s,eta] = cnoidal_displacement(H,T,h,duration);
    case 'SF' % stream function
        [t,s,eta] = stream_function_displacement(H,T,h,duration);
    otherwise
        disp(['wavemaker_reg_2d: wave type ',wave_type,' not recognized -- aborting'])
        t=NaN; s=NaN; eta=NaN;
        return
end

% check limits of board stroke time series
r = 40/duration; % tukeywin tapers r/2, so input 40 seconds
r_win = tukeywin(length(t),r)';
s = s.*r_win;
eta = eta.*r_win;
switch facility
    case 'LWF'
        limithit = check_lwf_displacement_limits(t,s,0);
    case 'DWB'
        limithit = check_dwb_displacement_limits(t,s,0);
    otherwise % we only have an LWF or a DWB
        disp(['wavemaker_reg_2d: facility ',facility,' not recognized -- aborting'])
        t=NaN; s=NaN; eta=NaN;
        return
end
if limithit
    if strcmp(wave_type,'LIN')
        % we hit a limit on linear waves, nothing we can do
        t = NaN;
        s = NaN;
        eta = NaN;
    else
        % we hit a limit on a nonlinear wave type
        % attempt once more with linear waves
        [t,s,eta] = linear_displacement(H,T,h,duration);
        r = 40/duration; % tukeywin tapers r/2, so input 40 seconds
        r_win = tukeywin(length(t),r)';
        s = s.*r_win;
        eta = eta.*r_win;
        switch facility
            case 'LWF'
                limithit = check_lwf_displacement_limits(t,s,0);
            case 'DWB'
                limithit = check_dwb_displacement_limits(t,s,0);
            otherwise % we only have an LWF or a DWB
                disp(['wavemaker_reg_2d: facility ',facility,' not recognized -- aborting'])
                t=NaN; s=NaN; eta=NaN;
                return
        end
        if limithit
            % we hit a limit on linear waves, nothing we can do
            t = NaN;
            s = NaN;
            eta = NaN;
        end
    end
end

% done
disp('wavemaker_reg_2d: done')
