function [t,X01,eta1] = wm_core_l(spec)

% WM_CORE_L
%
% [t,X01,eta1] = wm_core_l(spec);
%
% Linear wavemaker theory.  Subroutine of wavemaker_nl.
%
% References:
% Dean, R.G., Dalrymple, R.A., 1991, Water wave mechanics for engineers
% and scientists, V2 of Advanced Series on Ocean Engineering, World
% Scientific: Singapore.
%
% Schaffer H.A., 1996, Second-order wavemaker theory for irregular waves,
% Ocean Engineering, v23, n1, pp. 47-88.
%
% Schaffer H.A., Steenberg C.M., 2003, Second-order wavemaker theory for
% multidirectional waves, Ocean Engineering, v30, pp. 1203-1231.

% unpack spec structure
wave_type = spec.wave_type;
h = spec.h;
duration = spec.duration;
A = spec.A;
omega = spec.omega;
Nf = spec.Nf;
alpha = spec.alpha;
y_board = spec.y_board;

% dispersion
N_evanescent = 100; % number of evanescent modes
k = dispersion_complex(h,omega,N_evanescent);
k0 = k(1,:); % progressive

% Biesel coefficients (height-to-stroke ratio)
cj = 2*(cosh(2*k*h) - 1)./(sinh(2*k*h) + 2*k*h); % Schaffer & Steenberg, 2003, equation 43
cj(k*h>20) = 2; % asymptotic limit for deep water
c0 = cj(1,:); % progressive

% directional Biesel coefficients e
ej = zeros(size(cj));
ej(1,:) = c0 ./ cos(alpha); % Schaffer & Steenberg, 2003, equation 49
k0_sin_alpha = repmat(k(1,:).*sin(alpha),N_evanescent,1);
cos_alphaj = sqrt(1 + k0_sin_alpha.^2 ./ abs(k(2:end,:)).^2); % Schaffer & Steenberg, 2003, equation 53
ej(2:end,:) = cj(2:end,:) ./ cos_alphaj; % Schaffer & Steenberg, 2003, equation 52
e0 = ej(1,:); % progressive
ee = sum(ej(2:end,:)); % evanescent

% board stroke
Xa = A ./ e0; % Schaffer & Steenberg, 2003, equation 47

% generate free surface and board stroke time series
disp('wm_core_l: generating linear time series')
pause(0.1)
Y = length(y_board);
if strcmp(wave_type,'REG') % regular waves
    fs_min = 50; % at least 50Hz
    N = 2^nextpow2(duration * fs_min); % number of timesteps
    t = linspace(0,duration,N); % time
    eta1 = zeros(Y,N);
    X01 = zeros(Y,N);
    for nt=1:N
        for n_board = 1:Y % for each board
            % theta (phase)
            theta = omega*t(nt) - k0.*sin(alpha)*y_board(n_board);
            
            % free surface (Schaffer & Steenberg, 2003, equation 38)
            arg = (1 + ee./e0) .* A .* exp(1i*theta);
            eta1(n_board,nt) = 0.5*sum(arg + conj(arg));
            
            % board stroke (Schaffer & Steenberg, 2003, equation 36)
            arg = -1i*Xa .* exp(1i*theta);
            X01(n_board,nt) = 0.5*sum(arg + conj(arg));
        end
    end
else % random waves
    N = 2*(Nf-1); % number of timesteps
    t = linspace(0,duration,N+1); % time
    eta1 = zeros(Y,N+1);
    X01 = zeros(Y,N+1);
    for n_board = 1:length(y_board) % for each board
        % theta (phase)
        theta = -k0.*sin(alpha)*y_board(n_board);
        
        % free surface (Schaffer & Steenberg, 2003, equation 38)
        arg = (1 + ee./e0) .* A .* exp(1i*theta);
        arg = [0, arg(2:Nf-1), 2*real(arg(Nf)), conj(arg(Nf-1:-1:2))];
        eta1(n_board,1:N) = real(N/2*ifft(arg)); % generate timeseries
        
        % board stroke (Schaffer & Steenberg, 2003, equation 36)
        arg = -1i*Xa .* exp(1i*theta);
        arg = [0, arg(2:Nf-1), 2*real(arg(Nf)), conj(arg(Nf-1:-1:2))];
        X01(n_board,1:N) = real(N/2*ifft(arg)); % generate timeseries
    end
    eta1(:,end) = eta1(:,1); % last timestep is same as first
    X01(:,end) = X01(:,1); % last timestep is same as first
end

end