function [X02_plus,X02_minus,eta2_plus,eta2_minus,spec] = wm_core_nl(spec)

% WM_CORE_NL
%
% [X02_plus,X02_minus,eta2_plus,eta2_minus,spec] = wm_core_nl(spec);
%
% Nonlinear (second order Stokes) wavemaker theory.  Subroutine of wavemaker_nl.
%
% References:
% Andersen T.L., Frigaard P., 2014. Wave generation in physical models.
% DCE Technical Reports, Aalborg University.
%
% Dean, R.G., Dalrymple, R.A., 1991, Water wave mechanics for engineers
% and scientists, V2 of Advanced Series on Ocean Engineering, World
% Scientific: Singapore.
%
% Schaffer H.A., 1996, Second-order wavemaker theory for irregular waves,
% Ocean Engineering, v23, n1, pp. 47-88.
%
% Schaffer H.A., Steenberg C.M., 2003, Second-order wavemaker theory for
% multidirectional waves, Ocean Engineering, v30, pp. 1203-1231.

% unpack spec structure
wave_type = spec.wave_type;
h = spec.h;
duration = spec.duration;
nf = spec.nf;
Nf = spec.Nf;
A = spec.A(nf);
omega = spec.omega(nf);
alpha = spec.alpha(nf);
y_board = spec.y_board;

% dispersion
N_evanescent = 15; % number of evanescent modes
k = dispersion_complex(h,omega,N_evanescent);

% valid wavenumbers are those with no trough bumps
S = abs(k(1,:).*A.*cosh(k(1,:)*h).*(2+cosh(2*k(1,:)*h))./sinh(k(1,:)*h).^3); % nonlinearity parameter, Dean&Dalrymple, 1991, equation 11.41
if ~isempty(find(S<=1,1)) % Schaffer, 1996, equation 52
    disp('wm_core_nl: generating 2nd order terms')
    
    % second order n,m terms
    [An,Am]=meshgrid(A); % amplitudes
    [omega_n,omega_m] = meshgrid(omega); % frequencies
    [k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
    [alpha_n,alpha_m] = meshgrid(alpha); % angles

    % second order sum and difference wavenumbers
    [K0_plus,K0_minus] = dispersion_sumdif(h,omega);
    K0_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits 
    
    % second order transfer functions
    missing_FG = ~isfield(spec,'F_nm_plus') || ~isfield(spec,'F_nm_minus') ...
        || ~isfield(spec,'G_nm_plus') || ~isfield(spec,'G_nm_minus');
    if spec.new || missing_FG
        % we need to calculate them
        tol = 1e-2; % error/convergence tolerance on asymptotic estimates
        [F_nm_plus,G_nm_plus] = FG_plus(omega,k,alpha,K0_plus,h,tol);
        [F_nm_minus,G_nm_minus] = FG_minus(omega,k,alpha,K0_minus,h,tol);
        
        % save values into spec
        spec.F_nm_plus = F_nm_plus;
        spec.G_nm_plus = G_nm_plus;
        spec.F_nm_minus = F_nm_minus;
        spec.G_nm_minus = G_nm_minus;
   else
        % we can reuse previous calculation
        F_nm_plus = spec.F_nm_plus;
        G_nm_plus = spec.G_nm_plus;
        F_nm_minus = spec.F_nm_minus;
        G_nm_minus = spec.G_nm_minus;
    end
    
    % check on applicability
    S_plus = abs(4*sqrt(An.*Am).*G_nm_plus);
    F_nm_plus(S_plus>1) = 0;
    G_nm_plus(S_plus>1) = 0;
    S_minus = abs(4*sqrt(An.*Am).*G_nm_minus);
    F_nm_minus(S_minus>1) = 0;
    G_nm_minus(S_minus>1) = 0;
    
    % second order free surface and board stroke
    disp('wm_core_nl: generating 2nd order nonlinear time series')
    pause(0.1)
    Y = length(y_board); % number of wave boards
    if strcmp(wave_type,'REG') % regular waves
        fs_min = 50; % at least 50Hz
        N = 2^nextpow2(duration * fs_min); % number of timesteps
        t = linspace(0,duration,N); % time
        X02_plus = zeros(Y,N);
        X02_minus = zeros(Y,N);
        eta2_plus = zeros(Y,N);
        eta2_minus = zeros(Y,N);
        for nt=1:N
            for n_board = 1:Y % for each board
                % theta phases (Schaffer & Steenberg, 2003, equation 57)
                theta_n = omega_n*t(nt) - k_0n.*sin(alpha_n)*y_board(n_board);
                theta_m = omega_m*t(nt) - k_0m.*sin(alpha_m)*y_board(n_board);
                
                % free surface (Schaffer & Steenberg, 2003, equation 68)
                arg_plus = G_nm_plus.*An.*Am .* exp(1i*(theta_n + theta_m));
                arg_minus = G_nm_minus.*An.*conj(Am) .* exp(1i*(theta_n - theta_m));
                eta2_plus(n_board,nt) = 0.5*sum(sum(arg_plus + conj(arg_plus)));
                eta2_minus(n_board,nt) = 0.5*sum(sum(arg_minus + conj(arg_minus)));
                
                % board stroke (Schaffer & Steenberg, 2003, equation 85)
                arg_plus = -1i*F_nm_plus.*An.*Am/h .* exp(1i*(theta_n + theta_m));
                arg_minus = -1i*F_nm_minus.*An.*conj(Am)/h .* exp(1i*(theta_n - theta_m));
                X02_plus(n_board,nt) = 0.5*sum(sum(arg_plus + conj(arg_plus)));
                X02_minus(n_board,nt) = 0.5*sum(sum(arg_minus + conj(arg_minus)));
            end
        end
    else % random waves
        N = 2*(Nf-1); % number of timesteps
        X02_plus = zeros(Y,N+1);
        X02_minus = zeros(Y,N+1);
        eta2_plus = zeros(Y,N+1);
        eta2_minus = zeros(Y,N+1);
        for n_board = 1:length(y_board) % for each board
            for m = 1:length(nf) % for each m frequency
                % theta phases (Schaffer & Steenberg, 2003, equation 57)
                theta_n = -k_0n(m,:).*sin(alpha_n(m,:))*y_board(n_board);
                theta_m = -k_0m(m,:).*sin(alpha_m(m,:))*y_board(n_board);
                
                % find applicable frequency bands
                n_start_plus = find(spec.f == spec.f(nf(m))*2);
                n_end_plus = n_start_plus + length(nf) - m;
                n_band_plus = n_start_plus:n_end_plus;
                n_start_minus = 2; % skip zero bin
                n_end_minus = n_start_minus + length(nf) - m - 1;
                n_band_minus = n_start_minus:n_end_minus;
               
                % free surface (Schaffer & Steenberg, 2003, equation 68)
                % plus (sum)
                arg_plus = zeros(size(spec.f)); % zero-pad with all frequencies
                arg_plus(n_band_plus) = G_nm_plus(m,m:end).*An(m,m:end).*Am(m,m:end) .* exp(1i*(theta_n(m:end) + theta_m(m:end))); % equation 68
                arg_plus = [arg_plus(1), arg_plus(2:Nf-1), 2*real(arg_plus(Nf)), conj(arg_plus(Nf-1:-1:2))];
                eta2_plus(n_board,1:N) = eta2_plus(n_board,1:N) + real(N/2*ifft(arg_plus)); % generate timeseries
                % minus (difference)
                arg_minus = zeros(size(spec.f)); % zero-pad with all frequencies
                arg_minus(n_band_minus) = G_nm_minus(m,m+1:end).*An(m,m+1:end).*conj(Am(m,m+1:end)) .* exp(1i*(theta_n(m+1:end) - theta_m(m+1:end))); % equation 68
                arg_minus = [arg_minus(1), arg_minus(2:Nf-1), 2*real(arg_minus(Nf)), conj(arg_minus(Nf-1:-1:2))];
                eta2_minus(n_board,1:N) = eta2_minus(n_board,1:N) + real(N/2*ifft(arg_minus)); % generate timeseries
              
                % board stroke (Schaffer & Steenberg, 2003, equation 85)
                % plus (sum)
                arg_plus = zeros(size(spec.f)); % zero-pad with all frequencies
                arg_plus(n_band_plus) = -1i*F_nm_plus(m,m:end).*An(m,m:end).*Am(m,m:end)/h .* exp(1i*(theta_n(m:end) + theta_m(m:end))); % equation 85
                arg_plus = [arg_plus(1), arg_plus(2:Nf-1), 2*real(arg_plus(Nf)), conj(arg_plus(Nf-1:-1:2))];
                X02_plus(n_board,1:N) = X02_plus(n_board,1:N) + real(N/2*ifft(arg_plus)); % generate timeseries
                % minus (difference)
                arg_minus = zeros(size(spec.f)); % zero-pad with all frequencies
                arg_minus(n_band_minus) = -1i*F_nm_minus(m,m+1:end).*An(m,m+1:end).*conj(Am(m,m+1:end))/h .* exp(1i*(theta_n(m+1:end) - theta_m(m+1:end))); % equation 85
                arg_minus = [arg_minus(1), arg_minus(2:Nf-1), 2*real(arg_minus(Nf)), conj(arg_minus(Nf-1:-1:2))];
                X02_minus(n_board,1:N) = X02_minus(n_board,1:N) + real(N/2*ifft(arg_minus)); % generate timeseries
            end
        end
        eta2_plus(:,end) = eta2_plus(:,1); % last timestep is same as first
        eta2_minus(:,end) = eta2_minus(:,1); % last timestep is same as first
        X02_plus(:,end) = X02_plus(:,1); % last timestep is same as first
        X02_minus(:,end) = X02_minus(:,1); % last timestep is same as first
    end
else
    disp('wm_core_nl: not possible to generate valid second-order Stokes terms')
    Y = length(y_board); % number of wave boards
    if strcmp(wave_type,'REG') % regular waves
        fs_min = 50; % at least 50Hz
        N = 2^nextpow2(duration * fs_min); % number of timesteps
        X02_plus = zeros(Y,N);
        X02_minus = zeros(Y,N);
        eta2_plus = zeros(Y,N);
        eta2_minus = zeros(Y,N);
    else % random waves
        N = 2*(Nf-1); % number of timesteps
        X02_plus = zeros(Y,N+1);
        X02_minus = zeros(Y,N+1);
        eta2_plus = zeros(Y,N+1);
        eta2_minus = zeros(Y,N+1);
    end
end
end

function [K0_plus,K0_minus] = dispersion_sumdif(h,omega)
% Numerically solves the linear dispersion relationship for water waves. 
% Solves two sets of equations, one for sums:
% (omega_n + omega_m)^2 = g*K0_plus*tanh(K0_plus*h)
%
% and one for differences:
% (omega_n - omega_m)^2 = g*K0_minus*tanh(K0_minus*h)
%
% for all values of n,m = 1:length(omega) where omega = 2*pi./T (vector);
% K0_plus and K0_minus are the nxn matrices of sum/difference wavenumbers;
% g is gravity; h is water depth (scalar).
%
% Iterates using Newton-Raphson. Uses a nondimensional formulation by 
% Soulsby (2006) where x=omega.^2*h/g; y=k*h; therefore x = y * tanh(y)

% setup
[omega_n,omega_m] = meshgrid(omega);
omega_plus = omega_n + omega_m;
omega_plus = triu(omega_plus); % symmetric; take only the upper triangular part
omega_minus = omega_n - omega_m;
omega_minus = triu(omega_minus,1); % symmetric; take only the upper triangular part, eliminating n=m

% nondimensional frequency
g = gravity; % gravity
x_plus = omega_plus.^2*h/g;
x_minus = omega_minus.^2*h/g;

% initial guess
y_plus0 = sqrt(x_plus).*(x_plus<1) + x_plus.*(x_plus>=1);
y_plus0 = triu(y_plus0); % symmetric; take only the upper triangular part
y_minus0 = sqrt(x_minus).*(x_minus<1) + x_minus.*(x_minus>=1);
y_minus0 = triu(y_minus0,1); % symmetric; take only the upper triangular part, eliminating n=m

% solve
y_plus = dispersion_solver(x_plus,y_plus0);
y_plus = triu(y_plus); % symmetric; take only the upper triangular part
y_minus = dispersion_solver(x_minus,y_minus0);
y_minus = triu(y_minus,1); % symmetric; take only the upper triangular part, eliminating n=m

% done
K0_plus = y_plus/h; % y = kh
K0_minus = y_minus/h; % y = kh
end

function y = dispersion_solver(x,y0)
% newton-raphson solver per Soulsby
y = y0;
t = tanh(y);
f = y.*t - x;
dfdy = t + y.*(1-t.^2);
y1 = y - f./dfdy;
dy = y1-y;
y = y1;
count = 1;
while any(abs(dy(:))>3*eps(y(:))) && any(abs(f(:))>3*eps(x(:))) && count < 10
    t = tanh(y);
    f = y.*t - x;
    dfdy = t + y.*(1-t.^2);
    y1 = y - f./dfdy;
    dy = y1-y;
    y = y1;
    count = count + 1;
end
end

function [F_nm_plus,G_nm_plus] = FG_plus(omega,k,alpha,K0_plus,h,tol)
% Calculates transfer functions for positive (sums of different frequencies)
% 2nd order nonlinear wave-wave interactions.  F_nm_plus is the board
% stroke transfer function and G_nm_plus is the free surface transfer
% function. Uses asymptotic estimation.
%
% Takes inputs omega (vector), k (matrix, from dispersion_complex), alpha
% (vector), K0_plus (matrix, from dispersion_sumdif), and h (scalar).

% setup
disp('FG_plus: transfer function for positive (sum) 2nd order nonlinear wave-wave interactions')
g = gravity; % gravity
[Ne,Nf] = size(k); % number of evanescent modes, number of frequencies
[omega_n,omega_m] = meshgrid(omega);

% delta_nm (equation 56)
% all equation numbers are from Schaffer & Steenberg, 2003
% unless otherwise noted
disp('FG_plus: calculating delta_nm')
delta_nm = ones(Nf,Nf);
for n=1:Nf
    delta_nm(n,n) = 0.5;
end

% Biesel coefficients (height-to-stroke ratio)
disp('FG_plus: calculating cj')
cj = 2*(cosh(2*k*h) - 1)./(sinh(2*k*h) + 2*k*h); % equation 43
cj(k*h>20) = 2; % asymptotic limit for deep water
cj(k==0) = 0; % handle any zero wavenumbers
c0 = cj(1,:); % progressive

% directional Biesel coefficients e
disp('FG_plus: calculating ej')
ej = zeros(size(cj));
ej(1,:) = c0 ./ cos(alpha); % equation 49
k0_sin_alpha = repmat(k(1,:).*sin(alpha),Ne-1,1);
cos_alphaj = sqrt(1 + k0_sin_alpha.^2 ./ abs(k(2:end,:)).^2); % equation 53
ej(2:end,:) = cj(2:end,:) ./ cos_alphaj; % equation 52

% E_plus (equation 93)
% note that M1 = 0 for a piston
disp('FG_plus: calculating E_plus')
[e0_n,e0_m] = meshgrid(ej(1,:),ej(1,:));
E_plus = delta_nm.*K0_plus.^2*h./(e0_n.*e0_m.*(omega_n+omega_m).^3);
E_plus(~isfinite(E_plus)) = 0; % handle any singularities
E_plus = triu(E_plus); % symmetric; take only the upper triangular part

% G_nm_plus, for which we only need progressive part
disp('FG_plus: calculating G_nm_plus')
[k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
[alpha_n,alpha_m] = meshgrid(alpha); % angles
k_yn = k_0n.*sin(alpha_n); % equation 50, note k_yn has no imaginary (evanescent) components
k_ym = k_0m.*sin(alpha_m); % equation 50, note k_ym has no imaginary (evanescent) components
k_x0n = k_0n.*cos(alpha_n); % equations 51 and 53 for j=l=0
k_x0m = k_0m.*cos(alpha_m); % equations 51 and 53 for j=l=0
k_0n_dot_k_0m = k_x0n.*k_x0m + k_yn.*k_ym; % dot product in H_nm_plus and L_nm_plus

% H_nm_plus (equation 58, where j=l=0)
H_nm_plus = (omega_n + omega_m).*(omega_n.*omega_m - g^2*k_0n_dot_k_0m./(omega_n.*omega_m)) ...
    + 0.5*(omega_n.^3+omega_m.^3) - 0.5*g^2*(k_0n.^2./omega_n+k_0m.^2./omega_m);
H_nm_plus(~isfinite(H_nm_plus)) = 0; % handle any singularities
H_nm_plus = triu(H_nm_plus); % symmetric; take only the upper triangular part

% D_nm_plus
k_nm_plus = sqrt((k_x0n + k_x0m).^2 + (k_yn + k_ym).^2); % (equation 62, where j=l=0)
D_nm_plus = g*k_nm_plus.*tanh(k_nm_plus*h) - (omega_n+omega_m).^2; % (equation 63, where j=l=0)
D_nm_plus(~isfinite(D_nm_plus)) = 0; % handle any singularities
D_nm_plus = triu(D_nm_plus); % symmetric; take only the upper triangular part

% L_nm_plus (equation 65, where j=l=0)
L_nm_plus = 0.5*(g^2*k_0n_dot_k_0m./(omega_n.*omega_m) - omega_n.*omega_m - (omega_n.^2 + omega_m.^2)); 
L_nm_plus(~isfinite(L_nm_plus)) = 0; % handle any singularities
L_nm_plus = triu(L_nm_plus); % symmetric; take only the upper triangular part

% G_nm_plus (equation 67, where j=l=0)
% which yields eta_0_(21plus)
% desired progressive second order bound waves
G_nm_plus = delta_nm/g.*((omega_n+omega_m).*H_nm_plus./D_nm_plus - L_nm_plus);
G_nm_plus(~isfinite(G_nm_plus)) = 0; % handle any singularities
G_nm_plus = triu(G_nm_plus);  % symmetric; take only the upper triangular part

% single-summation terms
disp('FG_plus: calculating F_nm_plus single-summation terms')
pause(0.1)
sum_j = complex(zeros(Nf,Nf)); % preallocate
p = 3; % p coefficient
[R,Res] = r_res(Ne,p); % R and Res coefficients
j = 1;
done = 0;
guess_made = 0;
while j <= Ne && ~done
    [k_jn,k_lm] = meshgrid(k(j,:),k(j,:)); % wavenumbers
    if j==1 % progressive term, top of equation 53
        cos_alpha_jn = cos(alpha_n);
        cos_alpha_lm = cos(alpha_m);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_jn = sqrt(1 + k_yn.^2./abs(k_jn).^2);
        cos_alpha_lm = sqrt(1 + k_ym.^2./abs(k_lm).^2);
    end
    k_xjn = k_jn.*cos_alpha_jn; % equation 51
    k_xlm = k_lm.*cos_alpha_lm; % equation 51
    [e_jn,e_lm] = meshgrid(ej(j,:),ej(j,:)); % coefficients

    % asymptotic estimate argument for R(J), Res(J)
    J = j-1;

    % F_nm_plus first two terms of equation 92
    % note that M2=0 for a piston
    d_J = -g./(2*omega_n).*e_jn.*(k_xjn.^2 - k_yn.*k_ym)./(k_jn.^2 - K0_plus.^2).*(omega_n.^2 - (omega_n+omega_m).^2) ...
        - g./(2*omega_m).*e_lm.*(k_xlm.^2 - k_yn.*k_ym)./(k_lm.^2 - K0_plus.^2).*(omega_m.^2 - (omega_m+omega_n).^2);
    d_J = E_plus .* d_J;
    d_J(~isfinite(d_J)) = 0; % handle any singularities
    d_J = triu(d_J);  % symmetric; take only the upper triangular part
    
    if J < 2
        % equation 47 valid for J >= 2
        sum_j = sum_j + d_J;
        j = j + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_j + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_j = sum_j + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            j = j + 1; % keep going
        elseif j == Ne
            % last round, can't go on
            sum_j = sum_guess; % accept the guess
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J); % symmetric; take only the upper triangular part
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_j = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_j = sum_j + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                j = j + 1; % keep going
            end
        end
    end
end
F_nm_plus = sum_j;

% double-summation terms
disp('FG_plus: calculating F_nm_plus double-summation terms')
pause(0.1)
sum_j = complex(zeros(Nf,Nf)); % preallocate
j = 1;
done = 0;
guess_made = 0;
while j <= Ne && ~done
    % p coefficient
    if j==1
        p = 3;
    else
        p = 2;
    end
    
    % R and Res coefficients
    [R,Res] = r_res(Ne,p);

    % asymptotic estimate argument for R(J), Res(J)
    J = j-1;

    % asymptotic estimation of the sum from l=j to Ne of third term
    % of equation 92 for F_nm_minus
    d_J = asym_sum_l_plus(R,Res,j,k,K0_plus,ej,alpha,omega,tol);
    
    if J < 2
        % equation 47 valid for J >= 2
        sum_j = sum_j + d_J;
        j = j + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_j + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_j = sum_j + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            j = j + 1; % keep going
        elseif j == Ne
            % last round, can't go on
            sum_j = sum_guess; % accept the guess
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J); % symmetric; take only the upper triangular part
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_j = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_j = sum_j + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                j = j + 1; % keep going
            end
        end
    end
end
sum_l = complex(zeros(Nf,Nf)); % preallocate
l = 1;
done = 0;
guess_made = 0;
while l <= Ne-1 && ~done
    % p coefficient
    if l==1
        p = 3;
    else
        p = 2;
    end
    
    % R and Res coefficients
    [R,Res] = r_res(Ne,p);
    
    % asymptotic estimate argument for R(J), Res(J)
    J = l-1; 

    % asymptotic estimation of the sum from j=l+1 to Ne of third term
    % of equation 92 for F_nm_plus
    d_J = asym_sum_j_plus(R,Res,l,k,K0_plus,ej,alpha,omega,tol);
    
    if J < 2
        % equation 47 valid for J >= 2
        sum_l = sum_l + d_J;
        l = l + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_l + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_l = sum_l + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            l = l + 1; % keep going
        elseif l == Ne-1
            % last round, can't go on
            sum_l = sum_guess; % accept the asymptotic estimate
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J); % symmetric; take only the upper triangular part
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_l = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_l = sum_l + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                l = l + 1; % keep going
            end
        end
    end
end
F_nm_plus = F_nm_plus + E_plus.*(sum_j + sum_l);
F_nm_plus = triu(F_nm_plus); % symmetric; take only the upper triangular part

% done
disp('FG_plus: done')
end

function sum_l = asym_sum_l_plus(R,Res,j,k,K0_plus,ej,alpha,omega,tol)
% asymptotic estimation of the sum from l=j to Ne of third term
% of equation 92 for F_nm_plus
% setup
g = gravity; % gravity
[Ne,Nf] = size(k); % number of evanescent modes, number of frequencies
[omega_n,omega_m] = meshgrid(omega);
[alpha_n,alpha_m] = meshgrid(alpha); % angles
[k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
k_yn = k_0n.*sin(alpha_n); % equation 50, note k_yn has no imaginary (evanescent) components
k_ym = k_0m.*sin(alpha_m); % equation 50, note k_ym has no imaginary (evanescent) components

% sum l=j to infinity
l = j;
sum_l = complex(zeros(Nf,Nf));
guess_made = 0;
done = 0;
while l <= Ne && ~done
    [k_jn,k_lm] = meshgrid(k(j,:),k(l,:)); % wavenumbers
    [e_jn,e_lm] = meshgrid(ej(j,:),ej(l,:)); % coefficients
    if j==1 % progressive term, top of equation 53
        cos_alpha_jn = cos(alpha_n);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_jn = sqrt(1 + k_yn.^2./abs(k_jn).^2);
    end
    if l==1 % progressive term, top of equation 53
        cos_alpha_lm = cos(alpha_m);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_lm = sqrt(1 + k_ym.^2./abs(k_lm).^2);
    end
    k_xjn = k_jn.*cos_alpha_jn; % equation 51
    k_xlm = k_lm.*cos_alpha_lm; % equation 51
    k_jn_dot_k_lm = k_xjn.*k_xlm + k_yn.*k_ym; % dot product in H_nm_plus
    k_jnlm_plus = sqrt((k_xjn + k_xlm).^2 + (k_yn + k_ym).^2); % equation 62
    
    % H_jnlm_plus (equation 58)
    H_jnlm_plus = (omega_n + omega_m).*(omega_n.*omega_m - g^2.*k_jn_dot_k_lm./(omega_n.*omega_m)) ...
        + 0.5*(omega_n.^3+omega_m.^3) - 0.5*g^2*(k_jn.^2./omega_n+k_lm.^2./omega_m);
    H_jnlm_plus(~isfinite(H_jnlm_plus)) = 0; % handle any singularities
    H_jnlm_plus = triu(H_jnlm_plus);  % symmetric; take only the upper triangular part
    
    % F_nm_plus third term of equation 92
    d_J = e_jn.*e_lm.*(k_xjn+k_xlm)./(k_jnlm_plus.^2-K0_plus.^2).*H_jnlm_plus;
    d_J(~isfinite(d_J)) = 0; % handle any singularities
    d_J = triu(d_J);  % symmetric; take only the upper triangular part
    
    J = l-1; % asymptotic estimate argument for R(J), Res(J)
    if J < 2
        % equation 47 valid for J >= 2
        sum_l = sum_l + d_J;
        l = l + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_l + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_l = sum_l + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            l = l + 1; % keep going
        elseif l == Ne
            % last round, can't go on
            sum_l = sum_guess; % accept the asymptotic estimate
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J); % symmetric; take only the upper triangular part
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_l = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_l = sum_l + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                l = l + 1; % keep going
            end
        end
    end
end
end

function sum_j = asym_sum_j_plus(R,Res,l,k,K0_plus,ej,alpha,omega,tol)
% asymptotic estimation of the sum from j=l+1 to Ne of third term
% of equation 92 for F_nm_plus
% setup
g = gravity; % gravity
[Ne,Nf] = size(k); % number of evanescent modes, number of frequencies
[omega_n,omega_m] = meshgrid(omega);
[alpha_n,alpha_m] = meshgrid(alpha); % angles
[k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
k_yn = k_0n.*sin(alpha_n); % equation 50, note k_yn has no imaginary (evanescent) components
k_ym = k_0m.*sin(alpha_m); % equation 50, note k_ym has no imaginary (evanescent) components

% sum j=l+1 to infinity
j = l+1;
sum_j = complex(zeros(Nf,Nf));
guess_made = 0;
done = 0;
while j <= Ne && ~done
    [k_jn,k_lm] = meshgrid(k(j,:),k(l,:)); % wavenumbers
    [e_jn,e_lm] = meshgrid(ej(j,:),ej(l,:)); % coefficients
    if j==1 % progressive term, top of equation 53
        cos_alpha_jn = cos(alpha_n);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_jn = sqrt(1 + k_yn.^2./abs(k_jn).^2);
    end
    if l==1 % progressive term, top of equation 53
        cos_alpha_lm = cos(alpha_m);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_lm = sqrt(1 + k_ym.^2./abs(k_lm).^2);
    end
    k_xjn = k_jn.*cos_alpha_jn; % equation 51
    k_xlm = k_lm.*cos_alpha_lm; % equation 51
    k_jn_dot_k_lm = k_xjn.*k_xlm + k_yn.*k_ym; % dot product in H_nm_plus
    k_jnlm_plus = sqrt((k_xjn + k_xlm).^2 + (k_yn + k_ym).^2); % equation 62
    
    % H_jnlm_plus (equation 58)
    H_jnlm_plus = (omega_n + omega_m).*(omega_n.*omega_m - g^2.*k_jn_dot_k_lm./(omega_n.*omega_m)) ...
        + 0.5*(omega_n.^3+omega_m.^3) - 0.5*g^2*(k_jn.^2./omega_n+k_lm.^2./omega_m);
    H_jnlm_plus(~isfinite(H_jnlm_plus)) = 0; % handle any singularities
    H_jnlm_plus = triu(H_jnlm_plus);  % symmetric; take only the upper triangular part
    
    % F_nm_plus third term of equation 92
    d_J = e_jn.*e_lm.*(k_xjn+k_xlm)./(k_jnlm_plus.^2-K0_plus.^2).*H_jnlm_plus;
    d_J(~isfinite(d_J)) = 0; % handle any singularities
    d_J = triu(d_J); % symmetric; take only the upper triangular part
    
    J = j-1; % asymptotic estimate argument for R(J), Res(J)
    if J < 2
        % equation 47 valid for J >= 2
        sum_j = sum_j + d_J;
        j = j + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_j + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_j = sum_j + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            j = j + 1; % keep going
        elseif j == Ne
            % last round, can't go on
            sum_j = sum_guess; % accept the guess
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J); % symmetric; take only the upper triangular part
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_j = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_j = sum_j + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                j = j + 1; % keep going
            end
        end
    end
end
end

function [F_nm_minus,G_nm_minus] = FG_minus(omega,k,alpha,K0_minus,h,tol)
% Calculates transfer functions for negative (differences of different frequencies)
% 2nd order nonlinear wave-wave interactions.  F_nm_minus is the board
% stroke transfer function and G_nm_minus is the free surface transfer
% function. Uses asymptotic estimation.
%
% Takes inputs omega (vector), k (matrix, from dispersion_complex), alpha
% (vector), K0_minus (matrix, from dispersion_sumdif), and h (scalar).

% setup
disp('FG_minus: transfer function for negative (difference) 2nd order nonlinear wave-wave interactions')
g = gravity; % gravity
[Ne,Nf] = size(k); % number of evanescent modes, number of frequencies
if Nf < 2 % difference frequencies only possible with 2 or more frequencies
    G_nm_minus = zeros(Nf,Nf);
    F_nm_minus = zeros(Nf,Nf);
    return
end
[omega_n,omega_m] = meshgrid(omega);
K0_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

% delta_nm (equation 56)
% all equation numbers are from Schaffer & Steenberg, 2003
% unless otherwise noted
disp('FG_minus: calculating delta_nm')
delta_nm = ones(Nf,Nf);
for n=1:Nf
    delta_nm(n,n) = 0.5;
end

% Biesel coefficients (height-to-stroke ratio)
disp('FG_minus: calculating cj')
cj = 2*(cosh(2*k*h) - 1)./(sinh(2*k*h) + 2*k*h); % equation 43
cj(k*h>20) = 2; % asymptotic limit for deep water
cj(k==0) = 0; % handle any zero wavenumbers
c0 = cj(1,:); % progressive

% directional Biesel coefficients e
disp('FG_minus: calculating ej')
ej = zeros(size(cj));
ej(1,:) = c0 ./ cos(alpha); % equation 49
k0_sin_alpha = repmat(k(1,:).*sin(alpha),Ne-1,1);
cos_alphaj = sqrt(1 + k0_sin_alpha.^2 ./ abs(k(2:end,:)).^2); % equation 53
ej(2:end,:) = cj(2:end,:) ./ cos_alphaj; % equation 52

% E_minus (equation 93)
% note that M1 = 0 for a piston
disp('FG_minus: calculating E_minus')
[e0_n,e0_m] = meshgrid(ej(1,:),ej(1,:));
E_minus = delta_nm.*K0_minus.^2*h./(e0_n.*e0_m.*(omega_n-omega_m).^3);
E_minus(~isfinite(E_minus)) = 0; % handle any singularities
E_minus = triu(E_minus,1); % symmetric; take only the upper triangular part, eliminating n=m

% G_nm_minus, for which we only need progressive part
disp('FG_minus: calculating G_nm_minus')
[k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
[alpha_n,alpha_m] = meshgrid(alpha); % angles
k_yn = k_0n.*sin(alpha_n); % equation 50, note k_yn has no imaginary (evanescent) components
k_ym = k_0m.*sin(alpha_m); % equation 50, note k_ym has no imaginary (evanescent) components
k_x0n = k_0n.*cos(alpha_n); % equations 51 and 53 for j=l=0
k_x0m = k_0m.*cos(alpha_m); % equations 51 and 53 for j=l=0
k_0n_dot_k_0m = k_x0n.*k_x0m + k_yn.*k_ym; % dot product in H_nm_minus and L_nm_minus

% H_nm_minus (equation 58, where j=l=0)
H_nm_minus = (omega_n - omega_m).*(-omega_n.*omega_m - g^2*k_0n_dot_k_0m./(omega_n.*omega_m)) ...
    + 0.5*(omega_n.^3-omega_m.^3) - 0.5*g^2*(k_0n.^2./omega_n-k_0m.^2./omega_m);
H_nm_minus(~isfinite(H_nm_minus)) = 0; % handle any singularities
H_nm_minus = triu(H_nm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
H_nm_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

% D_nm_minus
k_nm_minus = sqrt((k_x0n - k_x0m).^2 + (k_yn - k_ym).^2); % (equation 62, where j=l=0)
D_nm_minus = g*k_nm_minus.*tanh(k_nm_minus*h) - (omega_n-omega_m).^2; % (equation 63, where j=l=0)
D_nm_minus(~isfinite(D_nm_minus)) = 0; % handle any singularities
D_nm_minus = triu(D_nm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
D_nm_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

% L_nm_minus (equation 65, where j=l=0)
% note that there is an error in the published version, corrected in this code
L_nm_minus = 0.5*(g^2*k_0n_dot_k_0m./(omega_n.*omega_m) + omega_n.*omega_m - (omega_n.^2 + omega_m.^2)); 
L_nm_minus(~isfinite(L_nm_minus)) = 0; % handle any singularities
L_nm_minus = triu(L_nm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
L_nm_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

% G_nm_minus (equation 67, where j=l=0)
% which yields eta_0_(21minus)
% desired progressive second order bound waves
G_nm_minus = delta_nm/g.*((omega_n-omega_m).*H_nm_minus./D_nm_minus - L_nm_minus);
G_nm_minus(~isfinite(G_nm_minus)) = 0; % handle any singularities
G_nm_minus = triu(G_nm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
G_nm_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

% single-summation terms
disp('FG_minus: calculating F_nm_minus single-summation terms')
pause(0.1)
sum_j = complex(zeros(Nf,Nf)); % preallocate
p = 3; % p coefficient
[R,Res] = r_res(Ne,p); % R and Res coefficients
j = 1;
done = 0;
guess_made = 0;
while j <= Ne && ~done
    [k_jn,k_lm] = meshgrid(k(j,:),k(j,:)); % wavenumbers
    if j==1 % progressive term, top of equation 53
        cos_alpha_jn = cos(alpha_n);
        cos_alpha_lm = cos(alpha_m);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_jn = sqrt(1 + k_yn.^2./abs(k_jn).^2);
        cos_alpha_lm = sqrt(1 + k_ym.^2./abs(k_lm).^2);
    end
    k_xjn = k_jn.*cos_alpha_jn; % equation 51
    k_xlm = k_lm.*cos_alpha_lm; % equation 51
    [e_jn,e_lm] = meshgrid(ej(j,:),ej(j,:)); % coefficients

    % asymptotic estimate argument for R(J), Res(J)
    J = j-1;
    
    % F_nm_minus first two terms of equation 92
    % note that M2=0 for a piston
    % conj(e_lm), conj(k_xlm), conj(k_lm) due to -:* notation
    d_J = g./(2*omega_n).*e_jn.*(k_xjn.^2 + k_yn.*k_ym)./(k_jn.^2 - K0_minus.^2).*(omega_n.^2 - (omega_n-omega_m).^2) ...
        + g./(2*omega_m).*conj(e_lm).*(conj(k_xlm).^2 + k_yn.*k_ym)./(conj(k_lm).^2 - K0_minus.^2).*(omega_m.^2 - (omega_m-omega_n).^2);
    d_J = E_minus .* d_J;
    d_J(~isfinite(d_J)) = 0; % handle any singularities
    d_J = triu(d_J,1); % symmetric; take only the upper triangular part, eliminating n=m
    d_J(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

    if J < 2
        % equation 47 valid for J >= 2
        sum_j = sum_j + d_J;
        j = j + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_j + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_j = sum_j + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            j = j + 1; % keep going
        elseif j == Ne
            % last round, can't go on
            sum_j = sum_guess; % accept the guess
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J,1); % symmetric; take only the upper triangular part, eliminating n=m
            epsilon_J(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_j = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_j = sum_j + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                j = j + 1; % keep going
            end
        end
    end
end
F_nm_minus = sum_j;

% double-summation terms
disp('FG_minus: calculating F_nm_minus double-summation terms')
pause(0.1)
sum_j = complex(zeros(Nf,Nf)); % preallocate
j = 1;
done = 0;
guess_made = 0;
while j <= Ne && ~done
    % p coefficient
    if j==1
        p = 3;
    else
        p = 2;
    end
    
    % R and Res coefficients
    [R,Res] = r_res(Ne,p);

    % asymptotic estimate argument for R(J), Res(J)
    J = j-1;

    % asymptotic estimation of the sum from l=j to Ne of third term
    % of equation 92 for F_nm_minus
    d_J = asym_sum_l_minus(R,Res,j,k,K0_minus,ej,alpha,omega,tol);

    if J < 2
        % equation 47 valid for J >= 2
        sum_j = sum_j + d_J;
        j = j + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_j + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_j = sum_j + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            j = j + 1; % keep going
        elseif j == Ne
            % last round, can't go on
            sum_j = sum_guess; % accept the guess
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J,1); % symmetric; take only the upper triangular part, eliminating n=m
            epsilon_J(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_j = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_j = sum_j + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                j = j + 1; % keep going
            end
        end
    end
end
sum_l = complex(zeros(Nf,Nf)); % preallocate
l = 1;
done = 0;
guess_made = 0;
while l <= Ne-1 && ~done
    % p coefficient
    if l==1
        p = 3;
    else
        p = 2;
    end
    
    % R and Res coefficients
    [R,Res] = r_res(Ne,p);
    
    % asymptotic estimate argument for R(J), Res(J)
    J = l-1; 

    % asymptotic estimation of the sum from j=l+1 to Ne of third term
    % of equation 92 for F_nm_minus
    d_J = asym_sum_j_minus(R,Res,l,k,K0_minus,ej,alpha,omega,tol);
    
    if J < 2
        % equation 47 valid for J >= 2
        sum_l = sum_l + d_J;
        l = l + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_l + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_l = sum_l + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            l = l + 1; % keep going
        elseif l == Ne-1
            % last round, can't go on
            sum_l = sum_guess; % accept the asymptotic estimate
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J,1); % symmetric; take only the upper triangular part, eliminating n=m
            epsilon_J(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_l = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_l = sum_l + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                l = l + 1; % keep going
            end
        end
    end
end
F_nm_minus = F_nm_minus + E_minus.*(sum_j + sum_l);
F_nm_minus = triu(F_nm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
F_nm_minus(K0_minus*h < 0.025) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

% done
disp('FG_minus: done')
end

function sum_l = asym_sum_l_minus(R,Res,j,k,K0_minus,ej,alpha,omega,tol)
% asymptotic estimation of the sum from l=j to Ne of third term
% of equation 92 for F_nm_minus
% setup
g = gravity; % gravity
[Ne,Nf] = size(k); % number of evanescent modes, number of frequencies
[omega_n,omega_m] = meshgrid(omega);
[alpha_n,alpha_m] = meshgrid(alpha); % angles
[k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
k_yn = k_0n.*sin(alpha_n); % equation 50, note k_yn has no imaginary (evanescent) components
k_ym = k_0m.*sin(alpha_m); % equation 50, note k_ym has no imaginary (evanescent) components

% sum from l=j to Ne
l = j;
sum_l = complex(zeros(Nf,Nf));
guess_made = 0;
done = 0;
while l <= Ne && ~done
    [k_jn,k_lm] = meshgrid(k(j,:),k(l,:)); % wavenumbers
    [e_jn,e_lm] = meshgrid(ej(j,:),ej(l,:)); % coefficients
    if j==1 % progressive term, top of equation 53
        cos_alpha_jn = cos(alpha_n);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_jn = sqrt(1 + k_yn.^2./abs(k_jn).^2);
    end
    if l==1 % progressive term, top of equation 53
        cos_alpha_lm = cos(alpha_m);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_lm = sqrt(1 + k_ym.^2./abs(k_lm).^2);
    end
    k_xjn = k_jn.*cos_alpha_jn; % equation 51
    k_xlm = k_lm.*cos_alpha_lm; % equation 51
    k_jn_dot_k_lm = k_xjn.*conj(k_xlm) + k_yn.*k_ym; % dot product in H_nm_plus, conj(k_xlm) due to -:* notation
    k_jnlm_minus = sqrt((k_xjn - conj(k_xlm)).^2 + (k_yn - k_ym).^2); % equation 62, conj(k_xlm) due to -:* notation
    
    % H_jnlm_minus (equation 58)
    H_jnlm_minus = (omega_n - omega_m).*(-omega_n.*omega_m - g^2.*k_jn_dot_k_lm./(omega_n.*omega_m)) ...
        + 0.5*(omega_n.^3-omega_m.^3) - 0.5*g^2*(k_jn.^2./omega_n-k_lm.^2./omega_m);
    H_jnlm_minus(~isfinite(H_jnlm_minus)) = 0; % handle any singularities
    H_jnlm_minus = triu(H_jnlm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
    H_jnlm_minus(K0_minus == 0) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
    
    % F_nm_minus third term of equation 92
    % conj(e_lm) and conj(k_xlm) due to -:* notation
    d_J = e_jn.*conj(e_lm).*(k_xjn-conj(k_xlm))./(k_jnlm_minus.^2-K0_minus.^2).*H_jnlm_minus;
    d_J(~isfinite(d_J)) = 0; % handle any singularities
    d_J = triu(d_J,1); % symmetric; take only the upper triangular part, eliminating n=m
    d_J(K0_minus == 0) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
    
    J = l-1; % asymptotic estimate argument for R(J), Res(J)
    if J < 2
        % equation 47 valid for J >= 2
        sum_l = sum_l + d_J;
        l = l + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_l + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_l = sum_l + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            l = l + 1; % keep going
        elseif l == Ne
            % last round, can't go on
            sum_l = sum_guess; % accept the asymptotic estimate
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J,1); % symmetric; take only the upper triangular part, eliminating n=m
            epsilon_J(K0_minus == 0) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits

            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_l = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_l = sum_l + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                l = l + 1; % keep going
            end
        end
    end
end
end

function sum_j = asym_sum_j_minus(R,Res,l,k,K0_minus,ej,alpha,omega,tol)
% asymptotic estimation of the sum from j=l+1 to Ne of third term
% of equation 92 for F_nm_minus
% setup
g = gravity; % gravity
[Ne,Nf] = size(k); % number of evanescent modes, number of frequencies
[omega_n,omega_m] = meshgrid(omega);
[alpha_n,alpha_m] = meshgrid(alpha); % angles
[k_0n,k_0m] = meshgrid(k(1,:)); % progressive wavenumbers
k_yn = k_0n.*sin(alpha_n); % equation 50, note k_yn has no imaginary (evanescent) components
k_ym = k_0m.*sin(alpha_m); % equation 50, note k_ym has no imaginary (evanescent) components

% sum from j=l+1 to Ne
j = l+1;
sum_j = complex(zeros(Nf,Nf));
guess_made = 0;
done = 0;
while j <= Ne && ~done
    [k_jn,k_lm] = meshgrid(k(j,:),k(l,:)); % wavenumbers
    [e_jn,e_lm] = meshgrid(ej(j,:),ej(l,:)); % coefficients
    if j==1 % progressive term, top of equation 53
        cos_alpha_jn = cos(alpha_n);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_jn = sqrt(1 + k_yn.^2./abs(k_jn).^2);
    end
    if l==1 % progressive term, top of equation 53
        cos_alpha_lm = cos(alpha_m);
    else % evanescent term, equations 50 and bottom of 53
        cos_alpha_lm = sqrt(1 + k_ym.^2./abs(k_lm).^2);
    end
    k_xjn = k_jn.*cos_alpha_jn; % equation 51
    k_xlm = k_lm.*cos_alpha_lm; % equation 51
    k_jn_dot_k_lm = k_xjn.*conj(k_xlm) + k_yn.*k_ym; % dot product in H_nm_plus, conj(k_xlm) due to -:* notation
    k_jnlm_minus = sqrt((k_xjn - conj(k_xlm)).^2 + (k_yn - k_ym).^2); % equation 62, conj(k_xlm) due to -:* notation
    
    % H_jnlm_minus (equation 58)
    H_jnlm_minus = (omega_n - omega_m).*(-omega_n.*omega_m - g^2.*k_jn_dot_k_lm./(omega_n.*omega_m)) ...
        + 0.5*(omega_n.^3-omega_m.^3) - 0.5*g^2*(k_jn.^2./omega_n-k_lm.^2./omega_m);
    H_jnlm_minus(~isfinite(H_jnlm_minus)) = 0; % handle any singularities
    H_jnlm_minus = triu(H_jnlm_minus,1); % symmetric; take only the upper triangular part, eliminating n=m
    H_jnlm_minus(K0_minus == 0) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
    
    % F_nm_minus third term of equation 92
    % conj(e_lm) and conj(k_xlm) due to -:* notation
    d_J = e_jn.*conj(e_lm).*(k_xjn-conj(k_xlm))./(k_jnlm_minus.^2-K0_minus.^2).*H_jnlm_minus;
    d_J(~isfinite(d_J)) = 0; % handle any singularities
    d_J = triu(d_J,1); % symmetric; take only the upper triangular part, eliminating n=m
    d_J(K0_minus == 0) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
    
    J = j-1; % asymptotic estimate argument for R(J), Res(J)
    if J < 2
        % equation 47 valid for J >= 2
        sum_j = sum_j + d_J;
        j = j + 1;
    else % J >=2
        % equation 47 of Schaffer (1993)
        sum_guess = sum_j + d_J*Res(J); % asymptotic estimate of the infinite sum
        
        if ~guess_made
            % first round, no previous asymptotic estimate and no testing
            guess_made = 1; % we have now made a guess
            sum_j = sum_j + d_J; % reject the asymptotic estimate
            d_Jm1 = d_J; % previous term
            sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
            j = j + 1; % keep going
        elseif j == Ne
            % last round, can't go on
            sum_j = sum_guess; % accept the guess
            done = 1; % stop
        else
            % error associated with the asymptotic estimate
            epsilon_J = abs(d_J./d_Jm1*R(J)-1).*abs(d_J)*Res(J); % equation 48 of Schaffer (1993)
            epsilon_J(~isfinite(epsilon_J)) = 0; % handle any singularities
            epsilon_J = triu(epsilon_J,1); % symmetric; take only the upper triangular part, eliminating n=m
            epsilon_J(K0_minus == 0) = 0; % per AwaSys manual p.167 to prevent board drift and stroke limits
            
            % test the asymptotic estimate
            if estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
                sum_j = sum_guess; % accept the guess
                done = 1; % stop
            else
                sum_j = sum_j + d_J; % reject the guess
                d_Jm1 = d_J; % previous term
                sum_guess_m1 = sum_guess; % previous asymptotic estimate of the infinite sum
                j = j + 1; % keep going
            end
        end
    end
end
end

function done = estimate_test(sum_guess,sum_guess_m1,epsilon_J,tol)
% test of asymptotic estimation
error_limits = abs(sum_guess(:))*tol; % acceptable error limits
done = all(epsilon_J(:) < error_limits); % estimated errors below limits
delta_sum = abs(sum_guess(:) - sum_guess_m1(:)); % changes in estimates
done = done && all(delta_sum < error_limits); % also below limits
end

function [R,Res] = r_res(jmax,p)
% Calculates the "R" and "Res" variables for asymptotic summation, using 
% the provided values of "jmax" and "p". Calculated for values of J=1:jmax.
% "p" must be an integer greater than or equal to 2 (and should be 2 or 3).
R = zeros(jmax,1);
Res = zeros(jmax,1);
Res(1) = zeta(p); % equation 46
for J=2:jmax
    R(J) = (J/(J-1))^p; % equation 43
    Res(J) = R(J)*(Res(J-1) - 1); % equation 45
end
end